@@ -482,8 +482,11 @@ void KateMainWindow::documentMenuAboutToShow()
 
   QListViewItem * item = filelist->firstChild();
   while( item ) {
+    // would it be saner to use the screen width as a limit that some random number??
+    QString name = KStringHandler::rsqueeze( ((KateFileListItem *)item)->document()->docName(), 150 ); 
+    Kate::Document* doc = ((KateFileListItem *)item)->document();
     documentMenu->insertItem (
-          KStringHandler::rsqueeze( ((KateFileListItem *)item)->document()->docName(), 150 ), // would it be saner to use the screen width as a limit that some random number??
+          doc->isModified() ? i18n("'document name [*]', [*] means modified", "%1 [*]").arg(name) : name,
           m_viewManager, SLOT (activateView (int)), 0,
           ((KateFileListItem *)item)->documentNumber () );
 
@@ -612,18 +615,18 @@ void KateMainWindow::slotOpenWithMenuAction(int idx)
 {
   KURL::List list;
   list.append( m_viewManager->activeView()->getDoc()->url() );
-  QString* appname = new QString( documentOpenWith->popupMenu()->text(idx) );
+  QString appname = documentOpenWith->popupMenu()->text(idx);
 
-  *appname = appname->remove('&'); //Remove a possible accelerator ... otherwise the application might not get found.
-  if ( appname->compare(i18n("Other...")) == 0 ) {
+  appname = appname.remove('&'); //Remove a possible accelerator ... otherwise the application might not get found.
+  if ( appname.compare(i18n("Other...")) == 0 ) {
     // display "open with" dialog
-    KOpenWithDlg* dlg = new KOpenWithDlg(list);
-    if (dlg->exec())
-      KRun::run(*dlg->service(), list);
+    KOpenWithDlg dlg(list);
+    if (dlg.exec())
+      KRun::run(*dlg.service(), list);
     return;
   }
 
-  QString qry = QString("((Type == 'Application') and (Name == '%1'))").arg( appname->latin1() );
+  QString qry = QString("((Type == 'Application') and (Name == '%1'))").arg( appname.latin1() );
   KMimeType::Ptr mime = KMimeType::findByURL( m_viewManager->activeView()->getDoc()->url() );
   KTrader::OfferList offers = KTrader::self()->query(mime->name(), qry);
 
@@ -632,9 +635,7 @@ void KateMainWindow::slotOpenWithMenuAction(int idx)
     KRun::run(*app, list);
   }
   else
-    KMessageBox::error(this, i18n("Application '%1' not found!").arg(appname->latin1()), i18n("Application Not Found!"));
-
-
+    KMessageBox::error(this, i18n("Application '%1' not found!").arg(appname.latin1()), i18n("Application Not Found!"));
 }
 
 void KateMainWindow::pluginHelp()
@@ -646,7 +647,10 @@ void KateMainWindow::slotMail()
 {
   KateMailDialog *d = new KateMailDialog(this, this);
   if ( ! d->exec() )
+  {
+    delete d;
     return;
+  }
   QPtrList<Kate::Document> attDocs = d->selectedDocs();
   delete d;
   // Check that all selected files are saved (or shouldn't be)
