@@ -1,6 +1,5 @@
 /* vi: ts=8 sts=4 sw=4
- *
- * $Id: bgsettings.cpp,v 1.12 2003/12/29 19:20:59 mueller Exp $
+ * kate: space-indent on; indent-width 4; indent-mode cstyle;
  *
  * This file is part of the KDE project, module kdesktop.
  * Copyright (C) 1999 Geert Jansen <g.t.jansen@stud.tue.nl>
@@ -23,32 +22,18 @@
 
 #include <dcopclient.h>
 #include <kapplication.h>
+#include <kdebug.h>
 #include <kglobalsettings.h>
 #include <ksimpleconfig.h>
 #include <kstandarddirs.h>
+#include <krandomsequence.h>
+#include <konq_defaults.h>
 
 #include "bgdefaults.h"
+#include "bghash.h"
 #include "bgsettings.h"
 
 
-/*
- * QString -> int hash. From Qt's QGDict::hashKeyString().
- */
-
-static int QHash(QString key)
-{
-    int g, h = 0;
-    const QChar *p = key.unicode();
-    for (unsigned i=0; i < key.length(); i++) {
-        h = (h << 4) + p[i].cell();
-        if ((g = (h & 0xf0000000)))
-            h ^= (g >> 24);
-        h &= ~g;
-    }
-    return h;
-}
-
-
 /**** KBackgroundPattern ****/
 
 
@@ -203,7 +188,7 @@ QStringList KBackgroundPattern::list()
     QStringList lst = dirs->findAllResources("dtop_pattern", "*.desktop",
                                              false, true);
     QStringList::Iterator it;
-    for (it=lst.begin(); it!=lst.end(); it++) {
+    for (it=lst.begin(); it!=lst.end(); ++it) {
         // Strip path and suffix
         int pos = (*it).findRev('/');
         if (pos != -1)
@@ -414,7 +399,7 @@ QStringList KBackgroundProgram::list()
     QStringList lst = dirs->findAllResources("dtop_program", "*.desktop",
                                              false, true);
     QStringList::Iterator it;
-    for (it=lst.begin(); it!=lst.end(); it++) {
+    for (it=lst.begin(); it!=lst.end(); ++it) {
         // Strip path and suffix
         int pos = (*it).findRev('/');
         if (pos != -1)
@@ -430,12 +415,14 @@ QStringList KBackgroundProgram::list()
 /**** KBackgroundSettings ****/
 
 
-KBackgroundSettings::KBackgroundSettings(int desk, KConfig *config)
+KBackgroundSettings::KBackgroundSettings(int desk, int screen, bool drawBackgroundPerScreen, KConfig *config)
     : KBackgroundPattern(),
       KBackgroundProgram()
 {
     dirty = false; hashdirty = true;
+	m_bDrawBackgroundPerScreen = drawBackgroundPerScreen;
     m_Desk = desk;
+    m_Screen = screen;
     m_bEnabled = true;
 
     // Default values.
@@ -469,6 +456,7 @@ KBackgroundSettings::KBackgroundSettings(int desk, KConfig *config)
     // Blend modes
     #define ADD_STRING(ID) m_BlMMap[#ID] = ID; m_BlMRevMap[ID] = (char *) #ID;
     ADD_STRING(NoBlending)
+    ADD_STRING(FlatBlending)
     ADD_STRING(HorizontalBlending)
     ADD_STRING(VerticalBlending)
     ADD_STRING(PyramidBlending)
@@ -490,6 +478,7 @@ KBackgroundSettings::KBackgroundSettings(int desk, KConfig *config)
     ADD_STRING(TiledMaxpect)
     ADD_STRING(Scaled)
     ADD_STRING(CentredAutoFit)
+    ADD_STRING(ScaleAndCrop)
     #undef ADD_STRING
 
     // Multiple wallpaper modes
@@ -553,15 +542,18 @@ void KBackgroundSettings::copyConfig(const KBackgroundSettings *settings)
     m_MultiMode = settings->m_MultiMode;
     m_Interval = settings->m_Interval;
     m_CurrentWallpaper = settings->m_CurrentWallpaper;
+    m_CurrentWallpaperName = settings->m_CurrentWallpaperName;
 
     KBackgroundPattern::copyConfig(settings);
     KBackgroundProgram::copyConfig(settings);
 }
 
 
-void KBackgroundSettings::load(int desk, bool reparseConfig)
+void KBackgroundSettings::load(int desk, int screen, bool drawBackgroundPerScreen, bool reparseConfig)
 {
     m_Desk = desk;
+    m_Screen = screen;	
+    m_bDrawBackgroundPerScreen = drawBackgroundPerScreen;
     readSettings(reparseConfig);
 }
 
@@ -664,15 +656,19 @@ void KBackgroundSettings::setWallpaperList(QStringList list)
     KStandardDirs *d = KGlobal::dirs();
     if (m_WallpaperList == list)
 	return;
+
     dirty = hashdirty = true;
     m_WallpaperList.clear();
     for(QStringList::ConstIterator it = list.begin();
         it != list.end(); ++it)
     {
-       m_WallpaperList.append(d->relativeLocation("wallpaper", *it));
+       QString rpath = d->relativeLocation("wallpaper", *it);
+       m_WallpaperList.append( !rpath.isEmpty() ? rpath : *it );
     }
     updateWallpaperFiles();
-    changeWallpaper(true);
+    // Try to keep the current wallpaper (-1 to set position to one before it)
+    m_CurrentWallpaper = m_WallpaperFiles.findIndex(m_CurrentWallpaperName) - 1;
+    changeWallpaper(m_CurrentWallpaper < 0);
 }
 
 
@@ -727,12 +723,20 @@ void KBackgroundSettings::setUseShm(bool use)
     m_bShm = use;
 }
 
+QString KBackgroundSettings::configGroupName() const
+{
+    QString screenName;
+    if (m_bDrawBackgroundPerScreen)
+        screenName = QString("Screen%1").arg(QString::number(m_Screen));
+    return QString("Desktop%1%2").arg(m_Desk).arg(screenName);
+}
+
 void KBackgroundSettings::readSettings(bool reparse)
 {
     if (reparse)
         m_pConfig->reparseConfiguration();
 
-    m_pConfig->setGroup(QString("Desktop%1").arg(m_Desk));
+    m_pConfig->setGroup(configGroupName());
 
     // Background mode (Flat, div. Gradients, Pattern or Program)
     m_ColorA = m_pConfig->readColorEntry("Color1", &defColorA);
@@ -774,11 +778,11 @@ void KBackgroundSettings::readSettings(bool reparse)
 
     // Multiple wallpaper config
     m_WallpaperList = m_pConfig->readPathListEntry("WallpaperList");
-    updateWallpaperFiles();
 
     m_Interval = m_pConfig->readNumEntry("ChangeInterval", 60);
     m_LastChange = m_pConfig->readNumEntry("LastChange", 0);
     m_CurrentWallpaper = m_pConfig->readNumEntry("CurrentWallpaper", 0);
+    m_CurrentWallpaperName = m_pConfig->readEntry("CurrentWallpaperName");
 
     m_MultiMode = defMultiMode;
     s = m_pConfig->readEntry("MultiWallpaperMode");
@@ -787,6 +791,12 @@ void KBackgroundSettings::readSettings(bool reparse)
 	m_MultiMode = mode;
     }
 
+    updateWallpaperFiles();
+    if( !m_CurrentWallpaperName.isEmpty())
+        m_CurrentWallpaper = m_WallpaperFiles.findIndex(m_CurrentWallpaperName);
+    if(m_CurrentWallpaper < 0)
+        m_CurrentWallpaper = 0;
+
     // Wallpaper mode (NoWallpaper, div. tilings)
     m_WallpaperMode = defWallpaperMode;
     m_Wallpaper = m_pConfig->readPathEntry("Wallpaper");
@@ -814,7 +824,7 @@ void KBackgroundSettings::writeSettings()
     if (!dirty)
         return;
 
-    m_pConfig->setGroup(QString("Desktop%1").arg(m_Desk));
+    m_pConfig->setGroup(configGroupName());
     m_pConfig->writeEntry("Color1", m_ColorA);
     m_pConfig->writeEntry("Color2", m_ColorB);
     m_pConfig->writePathEntry("Pattern", KBackgroundPattern::name());
@@ -832,7 +842,8 @@ void KBackgroundSettings::writeSettings()
     m_pConfig->writePathEntry("WallpaperList", m_WallpaperList);
     m_pConfig->writeEntry("ChangeInterval", m_Interval);
     m_pConfig->writeEntry("LastChange", m_LastChange);
-    m_pConfig->writeEntry("CurrentWallpaper", m_CurrentWallpaper);
+    m_pConfig->deleteEntry("CurrentWallpaper"); // obsolete, remember name
+    m_pConfig->writeEntry("CurrentWallpaperName", m_CurrentWallpaperName);
 
     m_pConfig->sync();
 
@@ -846,7 +857,7 @@ void KBackgroundSettings::updateWallpaperFiles()
 {
     QStringList::Iterator it;
     m_WallpaperFiles.clear();
-    for (it=m_WallpaperList.begin(); it!=m_WallpaperList.end(); it++) {
+    for (it=m_WallpaperList.begin(); it!=m_WallpaperList.end(); ++it) {
         QString file = locate("wallpaper", *it);
         if (file.isEmpty())
             continue;
@@ -859,7 +870,7 @@ void KBackgroundSettings::updateWallpaperFiles()
 	    QDir dir(file);
 	    QStringList lst = dir.entryList(QDir::Files | QDir::Readable);
 	    QStringList::Iterator it;
-	    for (it=lst.begin(); it!=lst.end(); it++)
+	    for (it=lst.begin(); it!=lst.end(); ++it)
 	    {
 		file = dir.absFilePath(*it);
 		QFileInfo fi(file);
@@ -868,16 +879,63 @@ void KBackgroundSettings::updateWallpaperFiles()
 	    }
 	}
     }
+
+   if (m_MultiMode == Random)
+       randomizeWallpaperFiles();
 }
 
+// Randomize the m_WallpaperFiles in a non-repeating method.
+void KBackgroundSettings::randomizeWallpaperFiles()
+{
+   if (m_WallpaperFiles.count() < 4)
+      return;
+
+   KRandomSequence rseq;
+   QStringList tmpList = m_WallpaperFiles;
+   QStringList randomList;
+   randomList.append(tmpList.front());
+   tmpList.pop_front();
+   while(tmpList.count())
+   {
+      randomList.insert(randomList.at(
+         rseq.getLong(randomList.count()+1)),
+         1, tmpList.front());
+
+      tmpList.pop_front();
+   }
+   m_WallpaperFiles = randomList;
+}
+
+QStringList KBackgroundSettings::wallpaperList() const
+{
+    if ( m_WallpaperMode == NoWallpaper )
+        return QStringList();
+    if ( m_MultiMode == NoMulti || m_MultiMode == NoMultiRandom )
+        return QStringList(m_Wallpaper);
+    return m_WallpaperList;
+}
+
+QStringList KBackgroundSettings::wallpaperFiles() const
+{
+    if ( m_WallpaperMode == NoWallpaper )
+        return QStringList();
+    if ( m_MultiMode == NoMulti || m_MultiMode == NoMultiRandom )
+        return QStringList(m_Wallpaper);
+    return m_WallpaperFiles;
+}
 
 /*
  * Select a new wallpaper from the list.
  */
 void KBackgroundSettings::changeWallpaper(bool init)
 {
-    if (m_WallpaperFiles.count() == 0)
+    if (m_WallpaperFiles.count() == 0) {
+        if( init ) {
+	    m_CurrentWallpaper = 0;
+            m_CurrentWallpaperName = QString();
+        }
         return;
+    }
 
     switch (m_MultiMode) {
     case InOrder:
@@ -887,15 +945,23 @@ void KBackgroundSettings::changeWallpaper(bool init)
 	break;
 
     case Random:
-	m_CurrentWallpaper = kapp->random() % m_WallpaperFiles.count();
+      // Random: m_WallpaperFiles is randomized in a non-repeating
+      //  method.  Hence we just increment the index.
+      m_CurrentWallpaper++;
+      if (init || (m_CurrentWallpaper >= (int) m_WallpaperFiles.count())) {
+         m_CurrentWallpaper = 0;
+         randomizeWallpaperFiles(); // Get a new random-ordered list.
+      }
 	break;
     default:
 	break;
     }
 
+    m_CurrentWallpaperName = m_WallpaperFiles[ m_CurrentWallpaper ];
     m_LastChange = (int) time(0L);
-    m_pConfig->setGroup(QString("Desktop%1").arg(m_Desk));
-    m_pConfig->writeEntry("CurrentWallpaper", m_CurrentWallpaper);
+    m_pConfig->setGroup(configGroupName());
+    m_pConfig->deleteEntry("CurrentWallpaper"); // obsolete, remember name
+    m_pConfig->writeEntry("CurrentWallpaperName", m_CurrentWallpaperName);
     m_pConfig->writeEntry("LastChange", m_LastChange);
     m_pConfig->sync();
 
@@ -903,13 +969,16 @@ void KBackgroundSettings::changeWallpaper(bool init)
 }
 
 
-QString KBackgroundSettings::currentWallpaper()
+QString KBackgroundSettings::currentWallpaper() const
 {
+    if ( m_WallpaperMode == NoWallpaper )
+	return QString::null;
     if (m_MultiMode == NoMulti || m_MultiMode == NoMultiRandom)
 	return m_Wallpaper;
-    if (m_CurrentWallpaper < (int) m_WallpaperFiles.count())
+
+    if (m_CurrentWallpaper >= 0 && m_CurrentWallpaper < (int) m_WallpaperFiles.count())
 	return m_WallpaperFiles[m_CurrentWallpaper];
-    return QString();
+    return QString::null;
 }
 
 bool KBackgroundSettings::discardCurrentWallpaper()
@@ -919,8 +988,8 @@ bool KBackgroundSettings::discardCurrentWallpaper()
        return false;
     }
     m_WallpaperFiles.remove(m_WallpaperFiles.at(m_CurrentWallpaper));
-    if (m_CurrentWallpaper >= (int) m_WallpaperFiles.count())
-	m_CurrentWallpaper = 0;
+    --m_CurrentWallpaper;
+    changeWallpaper();
 
     return true;
 }
@@ -1041,12 +1110,42 @@ void KGlobalBackgroundSettings::setLimitCache(bool limit)
 }
 
 
-void KGlobalBackgroundSettings::setCommonBackground(bool common)
+bool KGlobalBackgroundSettings::drawBackgroundPerScreen(int desk) const
 {
-    if (common == m_bCommon)
+    if ( desk > int(m_bDrawBackgroundPerScreen.size()) )
+        return _defDrawBackgroundPerScreen;
+    return m_bDrawBackgroundPerScreen[desk];
+}
+
+
+void KGlobalBackgroundSettings::setDrawBackgroundPerScreen(int desk, bool perScreen)
+{
+    if ( desk >= int(m_bDrawBackgroundPerScreen.size()) )
+        return;
+    
+    if ( m_bDrawBackgroundPerScreen[desk] == perScreen )
+        return;
+    
+    dirty = true;
+    m_bDrawBackgroundPerScreen[desk] = perScreen;
+}
+
+
+void KGlobalBackgroundSettings::setCommonScreenBackground(bool common)
+{
+    if (common == m_bCommonScreen)
 	return;
     dirty = true;
-    m_bCommon = common;
+    m_bCommonScreen = common;
+}
+
+
+void KGlobalBackgroundSettings::setCommonDeskBackground(bool common)
+{
+    if (common == m_bCommonDesk)
+	return;
+    dirty = true;
+    m_bCommonDesk = common;
 }
 
 
@@ -1091,23 +1190,46 @@ void KGlobalBackgroundSettings::setShadowEnabled(bool enabled)
     m_shadowEnabled = enabled;
 }
 
+void KGlobalBackgroundSettings::setTextLines(int lines)
+{
+    if (lines == m_textLines)
+        return;
+    dirty = true;
+    m_textLines = lines;
+}
+
+void KGlobalBackgroundSettings::setTextWidth(int width)
+{
+    if (width == m_textWidth)
+        return;
+    dirty = true;
+    m_textWidth = width;
+}
+
 void KGlobalBackgroundSettings::readSettings()
 {
     m_pConfig->setGroup("Background Common");
-    m_bCommon = m_pConfig->readBoolEntry("CommonDesktop", _defCommon);
+    m_bCommonScreen = m_pConfig->readBoolEntry("CommonScreen", _defCommonScreen);
+    m_bCommonDesk = m_pConfig->readBoolEntry("CommonDesktop", _defCommonDesk);
     m_bDock = m_pConfig->readBoolEntry("Dock", _defDock);
     m_bExport = m_pConfig->readBoolEntry("Export", _defExport);
     m_bLimitCache = m_pConfig->readBoolEntry("LimitCache", _defLimitCache);
     m_CacheSize = m_pConfig->readNumEntry("CacheSize", _defCacheSize);
+    
+    m_Names.clear();
+    NETRootInfo info( qt_xdisplay(), NET::DesktopNames | NET::NumberOfDesktops );
+    m_bDrawBackgroundPerScreen.resize(info.numberOfDesktops());
+    for ( int i = 0 ; i < info.numberOfDesktops() ; ++i )
+        m_bDrawBackgroundPerScreen[i] = m_pConfig->readBoolEntry( QString("DrawBackgroundPerScreen_%1").arg(i), _defDrawBackgroundPerScreen );
 
     m_TextColor = KGlobalSettings::textColor();
     m_pConfig->setGroup("FMSettings");
     m_TextColor = m_pConfig->readColorEntry("NormalTextColor", &m_TextColor);
     m_TextBackgroundColor = m_pConfig->readColorEntry("ItemTextBackground");
     m_shadowEnabled = m_pConfig->readBoolEntry("ShadowEnabled", true);
+    m_textLines = m_pConfig->readNumEntry("TextHeight", DEFAULT_TEXTHEIGHT);
+    m_textWidth = m_pConfig->readNumEntry("TextWidth", DEFAULT_TEXTWIDTH);
 
-    m_Names.clear();
-    NETRootInfo info( qt_xdisplay(), NET::DesktopNames | NET::NumberOfDesktops );
     for ( int i = 0 ; i < info.numberOfDesktops() ; ++i )
       m_Names.append( QString::fromUtf8(info.desktopName(i+1)) );
 
@@ -1120,16 +1242,23 @@ void KGlobalBackgroundSettings::writeSettings()
         return;
 
     m_pConfig->setGroup("Background Common");
-    m_pConfig->writeEntry("CommonDesktop", m_bCommon);
+    m_pConfig->writeEntry("CommonScreen", m_bCommonScreen);
+    m_pConfig->writeEntry("CommonDesktop", m_bCommonDesk);
     m_pConfig->writeEntry("Dock", m_bDock);
     m_pConfig->writeEntry("Export", m_bExport);
     m_pConfig->writeEntry("LimitCache", m_bLimitCache);
     m_pConfig->writeEntry("CacheSize", m_CacheSize);
 
+    for ( unsigned i = 0 ; i < m_bDrawBackgroundPerScreen.size() ; ++i )
+        m_pConfig->writeEntry(QString("DrawBackgroundPerScreen_%1").arg(i), m_bDrawBackgroundPerScreen[i] );
+
     m_pConfig->setGroup("FMSettings");
     m_pConfig->writeEntry("NormalTextColor", m_TextColor);
     m_pConfig->writeEntry("ItemTextBackground", m_TextBackgroundColor);
     m_pConfig->writeEntry("ShadowEnabled", m_shadowEnabled);
+    m_pConfig->writeEntry("TextHeight", m_textLines);
+    m_pConfig->writeEntry("TextWidth", m_textWidth);
+    
     m_pConfig->sync();
     dirty = false;
 
