@@ -12,31 +12,33 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include <qdir.h>
-#include <qvgroupbox.h>
 #include <qcheckbox.h>
-#include <qlabel.h>
-#include <qslider.h>
-#include <qpainter.h>
 #include <qcombobox.h>
+#include <qdir.h>
+#include <qlabel.h>
 #include <qlayout.h>
+#include <qpainter.h>
+#include <qslider.h>
+#include <qvgroupbox.h>
 #include <qwhatsthis.h>
 
-#include <kfiledialog.h>
-#include <kmessagebox.h>
+#include <kcolorbutton.h>
 #include <kcursor.h>
+#include <kfiledialog.h>
+#include <kgenericfactory.h>
 #include <kglobalsettings.h>
 #include <kinputdialog.h>
-#include <kstandarddirs.h>
-#include <kipc.h>
-#include <kcolorbutton.h>
-#include <kgenericfactory.h>
-#include <kprocess.h>
-
 #include <kio/netaccess.h>
+#include <kipc.h>
+#include <kmessagebox.h>
+#include <kprocess.h>
+#include <kstandarddirs.h>
+#include <kaboutdata.h>
 
+#if defined Q_WS_X11 && !defined K_WS_QTONLY
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
+#endif
 
 #include "../krdb/krdb.h"
 
@@ -93,9 +95,21 @@ QPixmap mkColorPreview(const WidgetCanvas *cs)
 KColorScheme::KColorScheme(QWidget *parent, const char *name, const QStringList &)
     : KCModule(KolorFactory::instance(), parent, name)
 {
-    m_bChanged = false;
     nSysSchemes = 2;
 
+    setQuickHelp( i18n("<h1>Colors</h1> This module allows you to choose"
+       " the color scheme used for the KDE desktop. The different"
+       " elements of the desktop, such as title bars, menu text, etc.,"
+       " are called \"widgets\". You can choose the widget whose"
+       " color you want to change by selecting it from a list, or by"
+       " clicking on a graphical representation of the desktop.<p>"
+       " You can save color settings as complete color schemes,"
+       " which can also be modified or deleted. KDE comes with several"
+       " predefined color schemes on which you can base your own.<p>"
+       " All KDE applications will obey the selected color scheme."
+       " Non-KDE applications may also obey some or all of the color"
+       " settings, if this option is enabled."));
+
     KConfig *cfg = new KConfig("kcmdisplayrc");
     cfg->setGroup("X11");
     useRM = cfg->readBoolEntry("useResourceManager", true);
@@ -225,6 +239,13 @@ KColorScheme::KColorScheme(QWidget *parent, const char *name, const QStringList
        " box where you can choose a color for the \"widget\" selected"
        " in the above list.") );
 
+    cbShadeList = new QCheckBox(i18n("Shade sorted column in lists"), this);
+    stackLayout->addWidget(cbShadeList);
+    connect(cbShadeList, SIGNAL(toggled(bool)), this, SLOT(slotShadeSortColumnChanged(bool)));
+
+    QWhatsThis::add(cbShadeList,
+       i18n("Check this box to show the sorted column in a list with a shaded background"));
+
     group = new QGroupBox(  i18n("Con&trast"), this );
     stackLayout->addWidget(group);
 
@@ -251,12 +272,22 @@ KColorScheme::KColorScheme(QWidget *parent, const char *name, const QStringList
 
     cbExportColors = new QCheckBox(i18n("Apply colors to &non-KDE applications"), this);
     topLayout->addMultiCellWidget( cbExportColors, 2, 2, 0, 1 );
-    connect(cbExportColors, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+    connect(cbExportColors, SIGNAL(toggled(bool)), this, SLOT(changed()));
 
     QWhatsThis::add(cbExportColors, i18n("Check this box to apply the"
-       " current color scheme to GTK applications."));
+       " current color scheme to non-KDE applications."));
 
     load();
+
+    KAboutData* about = new KAboutData("kcmcolors", I18N_NOOP("Colors"), 0, 0,
+        KAboutData::License_GPL,
+        I18N_NOOP("(c) 1997-2005 Colors Developers"), 0, 0);
+    about->addAuthor("Mark Donohoe", 0, 0);
+    about->addAuthor("Matthias Hoelzer", 0, 0);
+    about->addAuthor("Matthias Ettrich", 0, 0);
+    about->addAuthor("Geert Jansen", 0, 0);
+    about->addAuthor("Waldo Bastian", 0, 0);
+    setAboutData( about );
 }
 
 
@@ -265,12 +296,6 @@ KColorScheme::~KColorScheme()
     delete mSchemeList;
 }
 
-void KColorScheme::slotChanged()
-{
-    m_bChanged = true;
-    emit changed(true);
-}
-
 void KColorScheme::setColorName( const QString &name, int id )
 {
     wcCombo->changeItem(name, id);
@@ -278,14 +303,21 @@ void KColorScheme::setColorName( const QString &name, int id )
 }
 
 void KColorScheme::load()
+{
+   load( false );
+}
+void KColorScheme::load( bool useDefaults )
 {
     KConfig *config = KGlobal::config();
+    config->setReadDefaults(  useDefaults );
     config->setGroup("KDE");
     sCurrentScheme = config->readEntry("colorScheme");
 
     sList->setCurrentItem(findSchemeByName(sCurrentScheme));
     readScheme(0);
 
+    cbShadeList->setChecked(cs->shadeSortColumn);
+
     cs->drawSampleWidgets();
     slotWidgetColor(wcCombo->currentItem());
     sb->blockSignals(true);
@@ -297,16 +329,12 @@ void KColorScheme::load()
     bool exportColors = cfg.readBoolEntry("exportKDEColors", true);
     cbExportColors->setChecked(exportColors);
 
-    m_bChanged = false;
-    emit changed(false);
+    emit changed( useDefaults );
 }
 
 
 void KColorScheme::save()
 {
-    if (!m_bChanged)
-        return;
-
     KConfig *cfg = KGlobal::config();
     cfg->setGroup( "General" );
     cfg->writeEntry("background", cs->back, true, true);
@@ -321,6 +349,8 @@ void KColorScheme::save()
     cfg->writeEntry("visitedLinkColor", cs->visitedLink, true, true);
     cfg->writeEntry("alternateBackground", cs->alternateBackground, true, true);
 
+    cfg->writeEntry("shadeSortColumn", cs->shadeSortColumn, true, true);
+
     cfg->setGroup( "WM" );
     cfg->writeEntry("activeForeground", cs->aTxt, true, true);
     cfg->writeEntry("inactiveBackground", cs->iaTitle, true, true);
@@ -366,9 +396,11 @@ void KColorScheme::save()
         flags |= KRdbExportColors;
     else
     {
+#if defined Q_WS_X11 && !defined K_WS_QTONLY
         // Undo the property xrdb has placed on the root window (if any),
         // i.e. remove all entries, including ours
         XDeleteProperty( qt_xdisplay(), qt_xrootwin(), XA_RESOURCE_MANAGER );
+#endif
     }
     runRdb( flags );	// Save the palette to qtrc for KStyles
 
@@ -386,45 +418,15 @@ void KColorScheme::save()
     preview = mkColorPreview(cs);
     sList->changeItem(preview, sList->text(current), current);
 
-    m_bChanged = false;
     emit changed(false);
 }
 
 
 void KColorScheme::defaults()
 {
-    readScheme(1);
-    sList->setCurrentItem(1);
-
-    cs->drawSampleWidgets();
-    slotWidgetColor(wcCombo->currentItem());
-    sb->blockSignals(true);
-    sb->setValue(cs->contrast);
-    sb->blockSignals(false);
-
-    cbExportColors->setChecked(true);
-
-    m_bChanged = true;
-    emit changed(true);
+   load( true );
 }
 
-QString KColorScheme::quickHelp() const
-{
-    return i18n("<h1>Colors</h1> This module allows you to choose"
-       " the color scheme used for the KDE desktop. The different"
-       " elements of the desktop, such as title bars, menu text, etc.,"
-       " are called \"widgets\". You can choose the widget whose"
-       " color you want to change by selecting it from a list, or by"
-       " clicking on a graphical representation of the desktop.<p>"
-       " You can save color settings as complete color schemes,"
-       " which can also be modified or deleted. KDE comes with several"
-       " predefined color schemes on which you can base your own.<p>"
-       " All KDE applications will obey the selected color scheme."
-       " Non-KDE applications may also obey some or all of the color"
-       " settings. See the \"Style\" control module for more details.");
-}
-
-
 void KColorScheme::sliderValueChanged( int val )
 {
     cs->contrast = val;
@@ -432,7 +434,6 @@ void KColorScheme::sliderValueChanged( int val )
 
     sCurrentScheme = QString::null;
 
-    m_bChanged = true;
     emit changed(true);
 }
 
@@ -472,6 +473,7 @@ void KColorScheme::slotSave( )
     config->writeEntry("linkColor", cs->link);
     config->writeEntry("visitedLinkColor", cs->visitedLink);
     config->writeEntry("alternateBackground", cs->alternateBackground);
+    config->writeEntry("shadeSortColumn", cs->shadeSortColumn);
 
     delete config;
 }
@@ -580,14 +582,14 @@ void KColorScheme::slotImport()
 {
 	QString location = locateLocal( "data", "kdisplay/color-schemes/" );
 
-	KURL file = KFileDialog::getOpenFileName(QString::null, "*.kcsrc", this);
+	KURL file ( KFileDialog::getOpenFileName(QString::null, "*.kcsrc", this) );
 	if ( file.isEmpty() )
 		return;
 
 	//kdDebug() << "Location: " << location << endl;
-	if (!KIO::NetAccess::file_copy(file, location+file.fileName( false )  ) )
+	if (!KIO::NetAccess::file_copy(file, KURL( location+file.fileName( false ) ) ) )
 	{
-		KMessageBox::error(this, KIO::NetAccess::lastErrorString(),i18n("Import failed!"));
+		KMessageBox::error(this, KIO::NetAccess::lastErrorString(),i18n("Import failed."));
 		return;
 	}
 	else
@@ -687,7 +689,6 @@ void KColorScheme::slotSelectColor(const QColor &col)
 
     sCurrentScheme = QString::null;
 
-    m_bChanged = true;
     emit changed(true);
 }
 
@@ -718,6 +719,13 @@ void KColorScheme::slotColorForWidget(int indx, const QColor& col)
     slotSelectColor(col);
 }
 
+void KColorScheme::slotShadeSortColumnChanged(bool b)
+{
+    cs->shadeSortColumn = b;
+    sCurrentScheme = QString::null;
+
+    emit changed(true);
+}
 
 /*
  * Read a color scheme into "cs".
@@ -728,48 +736,55 @@ void KColorScheme::readScheme( int index )
 {
     KConfigBase* config;
 
-    QColor widget(238, 238, 230);
-    QColor kde31Beige(255,221,118);
-    QColor emerald1(143,159,180);
-    QColor emerald2(62,145,235);
+    QColor widget(239, 239, 239);
+    QColor kde34Blue(103,141,178);
+    QColor inactiveBackground(157,170,186);
+    QColor activeBackground(65,142,220);
+    QColor inactiveForeground(221,221,221);
+    QColor activeBlend(107,145,184);
+    QColor inactiveBlend(157,170,186);
+    QColor activeTitleBtnBg(220,220,220);
+    QColor inactiveTitleBtnBg(220,220,220);
+    QColor alternateBackground(237,244,249);
 
     QColor button;
     if (QPixmap::defaultDepth() > 8)
-      button.setRgb(238, 234, 222);
+      button.setRgb(221, 223, 228 );
     else
       button.setRgb(220, 220, 220);
 
-    QColor link(0, 0, 192);
-    QColor visitedLink(128, 0,128);
+    QColor link(0, 0, 238);
+    QColor visitedLink(82, 24,139);
 
     // note: keep default color scheme in sync with default Current Scheme
     if (index == 1) {
       sCurrentScheme  = "<default>";
       cs->txt         = black;
       cs->back        = widget;
-      cs->select      = kde31Beige;
-      cs->selectTxt   = black;
+      cs->select      = kde34Blue;
+      cs->selectTxt   = white;
       cs->window      = white;
       cs->windowTxt   = black;
-      cs->iaTitle     = emerald1;
-      cs->iaTxt       = white;
-      cs->iaBlend     = emerald1;
-      cs->aTitle      = emerald2;
+      cs->iaTitle     = inactiveBackground;
+      cs->iaTxt       = inactiveForeground;
+      cs->iaBlend     = inactiveBlend;
+      cs->aTitle      = activeBackground;
       cs->aTxt        = white;
-      cs->aBlend      = emerald2;
+      cs->aBlend      = activeBlend;
       cs->button      = button;
       cs->buttonTxt   = black;
-      cs->aTitleBtn   = cs->back;
-      cs->iTitleBtn   = cs->back;
+      cs->aTitleBtn   = activeTitleBtnBg;
+      cs->iTitleBtn   = inactiveTitleBtnBg;
       cs->aFrame      = cs->back;
       cs->aHandle     = cs->back;
       cs->iaFrame     = cs->back;
       cs->iaHandle    = cs->back;
       cs->link        = link;
       cs->visitedLink = visitedLink;
-      cs->alternateBackground = KGlobalSettings::calculateAlternateBackgroundColor(cs->window);
+      cs->alternateBackground = alternateBackground;
 
       cs->contrast    = 7;
+      cs->shadeSortColumn = KDE_DEFAULT_SHADE_SORT_COLUMN;
 
       return;
     }
@@ -790,11 +805,13 @@ void KColorScheme::readScheme( int index )
         sCurrentScheme = sCurrentScheme.mid(i+1);
     }
 
+    cs->shadeSortColumn = config->readBoolEntry( "shadeSortColumn", KDE_DEFAULT_SHADE_SORT_COLUMN );
+
     // note: defaults should be the same as the KDE default
     cs->txt = config->readColorEntry( "foreground", &black );
     cs->back = config->readColorEntry( "background", &widget );
-    cs->select = config->readColorEntry( "selectBackground", &kde31Beige );
-    cs->selectTxt = config->readColorEntry( "selectForeground", &black );
+    cs->select = config->readColorEntry( "selectBackground", &kde34Blue );
+    cs->selectTxt = config->readColorEntry( "selectForeground", &white );
     cs->window = config->readColorEntry( "windowBackground", &white );
     cs->windowTxt = config->readColorEntry( "windowForeground", &black );
     cs->button = config->readColorEntry( "buttonBackground", &button );
@@ -807,19 +824,19 @@ void KColorScheme::readScheme( int index )
     if (index == 0)
       config->setGroup( "WM" );
 
-    cs->iaTitle = config->readColorEntry("inactiveBackground", &emerald1);
-    cs->iaTxt = config->readColorEntry("inactiveForeground", &white);
-    cs->iaBlend = config->readColorEntry("inactiveBlend", &emerald1);
+    cs->iaTitle = config->readColorEntry("inactiveBackground", &inactiveBackground);
+    cs->iaTxt = config->readColorEntry("inactiveForeground", &inactiveForeground);
+    cs->iaBlend = config->readColorEntry("inactiveBlend", &inactiveBackground);
     cs->iaFrame = config->readColorEntry("inactiveFrame", &cs->back);
     cs->iaHandle = config->readColorEntry("inactiveHandle", &cs->back);
-    cs->aTitle = config->readColorEntry("activeBackground", &emerald2);
+    cs->aTitle = config->readColorEntry("activeBackground", &activeBackground);
     cs->aTxt = config->readColorEntry("activeForeground", &white);
-    cs->aBlend = config->readColorEntry("activeBlend", &emerald2);
+    cs->aBlend = config->readColorEntry("activeBlend", &activeBlend);
     cs->aFrame = config->readColorEntry("frame", &cs->back);
     cs->aHandle = config->readColorEntry("handle", &cs->back);
     // hack - this is all going away. For now just set all to button bg
-    cs->aTitleBtn = config->readColorEntry("activeTitleBtnBg", &cs->back);
-    cs->iTitleBtn = config->readColorEntry("inactiveTitleBtnBg", &cs->back);
+    cs->aTitleBtn = config->readColorEntry("activeTitleBtnBg", &activeTitleBtnBg);
+    cs->iTitleBtn = config->readColorEntry("inactiveTitleBtnBg", &inactiveTitleBtnBg);
 
     if (index == 0)
       config->setGroup( "KDE" );
@@ -847,7 +864,7 @@ void KColorScheme::readSchemeNames()
             "kdisplay/color-schemes/*.kcsrc", false, true);
 
     // And add them
-    for (QStringList::ConstIterator it = list.begin(); it != list.end(); it++) {
+    for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it) {
        KSimpleConfig *config = new KSimpleConfig(*it);
        config->setGroup("Color Scheme");
        QString str = config->readEntry("Name");
@@ -896,7 +913,9 @@ int KColorScheme::findSchemeByName(const QString &scheme)
 
    for(KColorSchemeEntry *entry = mSchemeList->first(); entry; entry = mSchemeList->next())
    {
-      if (entry->path.endsWith(search))
+      KURL url;
+      url.setPath(entry->path);
+      if (url.fileName() == search)
          return i+nSysSchemes;
       i++;
    }
@@ -911,6 +930,8 @@ void KColorScheme::slotPreviewScheme(int indx)
 
     // Set various appropriate for the scheme
 
+    cbShadeList->setChecked(cs->shadeSortColumn);
+
     cs->drawSampleWidgets();
     sb->blockSignals(true);
     sb->setValue(cs->contrast);
@@ -924,8 +945,7 @@ void KColorScheme::slotPreviewScheme(int indx)
        removeBt->setEnabled(entry ? entry->local : false);
     }
 
-    m_bChanged = (indx != 0);
-    emit changed(m_bChanged);
+    emit changed((indx != 0));
 }
 
 
