@@ -48,6 +48,7 @@ KPluginOptions::KPluginOptions( KConfig* config, QString group, QWidget *parent,
     : KCModule( parent, "kcmkonqhtml" ),
       m_pConfig( config ),
       m_groupname( group ),
+      m_nspluginscan (0),
       global_policies(config,group,true)
 {
     QVBoxLayout* toplevel = new QVBoxLayout( this, 0, KDialog::spacingHint() );
@@ -169,9 +170,15 @@ void KPluginOptions::updatePLabel(int p) {
     priorityLabel->setText(i18n("CPU priority for plugins: %1").arg(level));
 }
 
-
 void KPluginOptions::load()
 {
+	load( false );
+}
+
+void KPluginOptions::load( bool useDefaults )
+{
+	 
+
     // *** load ***
     global_policies.load();
     bool bPluginGlobal = global_policies.isFeatureEnabled();
@@ -185,6 +192,8 @@ void KPluginOptions::load()
 
   KConfig *config = new KConfig("kcmnspluginrc", true);
 
+  config->setReadDefaults( useDefaults );
+
   config->setGroup("Misc");
   m_widget->scanAtStartup->setChecked( config->readBoolEntry( "startkdeScan", false ) );
 
@@ -203,33 +212,12 @@ void KPluginOptions::load()
 
   delete config;
 
-  change( false );
+  emit changed( useDefaults );
 }
 
 void KPluginOptions::defaults()
 {
-    global_policies.defaults();
-    enablePluginsGloballyCB->setChecked( global_policies.isFeatureEnabled() );
-    enableHTTPOnly->setChecked(false);
-    enableUserDemand->setChecked(false);
-    priority->setValue(100);
-
-/*****************************************************************************/
-
-    KConfig *config = new KConfig( QString::null, true, false );
-
-    m_widget->scanAtStartup->setChecked( false );
-
-    m_widget->dirEdit->setURL("");
-    m_widget->dirEdit->setEnabled( false );
-    m_widget->dirRemove->setEnabled( false );
-
-    dirLoad( config );
-    pluginLoad( config );
-
-    delete config;
-
-    change();
+    load( true );
 }
 
 void KPluginOptions::save()
@@ -303,11 +291,12 @@ void KPluginOptions::scan()
              save();
     }
 
-    KProcIO* nspluginscan = new KProcIO;
+    m_nspluginscan = new KProcIO;
     QString scanExe = KGlobal::dirs()->findExe("nspluginscan");
     if (!scanExe) {
         kdDebug() << "can't find nspluginviewer" << endl;
-        delete nspluginscan;
+        delete m_nspluginscan; 
+        m_nspluginscan = 0L;
 
         KMessageBox::sorry ( this,
                              i18n("The nspluginscan executable cannot be found. "
@@ -321,27 +310,15 @@ void KPluginOptions::scan()
     m_progress->setProgress( 5 );
 
     // start nspluginscan
-    *nspluginscan << scanExe << "--verbose";
+    *m_nspluginscan << scanExe << "--verbose";
     kdDebug() << "Running nspluginscan" << endl;
-    connect(nspluginscan, SIGNAL(readReady(KProcIO*)),
+    connect(m_nspluginscan, SIGNAL(readReady(KProcIO*)),
             this, SLOT(progress(KProcIO*)));
-    connect(nspluginscan, SIGNAL(processExited(KProcess *)),
+    connect(m_nspluginscan, SIGNAL(processExited(KProcess *)),
             this, SLOT(scanDone()));
     connect(m_progress, SIGNAL(cancelled()), this, SLOT(scanDone()));
 
-    if (nspluginscan->start())
-       kapp->enter_loop();
-
-    delete nspluginscan;
-
-    // update dialog
-    if (m_progress) {
-        m_progress->setProgress(100);
-        load();
-        delete m_progress;
-        m_progress = 0;
-    }
-    m_widget->scanButton->setEnabled(true);
+    m_nspluginscan->start();
 }
 
 void KPluginOptions::progress(KProcIO *proc)
@@ -354,7 +331,12 @@ void KPluginOptions::progress(KProcIO *proc)
 
 void KPluginOptions::scanDone()
 {
-    kapp->exit_loop();
+    m_progress->setProgress(100);
+    load();
+
+    delete m_progress; m_progress = 0L;
+    m_nspluginscan->deleteLater(); m_nspluginscan = 0L;
+    m_widget->scanButton->setEnabled(true);
 }
 
 /***********************************************************************************/
