@@ -99,6 +99,7 @@ public:
 	~KPluginOptions();
 
     virtual void load();
+    virtual void load( bool useDefaults );
     virtual void save();
     virtual void defaults();
     QString quickHelp() const;
@@ -129,6 +130,7 @@ private:
   NSConfigWidget *m_widget;
   bool m_changed;
   QProgressDialog *m_progress;
+  KProcIO* m_nspluginscan;
   QSlider *priority;
   QLabel *priorityLabel;
   PluginPolicies global_policies;
