@@ -1,4 +1,5 @@
 /* vi: ts=8 sts=4 sw=4
+ * kate: space-indent on; tab-width 8; indent-width 4; indent-mode cstyle;
  *
  * This file is part of the KDE project, module kdesktop.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -11,6 +12,8 @@
 #include "bgrender.h"
 #include "bgmanager.h"
 #include "bgdefaults.h"
+#include "kdesktopsettings.h"
+#include "bgsettings.h"
 
 #include <assert.h>
 
@@ -41,8 +44,8 @@ template class QPtrVector<KBackgroundRenderer>;
 template class QPtrVector<KBackgroundCacheEntry>;
 template class QMemArray<int>;
 
-Atom prop_root;
-bool properties_inited = false;
+static Atom prop_root;
+static bool properties_inited = false;
 
 /**** KBackgroundManager ****/
 
@@ -61,9 +64,8 @@ KBackgroundManager::KBackgroundManager(QWidget *desktop, KWinModule* kwinModule)
     if (desktop == 0L)
         desktop = QApplication::desktop()->screen();
 
-    // Preallocate 4 desktops, higher ones are allocated when needed.
-    m_Renderer.resize( 4 );
-    m_Cache.resize( 4 );
+    m_Renderer.resize( 1 );
+    m_Cache.resize( 1 );
 
     m_Serial = 0; m_Hash = 0;
     m_pConfig = KGlobal::config();
@@ -78,8 +80,9 @@ KBackgroundManager::KBackgroundManager(QWidget *desktop, KWinModule* kwinModule)
         m_Cache[i]->pixmap = 0L;
         m_Cache[i]->hash = 0;
         m_Cache[i]->exp_from = -1;
-        m_Renderer.insert (i, new KBackgroundRenderer(i,m_pConfig));
+        m_Renderer.insert (i, new KVirtualBGRenderer(i,m_pConfig));
         connect(m_Renderer[i], SIGNAL(imageDone(int)), SLOT(slotImageDone(int)));
+        m_Renderer[i]->enableTiling( true ); // optimize
     }
 
     configure();
@@ -114,10 +117,13 @@ KBackgroundManager::~KBackgroundManager()
     int format;
     unsigned long length, after;
     unsigned char* data_root;
-    XGetWindowProperty( qt_xdisplay(), qt_xrootwin(), prop_root, 0L, 1L, False, AnyPropertyType,
-	&type, &format, &length, &after, &data_root);
-    if (type == XA_PIXMAP)
-	pm = *((Pixmap*)data_root);
+    if( XGetWindowProperty( qt_xdisplay(), qt_xrootwin(), prop_root, 0L, 1L, False, AnyPropertyType,
+	&type, &format, &length, &after, &data_root) == Success && data_root != NULL )
+    {
+	if (type == XA_PIXMAP)
+	    pm = *((Pixmap*)data_root);
+        XFree( data_root );
+    }
     // only if it's our pixmap
     if( pm == m_xrootpmap )
 	XDeleteProperty(qt_xdisplay(), qt_xrootwin(), prop_root);
@@ -128,8 +134,8 @@ KBackgroundManager::~KBackgroundManager()
 
     for (unsigned i=0; i<m_Cache.size(); i++)
     {
-	if (m_Cache[i]->pixmap)
-	    delete m_Cache[i]->pixmap;
+        delete m_Cache[i]->pixmap;
+        delete m_Cache[i];
     }
 }
 
@@ -187,9 +193,10 @@ void KBackgroundManager::configure()
 {
     // Global settings
     m_pConfig->reparseConfiguration();
+    KDesktopSettings::self()->readConfig();
 
     // Read individual settings
-    KBackgroundRenderer *r;
+    KVirtualBGRenderer *r;
     for (unsigned i=0; i<m_Renderer.size(); i++)
     {
         r = m_Renderer[i];
@@ -199,11 +206,10 @@ void KBackgroundManager::configure()
             removeCache(i);
     }
 
-    m_pConfig->setGroup("Background Common");
-    applyCommon(m_pConfig->readBoolEntry("CommonDesktop", _defCommon));
+    applyCommon(KDesktopSettings::commonDesktop());
 
-    bool limit = m_pConfig->readBoolEntry("LimitCache", _defLimitCache);
-    int size = m_pConfig->readNumEntry("CacheSize", _defCacheSize) * 1024;
+    bool limit = KDesktopSettings::limitCache();
+    int size = KDesktopSettings::cacheSize() * 1024;
     applyCache(limit, size);
 
     // Repaint desktop
@@ -258,8 +264,9 @@ void KBackgroundManager::slotChangeNumberOfDesktops(int num)
 	    m_Cache[i]->pixmap = 0L;
 	    m_Cache[i]->hash = 0;
 	    m_Cache[i]->exp_from = -1;
-	    m_Renderer.insert(i, new KBackgroundRenderer(i,m_pConfig));
+            m_Renderer.insert(i, new KVirtualBGRenderer(i,m_pConfig));
 	    connect(m_Renderer[i], SIGNAL(imageDone(int)), SLOT(slotImageDone(int)));
+            m_Renderer[i]->enableTiling( true ); // optimize
 	}
     }
 }
@@ -289,6 +296,8 @@ void KBackgroundManager::slotChangeDesktop(int desk)
 	exportBackground(m_Current, desk);
         return;
     }
+    m_Renderer[edesk]->stop();
+    m_Renderer[edesk]->cleanup();
 
     // If we have the background already rendered: set it
     for (unsigned i=0; i<m_Cache.size(); i++)
@@ -350,6 +359,9 @@ void KBackgroundManager::setPixmap(KPixmap *pm, int hash, int desk)
        { // clear the root window pixmap set by kdm
           root_cleared = true;
 	  QTimer::singleShot( 0, this, SLOT( clearRoot()));
+          // but make the pixmap visible until m_pDesktop is visible
+          QApplication::desktop()->screen()->setErasePixmap(*pm);
+          QApplication::desktop()->screen()->erase();
        }
     }
     else
@@ -384,7 +396,7 @@ void KBackgroundManager::clearRoot()
  */
 void KBackgroundManager::renderBackground(int desk)
 {
-    KBackgroundRenderer *r = m_Renderer[desk];
+    KVirtualBGRenderer *r = m_Renderer[desk];
     if (r->isActive())
     {
         kdDebug() << "renderer " << desk << " already active" << endl;
@@ -401,11 +413,10 @@ void KBackgroundManager::renderBackground(int desk)
 void KBackgroundManager::slotImageDone(int desk)
 {
     KPixmap *pm = new KPixmap();
-    KBackgroundRenderer *r = m_Renderer[desk];
-
-    *pm = *r->pixmap();
-    r->cleanup();
+    KVirtualBGRenderer *r = m_Renderer[desk];
+    bool do_cleanup = true;
 
+    *pm = r->pixmap();
     // If current: paint it
     bool current = (r->hash() == m_Renderer[effectiveDesktop()]->hash());
     if (current)
@@ -415,6 +426,8 @@ void KBackgroundManager::slotImageDone(int desk)
         {
             m_bBgInitDone = true;
             emit initDone();
+            QTimer::singleShot( 30000, this, SLOT( saveImages()));
+            do_cleanup = false;
         }
     }
     if (m_bExport || !m_bCommon)
@@ -424,9 +437,24 @@ void KBackgroundManager::slotImageDone(int desk)
 
     if (current)
         exportBackground(desk, realDesktop());
+
+    if( do_cleanup )
+    {
+        r->saveCacheFile();
+        r->cleanup();
+    }
 }
 
 
+void KBackgroundManager::saveImages()
+{
+    for (unsigned i=0; i<m_Renderer.size(); i++)
+    {
+        m_Renderer[i]->saveCacheFile();
+        m_Renderer[i]->cleanup();
+    }
+}
+
 /*
  * Size in bytes of a QPixmap. For use in the pixmap cache.
  */
@@ -549,20 +577,17 @@ void KBackgroundManager::slotTimeout()
 
     for (int i=0; i<NumDesks; i++)
     {
-        KBackgroundRenderer *r = m_Renderer[i];
+        KVirtualBGRenderer *r = m_Renderer[i];
         bool change = false;
-
-        if ((r->backgroundMode() == KBackgroundSettings::Program) &&
-	    (r->KBackgroundProgram::needUpdate()) &&
-            (!running.contains(r->hash()))
-	   )
+        
+        if (r->needProgramUpdate())
         {
-	    r->KBackgroundProgram::update();
+            r->programUpdate();
             change = true;
         }
-
+        
         if (r->needWallpaperChange())
-	{
+        {
             r->changeWallpaper();
             change = true;
         }
@@ -578,6 +603,9 @@ void KBackgroundManager::slotTimeout()
 // Return a valid desk number.
 int KBackgroundManager::validateDesk(int desk)
 {
+    if (desk > (int)m_Renderer.size())
+	slotChangeNumberOfDesktops( m_pKwinmodule->numberOfDesktops() );
+
     if ( (desk <= 0) || (desk > (int)m_Renderer.size()) )
         return realDesktop();
 
@@ -589,7 +617,8 @@ int KBackgroundManager::validateDesk(int desk)
 // 0 is for the current visible desktop.
 QString KBackgroundManager::currentWallpaper(int desk)
 {
-    KBackgroundRenderer *r = m_Renderer[validateDesk(desk)];
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    KBackgroundRenderer *r = m_Renderer[validateDesk(desk)]->renderer(0);
 
     return r->currentWallpaper();
 }
@@ -597,7 +626,7 @@ QString KBackgroundManager::currentWallpaper(int desk)
 // DCOP exported
 void KBackgroundManager::changeWallpaper()
 {
-    KBackgroundRenderer *r = m_Renderer[effectiveDesktop()];
+    KVirtualBGRenderer *r = m_Renderer[effectiveDesktop()];
 
     r->changeWallpaper();
     slotChangeDesktop(0);
@@ -615,27 +644,36 @@ void KBackgroundManager::setExport(int _export)
 void KBackgroundManager::setCommon(int common)
 {
     applyCommon(common);
-    m_pConfig->setGroup("Background Common");
-    m_pConfig->writeEntry("CommonDesktop", m_bCommon);
-    m_pConfig->sync();
+    KDesktopSettings::setCommonDesktop( m_bCommon );
+    KDesktopSettings::writeConfig();
     slotChangeDesktop(0);
 }
 
 // DCOP exported
 void KBackgroundManager::setWallpaper(QString wallpaper, int mode)
 {
-    KBackgroundRenderer *r = m_Renderer[effectiveDesktop()];
-    r->stop();
-    r->setWallpaperMode(mode);
-    r->setMultiWallpaperMode(KBackgroundSettings::NoMulti);
-    r->setWallpaper(wallpaper);
-    r->writeSettings();
+    if (mode < 0 || mode >= KBackgroundSettings::lastBackgroundMode) {
+      kdDebug() << "Invalid background mode " << mode << " passed to " << k_funcinfo << "\n";
+      return;
+    }
+
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    for (unsigned i=0; i < m_Renderer[effectiveDesktop()]->numRenderers(); ++i)
+    {
+        KBackgroundRenderer *r = m_Renderer[effectiveDesktop()]->renderer(i);
+        r->stop();
+        r->setWallpaperMode(mode);
+        r->setMultiWallpaperMode(KBackgroundSettings::NoMulti);
+        r->setWallpaper(wallpaper);
+        r->writeSettings();
+    }
     slotChangeDesktop(0);
 }
 
 void KBackgroundManager::setWallpaper(QString wallpaper)
 {
-    KBackgroundRenderer *r = m_Renderer[effectiveDesktop()];
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    KBackgroundRenderer *r = m_Renderer[effectiveDesktop()]->renderer(0);
     int mode = r->wallpaperMode();
     if (mode == KBackgroundSettings::NoWallpaper)
        mode = KBackgroundSettings::Tiled;
@@ -647,7 +685,8 @@ void KBackgroundManager::setWallpaper(QString wallpaper)
 // 0 is for current visible desktop.
 QStringList KBackgroundManager::wallpaperFiles(int desk)
 {
-    KBackgroundRenderer *r = m_Renderer[validateDesk(desk)];
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    KBackgroundRenderer *r = m_Renderer[validateDesk(desk)]->renderer(0);
 
     return r->wallpaperFiles();
 }
@@ -657,7 +696,8 @@ QStringList KBackgroundManager::wallpaperFiles(int desk)
 // show window) for specified desk.  0 is for current visible desktop.
 QStringList KBackgroundManager::wallpaperList(int desk)
 {
-    KBackgroundRenderer *r = m_Renderer[validateDesk(desk)];
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    KBackgroundRenderer *r = m_Renderer[validateDesk(desk)]->renderer(0);;
 
     return r->wallpaperList();
 }
@@ -666,12 +706,36 @@ QStringList KBackgroundManager::wallpaperList(int desk)
 void KBackgroundManager::setCache( int bLimit, int size )
 {
     applyCache( bLimit, size*1024 );
-    m_pConfig->setGroup( "Background Common" );
-    m_pConfig->writeEntry( "LimitCache", (bool) bLimit );
-    m_pConfig->writeEntry( "CacheSize", size );
-    m_pConfig->sync();
+    KDesktopSettings::setLimitCache( (bool) bLimit );
+    KDesktopSettings::setCacheSize( size );
+    KDesktopSettings::writeConfig();
 }
 
+// DCOP exported
+void KBackgroundManager::setWallpaper(int desk, QString wallpaper, int mode)
+{
+    if (mode < 0 || mode >= KBackgroundSettings::lastBackgroundMode) {
+      kdDebug() << "Invalid background mode " << mode << " passed to " << k_funcinfo << "\n";
+      return;
+    }
+
+    int sdesk = validateDesk(desk);
+
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    for (unsigned i=0; i < m_Renderer[sdesk]->numRenderers(); ++i)
+    {
+        KBackgroundRenderer *r = m_Renderer[sdesk]->renderer(i);
+    
+        setCommon(false);   // Force each desktop to have it's own wallpaper
+    
+        r->stop();
+        r->setWallpaperMode(mode);
+        r->setMultiWallpaperMode(KBackgroundSettings::NoMulti);
+        r->setWallpaper(wallpaper);
+        r->writeSettings();
+    }
+    slotChangeDesktop(sdesk);
+}
 
 void KBackgroundManager::repaintBackground()
 {
@@ -685,7 +749,7 @@ void KBackgroundManager::desktopResized()
 {
     for (unsigned i=0; i<m_Renderer.size(); i++)
     {
-        KBackgroundRenderer* r = m_Renderer[i];
+        KVirtualBGRenderer * r = m_Renderer[i];
         if( r->isActive())
             r->stop();
         removeCache(i);
@@ -702,23 +766,27 @@ void KBackgroundManager::desktopResized()
 // DCOP exported
 void KBackgroundManager::setColor(const QColor & c, bool isColorA)
 {
-    KBackgroundRenderer *r = m_Renderer[effectiveDesktop()];
-    r->stop();
-
-    if (isColorA)
-        r->setColorA(c);
-    else
-        r->setColorB(c);
-
-    int mode = r->backgroundMode();
-    if (mode == KBackgroundSettings::Program)
-       mode = KBackgroundSettings::Flat;
-
-    if (!isColorA && (mode == KBackgroundSettings::Flat))
-       mode = KBackgroundSettings::VerticalGradient;
-    r->setBackgroundMode(mode);
-
-    r->writeSettings();
+    //TODO Is the behaviour of this function appropriate for multiple screens?
+    for (unsigned i=0; i < m_Renderer[effectiveDesktop()]->numRenderers(); ++i)
+    {
+        KBackgroundRenderer *r = m_Renderer[effectiveDesktop()]->renderer(i);
+        r->stop();
+    
+        if (isColorA)
+            r->setColorA(c);
+        else
+            r->setColorB(c);
+    
+        int mode = r->backgroundMode();
+        if (mode == KBackgroundSettings::Program)
+        mode = KBackgroundSettings::Flat;
+    
+        if (!isColorA && (mode == KBackgroundSettings::Flat))
+        mode = KBackgroundSettings::VerticalGradient;
+        r->setBackgroundMode(mode);
+    
+        r->writeSettings();
+    }
     slotChangeDesktop(0);
 }
 
