@@ -10,7 +10,9 @@
 
 #include "lockprocess.h"
 #include "lockdlg.h"
+
 #include <kcheckpass.h>
+#include <dmctl.h>
 
 #include <kapplication.h>
 #include <klocale.h>
@@ -25,6 +27,7 @@
 #include <kdebug.h>
 #include <kuser.h>
 #include <dcopref.h>
+#include <kmessagebox.h>
 
 #include <qlayout.h>
 #include <qpushbutton.h>
@@ -35,6 +38,9 @@
 #include <qfontmetrics.h>
 #include <qstyle.h>
 #include <qapplication.h>
+#include <qlistview.h>
+#include <qheader.h>
+#include <qcheckbox.h>
 
 #include <ctype.h>
 #include <unistd.h>
@@ -58,7 +64,7 @@
 //
 // Simple dialog for entering a password.
 //
-PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, bool nsess)
+PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin)
     : QDialog(parent, "password dialog", true, WX11BypassWM),
       mPlugin( plugin ),
       mCapsLocked(-1),
@@ -74,7 +80,7 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, bool
     KUser user;
     QLabel *greetLabel = new QLabel( user.fullName().isEmpty() ?
             i18n("<nobr><b>The session is locked</b><br>") :
-            i18n("<nobr><b>The session is locked by %1</b><br>").arg( user.fullName() ), frame );
+            i18n("<nobr><b>The session was locked by %1</b><br>").arg( user.fullName() ), frame );
 
     mStatusLabel = new QLabel( "<b> </b>", frame );
     mStatusLabel->setAlignment( QLabel::AlignCenter );
@@ -84,11 +90,11 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, bool
 
     KSeparator *sep = new KSeparator( KSeparator::HLine, frame );
 
-    mNewSessButton = new KPushButton( KGuiItem(i18n("&Start New Session..."), "fork"), frame );
+    mNewSessButton = new KPushButton( KGuiItem(i18n("Sw&itch User..."), "fork"), frame );
     ok = new KPushButton( i18n("Unl&ock"), frame );
     cancel = new KPushButton( KStdGuiItem::cancel(), frame );
 
-    greet = plugin->info->create( this, this, mLayoutButton, QString::null,
+    greet = plugin->info->create( this, 0, this, mLayoutButton, QString::null,
               KGreeterPlugin::Authenticate, KGreeterPlugin::ExUnlock );
 
 
@@ -120,9 +126,9 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, bool
     connect(mLayoutButton, SIGNAL(clicked()), this, SLOT(layoutClicked()));
     connect(cancel, SIGNAL(clicked()), SLOT(reject()));
     connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
-    connect(mNewSessButton, SIGNAL(clicked()), SLOT(slotStartNewSession()));
+    connect(mNewSessButton, SIGNAL(clicked()), SLOT(slotSwitchUser()));
 
-    if (!nsess || !kapp->authorize("start_new_session"))
+    if (!DM().isSwitchable())
         mNewSessButton->hide();
 
     installEventFilter(this);
@@ -137,9 +143,13 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, bool
     if( !kxkb.isNull() ) {
         layoutsList = kxkb.call("getLayoutsList");
         QString currentLayout = kxkb.call("getCurrentLayout");
-        if( !currentLayout.isEmpty() && layoutsList.count() > 1 )
-            setLayoutText(currentLayout);
-        else
+        if( !currentLayout.isEmpty() && layoutsList.count() > 1 ) {
+            currLayout = layoutsList.find(currentLayout);
+            if (currLayout == layoutsList.end())
+                setLayoutText("err");
+            else
+                setLayoutText(*currLayout);
+        } else
             mLayoutButton->hide();
     } else {
         mLayoutButton->hide(); // no kxkb running
@@ -156,20 +166,13 @@ PasswordDlg::~PasswordDlg()
 
 void PasswordDlg::layoutClicked()
 {
-    QStringList::iterator it = layoutsList.find( mLayoutButton->text() );
-    if( it == layoutsList.end() ) {  // huh?
 
-	setLayoutText( "err" );
+    if( ++currLayout == layoutsList.end() )
+        currLayout = layoutsList.begin();
 
-    } else {
+    DCOPRef kxkb("kxkb", "kxkb");
+    setLayoutText( kxkb.call("setLayout", *currLayout) ? *currLayout : "err" );
 
-	if( ++it == layoutsList.end() )
-	    it = layoutsList.begin();
-
-	DCOPRef kxkb("kxkb", "kxkb");
-	setLayoutText( kxkb.call("setLayout", *it) ? *it : "err" );
-
-    }
 }
 
 void PasswordDlg::setLayoutText( const QString &txt )
@@ -417,8 +420,8 @@ void PasswordDlg::cantCheck()
 {
     greet->failed();
     static_cast< LockProcess* >(parent())->msgBox( QMessageBox::Critical,
-        i18n("Cannot unlock the session because the authentication system failed to work.\n"
-             "You must kill kdesktop_lock (pid %1) manually.").arg(getpid()) );
+        i18n("Cannot unlock the session because the authentication system failed to work;\n"
+             "you must kill kdesktop_lock (pid %1) manually.").arg(getpid()) );
     greet->revive();
 }
 
@@ -502,6 +505,11 @@ void PasswordDlg::show()
 
 void PasswordDlg::slotStartNewSession()
 {
+    if (!KMessageBox::shouldBeShownContinue( ":confirmNewSession" )) {
+        DM().startReserve();
+        return;
+    }
+
     killTimer(mTimeoutTimerId);
     mTimeoutTimerId = 0;
 
@@ -523,7 +531,9 @@ void PasswordDlg::slotStartNewSession()
                "F%1 is usually assigned to the first session, "
                "F%2 to the second session and so on. "
                "You can switch between sessions by pressing "
-               "CTRL, ALT and the appropriate F-key at the same time.")
+               "Ctrl, Alt and the appropriate F-key at the same time. "
+               "Additionally, the KDE Panel and Desktop menus have "
+               "actions for switching between sessions.")
             .arg(7).arg(8);
     QLabel *label2 = new QLabel( qt_text, winFrame );
     KPushButton *okbutton = new KPushButton( KGuiItem(i18n("&Start New Session"), "fork"), winFrame );
@@ -532,18 +542,22 @@ void PasswordDlg::slotStartNewSession()
     KPushButton *cbutton = new KPushButton( KStdGuiItem::cancel(), winFrame );
     connect( cbutton, SIGNAL( clicked() ), dialog, SLOT( reject() ) );
 
-    QBoxLayout *hbox = new QHBoxLayout( 0 );
+    QBoxLayout *mbox = new QVBoxLayout( winFrame, KDialog::marginHint(), KDialog::spacingHint() );
+
+    QGridLayout *grid = new QGridLayout( mbox, 2, 2, 2 * KDialog::spacingHint() );
+    grid->setMargin( KDialog::marginHint() );
+    grid->addWidget( label1, 0, 0, Qt::AlignCenter );
+    grid->addWidget( label2, 0, 1, Qt::AlignCenter );
+    QCheckBox *cb = new QCheckBox( i18n("&Do not ask again"), winFrame );
+    grid->addMultiCellWidget( cb, 1,1, 0,1 );
+
+    QBoxLayout *hbox = new QHBoxLayout( mbox, KDialog::spacingHint() );
     hbox->addStretch( 1 );
     hbox->addWidget( okbutton );
     hbox->addStretch( 1 );
     hbox->addWidget( cbutton );
     hbox->addStretch( 1 );
 
-    QGridLayout *grid = new QGridLayout( winFrame, 2, 2, 10 );
-    grid->addWidget( label1, 0, 0, Qt::AlignCenter );
-    grid->addWidget( label2, 0, 1, Qt::AlignCenter );
-    grid->addMultiCellLayout( hbox, 1,1, 0,1 );
-
     // stolen from kmessagebox
     int pref_width = 0;
     int pref_height = 0;
@@ -585,12 +599,114 @@ void PasswordDlg::slotStartNewSession()
 
     delete dialog;
 
-    if (ret == QDialog::Accepted)
-        emit startNewSession();
+    if (ret == QDialog::Accepted) {
+        if (cb->isChecked())
+            KMessageBox::saveDontShowAgainContinue( ":confirmNewSession" );
+        DM().startReserve();
+    }
 
     mTimeoutTimerId = startTimer(PASSDLG_HIDE_TIMEOUT);
 }
 
+class LockListViewItem : public QListViewItem {
+public:
+    LockListViewItem( QListView *parent,
+		      const QString &sess, const QString &loc, int _vt )
+	: QListViewItem( parent )
+	, vt( _vt )
+    {
+	setText( 0, sess );
+	setText( 1, loc );
+    }
+
+    int vt;
+};
+
+void PasswordDlg::slotSwitchUser()
+{
+    int p = 0;
+    DM dm;
+
+    QDialog dialog( this, "sessbox", true, WX11BypassWM );
+    QFrame *winFrame = new QFrame( &dialog );
+    winFrame->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
+    winFrame->setLineWidth( 2 );
+    QBoxLayout *vbox = new QVBoxLayout( &dialog );
+    vbox->addWidget( winFrame );
+
+    QBoxLayout *hbox = new QHBoxLayout( winFrame, KDialog::marginHint(), KDialog::spacingHint() );
+
+    QBoxLayout *vbox1 = new QVBoxLayout( hbox, KDialog::spacingHint() );
+    QBoxLayout *vbox2 = new QVBoxLayout( hbox, KDialog::spacingHint() );
+
+    KPushButton *btn;
+
+    SessList sess;
+    if (dm.localSessions( sess )) {
+
+        lv = new QListView( winFrame );
+        connect( lv, SIGNAL(doubleClicked(QListViewItem *, const QPoint&, int)), SLOT(slotSessionActivated()) );
+        connect( lv, SIGNAL(doubleClicked(QListViewItem *, const QPoint&, int)), &dialog, SLOT(reject()) );
+        lv->setAllColumnsShowFocus( true );
+        lv->addColumn( i18n("Session") );
+        lv->addColumn( i18n("Location") );
+        lv->setColumnWidthMode( 0, QListView::Maximum );
+        lv->setColumnWidthMode( 1, QListView::Maximum );
+        QListViewItem *itm = 0;
+        QString user, loc;
+        int ns = 0;
+        for (SessList::ConstIterator it = sess.begin(); it != sess.end(); ++it) {
+            DM::sess2Str2( *it, user, loc );
+            itm = new LockListViewItem( lv, user, loc, (*it).vt );
+            if (!(*it).vt)
+                itm->setEnabled( false );
+            if ((*it).self) {
+                lv->setCurrentItem( itm );
+                itm->setSelected( true );
+            }
+            ns++;
+        }
+        int fw = lv->frameWidth() * 2;
+        QSize hds( lv->header()->sizeHint() );
+        lv->setMinimumWidth( fw + hds.width() +
+            (ns > 10 ? style().pixelMetric(QStyle::PM_ScrollBarExtent) : 0 ) );
+        lv->setFixedHeight( fw + hds.height() +
+            itm->height() * (ns < 6 ? 6 : ns > 10 ? 10 : ns) );
+        lv->header()->adjustHeaderSize();
+        vbox1->addWidget( lv );
+
+        btn = new KPushButton( KGuiItem(i18n("session", "&Activate"), "fork"), winFrame );
+        connect( btn, SIGNAL(clicked()), SLOT(slotSessionActivated()) );
+        connect( btn, SIGNAL(clicked()), &dialog, SLOT(reject()) );
+        vbox2->addWidget( btn );
+        vbox2->addStretch( 2 );
+    }
+
+    if (kapp->authorize("start_new_session") && (p = dm.numReserve()) >= 0)
+    {
+        btn = new KPushButton( KGuiItem(i18n("Start &New Session"), "fork"), winFrame );
+        connect( btn, SIGNAL(clicked()), SLOT(slotStartNewSession()) );
+        connect( btn, SIGNAL(clicked()), &dialog, SLOT(reject()) );
+        if (!p)
+            btn->setEnabled( false );
+        vbox2->addWidget( btn );
+        vbox2->addStretch( 1 );
+    }
+
+    btn = new KPushButton( KStdGuiItem::cancel(), winFrame );
+    connect( btn, SIGNAL(clicked()), &dialog, SLOT(reject()) );
+    vbox2->addWidget( btn );
+
+    static_cast< LockProcess* >(parent())->execDialog( &dialog );
+}
+
+void PasswordDlg::slotSessionActivated()
+{
+    LockListViewItem *itm = (LockListViewItem *)lv->currentItem();
+    if (itm && itm->vt > 0)
+        DM().switchVT( itm->vt );
+}
+
 void PasswordDlg::capsLocked()
 {
     unsigned int lmask;
