@@ -191,17 +191,23 @@ LockProcess::~LockProcess()
     }
 }
 
-static int sigterm_pipe[2];
+static int signal_pipe[2];
 
 static void sigterm_handler(int)
 {
-    char tmp;
-    ::write( sigterm_pipe[1], &tmp, 1);
+    char tmp = 'T';
+    ::write( signal_pipe[1], &tmp, 1);
+}
+
+static void sighup_handler(int)
+{
+    char tmp = 'H';
+    ::write( signal_pipe[1], &tmp, 1);
 }
 
 void LockProcess::timerEvent(QTimerEvent *ev)
 {
-	if (ev->timerId() == mAutoLogoutTimerId)
+	if (mAutoLogout && ev->timerId() == mAutoLogoutTimerId)
 	{
 		killTimer(mAutoLogoutTimerId);
 		AutoLogout autologout(this);
@@ -211,13 +217,7 @@ void LockProcess::timerEvent(QTimerEvent *ev)
 
 void LockProcess::setupSignals()
 {
-    // ignore SIGHUP
     struct sigaction act;
-    act.sa_handler=SIG_IGN;
-    sigemptyset(&(act.sa_mask));
-    sigaddset(&(act.sa_mask), SIGHUP);
-    act.sa_flags = 0;
-    sigaction(SIGHUP, &act, 0L);
     // ignore SIGINT
     act.sa_handler=SIG_IGN;
     sigemptyset(&(act.sa_mask));
@@ -236,17 +236,30 @@ void LockProcess::setupSignals()
     sigaddset(&(act.sa_mask), SIGTERM);
     act.sa_flags = 0;
     sigaction(SIGTERM, &act, 0L);
+    // SIGHUP forces lock
+    act.sa_handler= sighup_handler;
+    sigemptyset(&(act.sa_mask));
+    sigaddset(&(act.sa_mask), SIGHUP);
+    act.sa_flags = 0;
+    sigaction(SIGHUP, &act, 0L);
 
-    pipe(sigterm_pipe);
-    QSocketNotifier* notif = new QSocketNotifier(sigterm_pipe[0],
+    pipe(signal_pipe);
+    QSocketNotifier* notif = new QSocketNotifier(signal_pipe[0],
 	QSocketNotifier::Read, this );
-    connect( notif, SIGNAL(activated(int)), SLOT(sigtermPipeSignal()));
+    connect( notif, SIGNAL(activated(int)), SLOT(signalPipeSignal()));
 }
 
 
-void LockProcess::sigtermPipeSignal()
+void LockProcess::signalPipeSignal()
 {
-    quitSaver();
+    char tmp;
+    ::read( signal_pipe[0], &tmp, 1);
+    if( tmp == 'T' )
+        quitSaver();
+    else if( tmp == 'H' ) {
+        if( !mLocked )
+            startLock();
+    }
 }
 
 //---------------------------------------------------------------------------
@@ -498,14 +511,15 @@ void LockProcess::saveVRoot()
       Atom actual_type;
       int actual_format;
       unsigned long nitems, bytesafter;
-      Window *newRoot = (Window *)0;
+      unsigned char *newRoot = 0;
 
       if ((XGetWindowProperty(qt_xdisplay(), children[i], gXA_VROOT, 0, 1,
           False, XA_WINDOW, &actual_type, &actual_format, &nitems, &bytesafter,
-          (unsigned char **) &newRoot) == Success) && newRoot)
+          &newRoot) == Success) && newRoot)
       {
         gVRoot = children[i];
-        gVRootData = *newRoot;
+        Window *dummy = (Window*)newRoot;
+        gVRootData = *dummy;
         XFree ((char*) newRoot);
         break;
       }
@@ -860,7 +874,9 @@ void LockProcess::resume()
 //
 bool LockProcess::checkPass()
 {
-    killTimer(mAutoLogoutTimerId);
+    if (mAutoLogout)
+        killTimer(mAutoLogoutTimerId);
+
     PasswordDlg passDlg( this, &greetPlugin);
 
     int ret = execDialog( &passDlg );
