@@ -521,6 +521,7 @@ int Minicli::runCommand()
             }
           }
         }
+        // fall-through to shell case
         case KURIFilterData::SHELL:
         {
           if (kapp->authorize("shell_access"))
@@ -858,8 +859,12 @@ void Minicli::slotPriority(int priority)
 
 QString Minicli::calculate(const QString &exp)
 {
-   QString result;
-   QString cmd = QString("echo $((%1))").arg(exp);
+   QString result, cmd;
+   const QString bc = KStandardDirs::findExe("bc");
+   if ( !bc.isEmpty() )
+      cmd = QString("echo %1 | %2").arg(KProcess::quote(QString("scale=8; ")+exp), KProcess::quote(bc));
+   else
+      cmd = QString("echo $((%1))").arg(exp);
    FILE *fs = popen(QFile::encodeName(cmd).data(), "r");
    if (fs)
    {
