@@ -191,8 +191,6 @@ closeCtrl( struct display *d )
 		unlink( cr->path );
 		*strrchr( cr->path, '/' ) = 0;
 		rmdir( cr->path );
-		*strrchr( cr->path, '/' ) = 0;
-		rmdir( cr->path );
 		free( cr->path );
 		cr->path = 0;
 		while (cr->css) {
@@ -206,8 +204,6 @@ closeCtrl( struct display *d )
 		CloseNClearCloseOnFork( cr->fifo.fd );
 		cr->fifo.fd = -1;
 		unlink( cr->fpath );
-		*strrchr( cr->fpath, '/' ) = 0;
-		rmdir( cr->fpath );
 		free( cr->fpath );
 		cr->fpath = 0;
 		if (cr->fifo.buffer)
@@ -770,6 +766,9 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 			case BO_NOMAN:
 				fLog( d, fd, "notsup", "boot options unavailable" );
 				goto bust;
+			case BO_IO:
+				fLog( d, fd, "io", "io error" );
+				goto bust;
 			}
 			Reply( "ok\t" );
 			for (i = 0; opts[i]; i++) {
@@ -843,7 +842,7 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 					break;
 				case remoteLogin:
 					if (di->serverPid != -1 && nuke)
-						TerminateProcess( di->serverPid, d->termSignal );
+						TerminateProcess( di->serverPid, di->termSignal );
 					break;
 				case reserve:
 					di->status = notRunning;
