@@ -26,6 +26,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ******************************************************************/
 
 #include <qcursor.h>
+#include <qlayout.h>
+#include <qpopupmenu.h>
 #include <qtimer.h>
 
 #include <dcopclient.h>
@@ -41,7 +43,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kiconloader.h>
 #include <kwin.h>
 
-#include "hidebutton.h"
+#include "simplebutton.h"
 
 #include "systemtrayapplet.h"
 #include "systemtrayapplet.moc"
@@ -61,17 +63,18 @@ extern "C"
 SystemTrayApplet::SystemTrayApplet(const QString& configFile, Type type, int actions,
                                    QWidget *parent, const char *name)
   : KPanelApplet(configFile, type, actions, parent, name),
-    showFrame(false),
-    hideFrameTimer(0),
+    m_showFrame(false),
     m_showHidden(false),
     m_expandButton(0),
     m_settingsDialog(0),
-    m_iconSelector(0)
+    m_iconSelector(0),
+    m_autoRetractTimer(0),
+    m_autoRetract(false),
+    m_iconSize(24),
+    m_layout(0)
 {
     loadSettings();
 
-    m_shownWins.setAutoDelete(true);
-    m_hiddenWins.setAutoDelete(true);
     setBackgroundOrigin(AncestorOrigin);
 
     kwin_module = new KWinModule(this);
@@ -116,8 +119,8 @@ void SystemTrayApplet::initialize()
 
     Display *display = qt_xdisplay();
 
-    net_system_tray_selection = XInternAtom( display, trayatom, FALSE );
-    net_system_tray_opcode = XInternAtom( display, "_NET_SYSTEM_TRAY_OPCODE", FALSE );
+    net_system_tray_selection = XInternAtom(display, trayatom, false);
+    net_system_tray_opcode = XInternAtom(display, "_NET_SYSTEM_TRAY_OPCODE", false);
 
     // Acquire system tray
     XSetSelectionOwner(display,
@@ -148,8 +151,20 @@ void SystemTrayApplet::initialize()
 
 SystemTrayApplet::~SystemTrayApplet()
 {
-    m_shownWins.clear();
-    m_hiddenWins.clear();
+    for (TrayEmbedList::const_iterator it = m_hiddenWins.constBegin();
+         it != m_hiddenWins.constEnd();
+         ++it)
+    {
+        delete *it;
+    }
+
+    for (TrayEmbedList::const_iterator it = m_shownWins.constBegin();
+         it != m_shownWins.constEnd();
+         ++it)
+    {
+        delete *it;
+    }
+
     KGlobal::locale()->removeCatalogue("ksystemtrayapplet");
 }
 
@@ -165,40 +180,12 @@ bool SystemTrayApplet::x11Event( XEvent *e )
                 return true;
             embedWindow( e->xclient.data.l[2], false );
             layoutTray();
-            emit updateLayout();
             return true;
         }
     }
     return KPanelApplet::x11Event( e ) ;
 }
 
-void SystemTrayApplet::enterEvent( QEvent * )
-{
-    /*
-     * NOTE: This doesn't actually work as one would expect.  Since system tray
-     * icons are actually separate windows (or something like that) enter and
-     * leave events are triggered when the mouse goes over the icons.
-     *
-     * This makes it possible to "skip" over the window (i.e. leave the system
-     * tray while a popup menu from one of the systray icons is active) and no
-     * leave event will be sent, thus leaving the border shown.
-     */
-
-    if (showFrame && (frameStyle() == NoFrame))
-    {
-        setFrameStyle( Panel|Sunken );
-        hideFrameTimer->start(100, true);
-    }
-}
-
-void SystemTrayApplet::leaveEvent( QEvent * )
-{
-    if (showFrame)
-    {
-        checkFrameVisibility();
-    }
-}
-
 void SystemTrayApplet::preferences()
 {
     if (m_settingsDialog)
@@ -290,31 +277,31 @@ void SystemTrayApplet::applySettings()
     conf->writeEntry("Hidden", m_hiddenIconList);
     conf->sync();
 
-    for (TrayEmbed* emb = m_shownWins.first(); emb != 0; )
+    TrayEmbedList::iterator it = m_shownWins.begin();
+    while (it != m_shownWins.end())
     {
-        if (shouldHide(emb->embeddedWinId()))
+        if (shouldHide((*it)->embeddedWinId()))
         {
-            emb = m_shownWins.take(); // Next item becomes current
-            m_hiddenWins.append(emb);
-            emb = m_shownWins.current();
+            m_hiddenWins.append(*it);
+            it = m_shownWins.erase(it);
         }
         else
         {
-            emb = m_shownWins.next();
+            ++it;
         }
     }
 
-    for (TrayEmbed* emb = m_hiddenWins.first(); emb != 0; )
+    it = m_hiddenWins.begin();
+    while (it != m_hiddenWins.end())
     {
-        if (!shouldHide(emb->embeddedWinId()))
+        if (!shouldHide((*it)->embeddedWinId()))
         {
-            emb = m_hiddenWins.take(); // Next item becomes current
-            m_shownWins.append(emb);
-            emb = m_hiddenWins.current();
+            m_shownWins.append(*it);
+            it = m_hiddenWins.erase(it);
         }
         else
         {
-            emb = m_hiddenWins.next();
+            ++it;
         }
     }
 
@@ -322,7 +309,39 @@ void SystemTrayApplet::applySettings()
 
     updateVisibleWins();
     layoutTray();
-    emit updateLayout();
+}
+
+void SystemTrayApplet::checkAutoRetract()
+{
+    if (!m_autoRetractTimer)
+    {
+        return;
+    }
+
+    if (!geometry().contains(mapFromGlobal(QCursor::pos())))
+    {
+        m_autoRetractTimer->stop();
+        if (m_autoRetract)
+        {
+            m_autoRetract = false;
+
+            if (m_showHidden)
+            {
+                retract();
+            }
+        }
+        else
+        {
+            m_autoRetract = true;
+            m_autoRetractTimer->start(2000, true);
+        }
+
+    }
+    else
+    {
+        m_autoRetract = false;
+        m_autoRetractTimer->start(250, true);
+    }
 }
 
 void SystemTrayApplet::showExpandButton(bool show)
@@ -331,20 +350,28 @@ void SystemTrayApplet::showExpandButton(bool show)
     {
         if (!m_expandButton)
         {
-            m_expandButton = new HideButton(this, "expandButton");
+            m_expandButton = new SimpleButton(this, "expandButton");
+            m_expandButton->installEventFilter(this);
             refreshExpandButton();
 
             if (orientation() == Vertical)
             {
-                int iconSize = (m_expandButton->pixmap() ? m_expandButton->pixmap()->height() : 16);
-                m_expandButton->setFixedSize(width() - 4, iconSize);
+                m_expandButton->setFixedSize(width() - 4,
+                                             m_expandButton->sizeHint()
+                                                            .height());
             }
             else
             {
-                int iconSize = (m_expandButton->pixmap() ? m_expandButton->pixmap()->width() : 100);
-                m_expandButton->setFixedSize(iconSize, height() - 4);
+                m_expandButton->setFixedSize(m_expandButton->sizeHint()
+                                                            .width(),
+                                             height() - 4);
             }
-            connect(m_expandButton, SIGNAL(clicked()), this, SLOT(toggleExpanded()));
+            connect(m_expandButton, SIGNAL(clicked()),
+                    this, SLOT(toggleExpanded()));
+
+            m_autoRetractTimer = new QTimer(this);
+            connect(m_autoRetractTimer, SIGNAL(timeout()),
+                    this, SLOT(checkAutoRetract()));
         }
         else
         {
@@ -359,55 +386,31 @@ void SystemTrayApplet::showExpandButton(bool show)
     }
 }
 
-void SystemTrayApplet::orientationChange( Orientation /*orientation*/ ) 
+void SystemTrayApplet::orientationChange( Orientation /*orientation*/ )
 {
     refreshExpandButton();
 }
 
-void SystemTrayApplet::checkFrameVisibility()
-{
-    if (!geometry().contains( mapFromGlobal( QCursor::pos() ) ))
-    {
-        if (frameStyle() != NoFrame)
-        {
-            setFrameStyle( NoFrame );
-        }
-
-        hideFrameTimer->stop();
-    }
-    else
-    {
-        hideFrameTimer->start(100, true);
-    }
-}
-
 void SystemTrayApplet::loadSettings()
 {
     // set our defaults
     setFrameStyle(NoFrame);
-    showFrame = false;
-    delete hideFrameTimer; hideFrameTimer = 0;
+    m_showFrame = false;
 
     KConfig *conf = config();
     conf->setGroup("General");
-    KConfigGroup globalConf(KGlobal::config(), "General");
 
     if (conf->readBoolEntry("ShowPanelFrame", false))
     {
         setFrameStyle(Panel | Sunken);
     }
-    else if ((globalConf.readBoolEntry("FadeOutAppletHandles", false) ||
-             globalConf.readBoolEntry("HideAppletHandles", false)) &&
-             !conf->readBoolEntry("HideTrayFrame", false))
-    {
-        // only do the "show on hover" trick with the frame when we DON'T have handles
-        showFrame = true;
-        hideFrameTimer = new QTimer(this);
-        connect(hideFrameTimer, SIGNAL(timeout()), SLOT(checkFrameVisibility()));
-    }
 
     conf->setGroup("HiddenTrayIcons");
     m_hiddenIconList = conf->readListEntry("Hidden");
+
+    //Note This setting comes from kdeglobal.
+    conf->setGroup("System Tray");
+    m_iconSize = conf->readNumEntry("systrayIconWidth", 22);
 }
 
 void SystemTrayApplet::systemTrayWindowAdded( WId w )
@@ -421,12 +424,10 @@ void SystemTrayApplet::systemTrayWindowAdded( WId w )
     embedWindow(w, true);
     updateVisibleWins();
     layoutTray();
-    emit updateLayout();
 
-    if (showFrame && frameStyle() == NoFrame)
+    if (m_showFrame && frameStyle() == NoFrame)
     {
         setFrameStyle(Panel|Sunken);
-        hideFrameTimer->start(100, true);
     }
 }
 
@@ -456,7 +457,8 @@ void SystemTrayApplet::embedWindow( WId w, bool kde_tray )
     }
 
     connect(emb, SIGNAL(embeddedWindowDestroyed()), SLOT(updateTrayWindows()));
-    emb->resize(24, 24);
+    emb->setMinimumSize(m_iconSize, m_iconSize);
+
     if (shouldHide(w))
     {
         emb->hide();
@@ -539,7 +541,10 @@ void SystemTrayApplet::refreshExpandButton()
         return;
     }
 
-    if (orientation() == Vertical)
+    Qt::Orientation o = orientation();
+    m_expandButton->setOrientation(o);
+
+    if (o == Vertical)
     {
         m_expandButton->setPixmap(m_showHidden ?
             kapp->iconLoader()->loadIcon("1downarrow", KIcon::Panel, 16) :
@@ -560,58 +565,71 @@ void SystemTrayApplet::expand()
 
     updateVisibleWins();
     layoutTray();
-    emit updateLayout();
+
+    if (m_autoRetractTimer)
+    {
+        m_autoRetractTimer->start(250, true);
+    }
 }
 
 void SystemTrayApplet::retract()
 {
+    if (m_autoRetractTimer)
+    {
+        m_autoRetractTimer->stop();
+    }
+
     m_showHidden = false;
     refreshExpandButton();
 
     updateVisibleWins();
     layoutTray();
-    emit updateLayout();
 }
 
 void SystemTrayApplet::updateTrayWindows()
 {
-    TrayEmbed* emb = m_shownWins.first();
-    while ((emb = m_shownWins.current()) != 0)
+    TrayEmbedList::iterator emb = m_shownWins.begin();
+    while (emb != m_shownWins.end())
     {
-        WId wid = emb->embeddedWinId();
+        WId wid = (*emb)->embeddedWinId();
         if ((wid == 0) ||
-            (emb->kdeTray() && !kwin_module->systemTrayWindows().contains(wid)))
+            ((*emb)->kdeTray() &&
+             !kwin_module->systemTrayWindows().contains(wid)))
         {
-//            emb->setMapAfterRelease( false ); // don't map, they went away on their own
-            m_shownWins.remove(emb);
+            (*emb)->deleteLater();
+            emb = m_shownWins.erase(emb);
         }
         else
         {
-            m_shownWins.next();
+            ++emb;
         }
     }
 
-    emb = m_hiddenWins.first();
-    while ((emb = m_hiddenWins.current()) != 0L)
+    emb = m_hiddenWins.begin();
+    while (emb != m_hiddenWins.end())
     {
-        WId wid = emb->embeddedWinId();
+        WId wid = (*emb)->embeddedWinId();
         if ((wid == 0) ||
-            (emb->kdeTray() && !kwin_module->systemTrayWindows().contains(wid)))
-            m_hiddenWins.remove(emb);
+            ((*emb)->kdeTray() &&
+             !kwin_module->systemTrayWindows().contains(wid)))
+        {
+            (*emb)->deleteLater();
+            emb = m_hiddenWins.erase(emb);
+        }
         else
-            m_hiddenWins.next();
+        {
+            ++emb;
+        }
     }
 
     showExpandButton(!m_hiddenWins.isEmpty());
-
     updateVisibleWins();
     layoutTray();
-    emit updateLayout();
 }
 
 int SystemTrayApplet::maxIconWidth() const
 {
-    int largest = 24;
+    int largest = m_iconSize;
 
     TrayEmbedList::const_iterator lastEmb = m_shownWins.end();
     for (TrayEmbedList::const_iterator emb = m_shownWins.begin(); emb != lastEmb; ++emb)
@@ -621,7 +639,7 @@ int SystemTrayApplet::maxIconWidth() const
             continue;
         }
 
-        int width = (*emb)->sizeHint().width();
+        int width = (*emb)->width();
         if (width > largest)
         {
             largest = width;
@@ -633,12 +651,7 @@ int SystemTrayApplet::maxIconWidth() const
         lastEmb = m_hiddenWins.end();
         for (TrayEmbedList::const_iterator emb = m_hiddenWins.begin(); emb != lastEmb; ++emb)
         {
-            if (*emb == 0)
-            {
-                continue;
-            }
-
-            int width = (*emb)->sizeHint().width();
+            int width = (*emb)->width();
             if (width > largest)
             {
                 largest = width;
@@ -651,7 +664,7 @@ int SystemTrayApplet::maxIconWidth() const
 
 int SystemTrayApplet::maxIconHeight() const
 {
-    int largest = 24;
+    int largest = m_iconSize;
 
     TrayEmbedList::const_iterator lastEmb = m_shownWins.end();
     for (TrayEmbedList::const_iterator emb = m_shownWins.begin(); emb != m_shownWins.end(); ++emb)
@@ -661,7 +674,7 @@ int SystemTrayApplet::maxIconHeight() const
             continue;
         }
 
-        int height = (*emb)->sizeHint().height();
+        int height = (*emb)->height();
         if (height > largest)
         {
             largest = height;
@@ -678,7 +691,7 @@ int SystemTrayApplet::maxIconHeight() const
                 continue;
             }
 
-            int height = (*emb)->sizeHint().height();
+            int height = (*emb)->height();
             if (height > largest)
             {
                 largest = height;
@@ -689,70 +702,79 @@ int SystemTrayApplet::maxIconHeight() const
     return largest;
 }
 
-int SystemTrayApplet::widthForHeight(int h) const
+bool SystemTrayApplet::eventFilter(QObject* watched, QEvent* e)
 {
-    int iconWidth = maxIconWidth(), iconHeight = maxIconHeight();
-    int iconCount = m_shownWins.count();
-
-    if (m_showHidden)
+    if (watched == m_expandButton)
     {
-        iconCount += m_hiddenWins.count();
-    }
-
-    if (h < iconHeight)
-    {
-        // avoid div by 0 later
-        h = iconHeight;
-    }
-
-    int ret = 0;
-
-    if (iconCount > 0)
-    {
-        ret = (((iconCount - 1) / (h / iconHeight)) + 1) * iconWidth + 4;
-
-        if (ret < iconWidth + 4)
+        QPoint p;
+        if (e->type() == QEvent::ContextMenu)
         {
-            ret = 0;
+            p = static_cast<QContextMenuEvent*>(e)->globalPos();
+        }
+        else if (e->type() == QEvent::MouseButtonPress)
+        {
+            QMouseEvent* me = static_cast<QMouseEvent*>(e);
+            if (me->button() == Qt::RightButton)
+            {
+                p = me->globalPos();
+            }
+        }
+
+        if (!p.isNull())
+        {
+            QPopupMenu* contextMenu = new QPopupMenu(this);
+            contextMenu->insertItem(SmallIcon("configure"), i18n("Configure System Tray..."),
+                                    this, SLOT(configure()));
+
+            contextMenu->exec(static_cast<QContextMenuEvent*>(e)->globalPos());
+
+            delete contextMenu;
+            return true;
         }
     }
 
-    if (m_expandButton && m_expandButton->isVisibleTo(const_cast<SystemTrayApplet*>(this)))
+    return false;
+}
+
+int SystemTrayApplet::widthForHeight(int h) const
+{
+    if (orientation() == Qt::Vertical)
     {
-        ret += m_expandButton->width() + 2;
+        return width();
     }
 
-    return ret;
+    int currentHeight = height();
+    int minHeight = m_iconSize + 4;
+    if (QABS(currentHeight-h) >= 4 && currentHeight != minHeight)
+    {
+        SystemTrayApplet* me = const_cast<SystemTrayApplet*>(this);
+        me->setMinimumSize(0, 0);
+        me->setMaximumSize(32767, 32767);
+        me->setFixedHeight(h);
+    }
+
+    return sizeHint().width();
 }
 
 int SystemTrayApplet::heightForWidth(int w) const
 {
-    int iconWidth = maxIconWidth(), iconHeight = maxIconHeight();
-    int iconCount = m_shownWins.count();
-
-    if (m_showHidden)
+    if (orientation() == Qt::Horizontal)
     {
-        iconCount += m_hiddenWins.count();
+        return height();
     }
 
-    if (w < iconWidth)
+    int currentWidth = width();
+    int minSize = m_iconSize + 4;
+    if (QABS(currentWidth - w) >= 4 && currentWidth != minSize)
     {
-        // avoid div by 0 later
-        w = iconWidth;
-    }
-    int ret = (((iconCount - 1) / (w / iconWidth)) + 1) * iconHeight + 4;
-
-    if (ret < iconHeight + 4)
-    {
-        ret = 0;
+        SystemTrayApplet* me = const_cast<SystemTrayApplet*>(this);
+        me->setMinimumSize(0, 0);
+        me->setMaximumSize(32767, 32767);
+        //kdDebug() << "min/max size is " << me->minimumSize() << ", " << me->maximumSize() << endl;
+        me->setFixedWidth(w);
     }
 
-    if (m_expandButton && m_expandButton->isVisibleTo(const_cast<SystemTrayApplet*>(this)))
-    {
-        ret += m_expandButton->height() + 2;
-    }
-
-    return ret;
+    return sizeHint().height();
 }
 
 void SystemTrayApplet::resizeEvent( QResizeEvent* )
@@ -761,21 +783,25 @@ void SystemTrayApplet::resizeEvent( QResizeEvent* )
     {
         if (orientation() == Vertical)
         {
-            int iconSize = (m_expandButton->pixmap() ? m_expandButton->pixmap()->height() : 16);
-            m_expandButton->setFixedSize(width() - 4, iconSize);
+            m_expandButton->setFixedSize(width() - 4,
+                                         m_expandButton->sizeHint().height());
         }
         else
         {
-            int iconSize = (m_expandButton->pixmap() ? m_expandButton->pixmap()->width() : 16);
-            m_expandButton->setFixedSize(iconSize, height() - 4);
+            m_expandButton->setFixedSize(m_expandButton->sizeHint().width(),
+                                         height() - 4);
         }
     }
 
     layoutTray();
+    // we need to give ourselves a chance to adjust our size before calling this
+    QTimer::singleShot(0, this, SIGNAL(updateLayout()));
 }
 
 void SystemTrayApplet::layoutTray()
 {
+    setUpdatesEnabled(false);
+
     int iconCount = m_shownWins.count();
 
     if (m_showHidden)
@@ -783,76 +809,89 @@ void SystemTrayApplet::layoutTray()
         iconCount += m_hiddenWins.count();
     }
 
-    if (iconCount == 0)
-    {
-        return;
-    }
-
     /* heightWidth = height or width in pixels (depends on orientation())
      * nbrOfLines = number of rows or cols (depends on orientation())
-     * spacing = size of spacing in pixels between lines (rows or cols)
      * line = what line to draw an icon in */
-    int i = 0, line, spacing, nbrOfLines, heightWidth;
-    int iconWidth = maxIconWidth(), iconHeight = maxIconHeight();
+    int i = 0, line, nbrOfLines, heightWidth;
+    bool showExpandButton = m_expandButton && m_expandButton->isVisibleTo(this);
+    delete m_layout;
+    m_layout = new QGridLayout(this, 1, 1, 2, 2);
 
     // col = column or row, depends on orientation(),
     // the opposite direction of line
     int col = 0;
 
-    if (m_expandButton && m_expandButton->isVisibleTo(this) && !kapp->reverseLayout())
-    {
-        m_expandButton->move(2, 2);
-        if (orientation() == Vertical) 
-            col += m_expandButton->height() + 2;
-        else 
-            col += m_expandButton->width() + 2;
-    }
-
     if (orientation() == Vertical)
     {
+        int iconWidth = maxIconWidth() + 2; // +2 for the margins that implied by the layout
         heightWidth = width();
         // to avoid nbrOfLines=0 we ensure heightWidth >= iconWidth!
         heightWidth = heightWidth < iconWidth ? iconWidth : heightWidth;
         nbrOfLines = heightWidth / iconWidth;
-        spacing = (heightWidth - iconWidth*nbrOfLines) / (nbrOfLines + 1);
+
+        if (showExpandButton)
+        {
+            m_layout->addMultiCellWidget(m_expandButton,
+                                         col, col,
+                                         0, nbrOfLines,
+                                         Qt::AlignHCenter | Qt::AlignVCenter);
+            ++col;
+        }
 
         if (m_showHidden)
         {
             TrayEmbedList::const_iterator lastEmb = m_hiddenWins.end();
-            for (TrayEmbedList::const_iterator emb = m_hiddenWins.begin(); emb != lastEmb; ++emb)
+            for (TrayEmbedList::const_iterator emb = m_hiddenWins.begin();
+                 emb != lastEmb; ++emb)
             {
                 line = i % nbrOfLines;
                 (*emb)->hide();
                 (*emb)->show();
-                (*emb)->move(spacing*(line+1) + line*iconWidth, 2 + col);
+                m_layout->addWidget(*emb, col, line,
+                                    Qt::AlignHCenter | Qt::AlignVCenter);
+
                 if (line + 1 == nbrOfLines)
                 {
-                    col += iconHeight;
+                    ++col;
                 }
-                i++;
+
+                ++i;
             }
         }
 
         TrayEmbedList::const_iterator lastEmb = m_shownWins.end();
-        for (TrayEmbedList::const_iterator emb = m_shownWins.begin(); emb != lastEmb; ++emb)
+        for (TrayEmbedList::const_iterator emb = m_shownWins.begin();
+             emb != lastEmb; ++emb)
         {
             line = i % nbrOfLines;
             (*emb)->hide();
             (*emb)->show();
-            (*emb)->move(spacing*(line+1) + line*iconWidth, 2 + col);
+            m_layout->addWidget(*emb, col, line,
+                                Qt::AlignHCenter | Qt::AlignVCenter);
+
             if (line + 1 == nbrOfLines)
             {
-                col += iconHeight;
+                ++col;
             }
-            i++;
+
+            ++i;
         }
     }
-    else
+    else // horizontal
     {
+        int iconHeight = maxIconHeight() + 2; // +2 for the margins that implied by the layout
         heightWidth = height();
         heightWidth = heightWidth < iconHeight ? iconHeight : heightWidth; // to avoid nbrOfLines=0
         nbrOfLines = heightWidth / iconHeight;
-        spacing = (heightWidth - iconHeight*nbrOfLines) / (nbrOfLines + 1);
+
+        if (showExpandButton)
+        {
+            m_layout->addMultiCellWidget(m_expandButton,
+                                         0, nbrOfLines - 1,
+                                         0, 0,
+                                         Qt::AlignHCenter | Qt::AlignVCenter);
+            col = 1;
+        }
 
         if (m_showHidden)
         {
@@ -862,38 +901,38 @@ void SystemTrayApplet::layoutTray()
                 line = i % nbrOfLines;
                 (*emb)->hide();
                 (*emb)->show();
-                (*emb)->move(2 + col, spacing*(line+1) + line*iconHeight);
+                m_layout->addWidget(*emb, line, col,
+                                    Qt::AlignHCenter | Qt::AlignVCenter);
+
                 if (line + 1 == nbrOfLines)
                 {
-                    col += iconWidth;
+                    ++col;
                 }
-                i++;
+
+                ++i;
             }
         }
 
         TrayEmbedList::const_iterator lastEmb = m_shownWins.end();
-        for (TrayEmbedList::const_iterator emb = m_shownWins.begin(); emb != lastEmb; ++emb)
+        for (TrayEmbedList::const_iterator emb = m_shownWins.begin();
+             emb != lastEmb; ++emb)
         {
             line = i % nbrOfLines;
             (*emb)->hide();
             (*emb)->show();
-            (*emb)->move(2 + col, spacing*(line+1) + line*iconHeight);
+            m_layout->addWidget(*emb, line, col,
+                                Qt::AlignHCenter | Qt::AlignVCenter);
 
             if (line + 1 == nbrOfLines)
             {
-                col += iconWidth;
+                ++col;
             }
 
-            i++;
+            ++i;
         }
     }
 
-    if (m_expandButton && m_expandButton->isVisibleTo(this) && kapp->reverseLayout())
-    {
-        m_expandButton->move(width() - m_expandButton->width() - 2, 2);
-        col++;
-    }
-
+    setUpdatesEnabled(true);
     updateGeometry();
 }
 
