@@ -685,6 +685,10 @@ void ExtensionContainer::unhideTriggered(UnhideTrigger::Trigger tr, int Xinerama
             autoHide(false);
             maybeStartAutoHideTimer();
         }
+        else if (m_hideMode == BackgroundHide)
+        {
+            KWin::raiseWindow(winId());
+        }
     }
 }
 
@@ -983,48 +987,31 @@ void ExtensionContainer::updateWindowManager()
             h = height();
         }
 
-        //FIXME do not set the strut if the edge we are placing ourselves along is not also an edge
-        //of the virual screen, since NETWM has no way to represent the resulting non-rectangular
-        //client areas that can result in an Xinerama environment.
-        //Remove these checks if NetWM ever gets smarter about this (either struts that don't take
-        //up an entire edge or per-screen struts
         switch (position())
         {
             case KPanelExtension::Top:
-                if (screenRect.top() == virtRect.top())
-                {
-                    strut.top_width = geom.y() + h;
-                    strut.top_start = x();
-                    strut.top_end = x() + width() - 1;
-                }
+                strut.top_width = geom.y() + h;
+                strut.top_start = x();
+                strut.top_end = x() + width() - 1;
                 break;
 
             case KPanelExtension::Bottom:
-                if (screenRect.bottom() == virtRect.bottom())
-                {
-                    // also claim the non-visible part at the bottom
-                    strut.bottom_width = (virtRect.bottom() - geom.bottom()) + h;
-                    strut.bottom_start = x();
-                    strut.bottom_end = x() + width() - 1;
-                }
+                // also claim the non-visible part at the bottom
+                strut.bottom_width = (virtRect.bottom() - geom.bottom()) + h;
+                strut.bottom_start = x();
+                strut.bottom_end = x() + width() - 1;
                 break;
 
             case KPanelExtension::Right:
-                if (screenRect.right() == virtRect.right())
-                {
-                    strut.right_width = (virtRect.right() - geom.right()) + w;
-                    strut.right_start = y();
-                    strut.right_end = y() + height() - 1;
-                }
+                strut.right_width = (virtRect.right() - geom.right()) + w;
+                strut.right_start = y();
+                strut.right_end = y() + height() - 1;
                 break;
 
             case KPanelExtension::Left:
-                if(screenRect.left() == virtRect.left())
-                {
-                    strut.left_width = geom.x() + w;
-                    strut.left_start = y();
-                    strut.left_end = y() + height() - 1;
-                }
+                strut.left_width = geom.x() + w;
+                strut.left_start = y();
+                strut.left_end = y() + height() - 1;
                 break;
 
             case KPanelExtension::Floating:
@@ -1125,7 +1112,7 @@ void ExtensionContainer::blockUserInput( bool block )
 
 void ExtensionContainer::maybeStartAutoHideTimer()
 {
-    if (m_hideMode == AutomaticHide &&
+    if (m_hideMode != ManualHide &&
         !_autoHidden &&
         !_userHidden)
     {
@@ -1760,12 +1747,21 @@ QPoint ExtensionContainer::initialLocation(KPanelExtension::Position p,
             break;
 
             case KPanelExtension::Center:
+            {
                 left = wholeScreen.left() + ( wholeScreen.width() - s.width() ) / 2;
+                int right = left + s.width();
+                if (right > workArea.right())
+                {
+                    left = left - (right - workArea.right());
+                }
+
                 if (left < workArea.left())
                 {
                     left = workArea.left();
                 }
+
             break;
+            }
 
             case KPanelExtension::RightBottom:
                 left = workArea.right() - s.width() + 1;
@@ -1796,12 +1792,20 @@ QPoint ExtensionContainer::initialLocation(KPanelExtension::Position p,
             break;
 
             case KPanelExtension::Center:
+            {
                 top = wholeScreen.top() + ( wholeScreen.height() - s.height() ) / 2;
+                int bottom = top + s.height();
+                if (bottom > workArea.bottom())
+                {
+                    top = top - (bottom - workArea.bottom());
+                }
+
                 if (top < workArea.top())
                 {
                     top = workArea.top();
                 }
             break;
+            }
 
             case KPanelExtension::RightBottom:
                 top = workArea.bottom() - s.height() + 1;
