@@ -37,6 +37,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kdebug.h>
 #include <kglobal.h>
 #include <kglobalaccel.h>
+#include <kimageeffect.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 
@@ -53,8 +54,12 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 TaskBar::TaskBar( QWidget *parent, const char *name )
     : Panner( parent, name ),
+      m_showAllWindows(false),
       m_currentScreen(-1),
       m_showOnlyCurrentScreen(false),
+      m_sortByDesktop(false),
+      m_showIcon(false),
+      m_showOnlyIconified(false),
       m_textShadowEngine(0),
       m_ignoreUpdates(false)
 {
@@ -125,11 +130,18 @@ TaskBar::TaskBar( QWidget *parent, const char *name )
 
 TaskBar::~TaskBar()
 {
+    for (TaskContainer::Iterator it = m_hiddenContainers.begin();
+         it != m_hiddenContainers.end();
+         ++it)
+    {
+        (*it)->deleteLater();
+    }
+
     for (TaskContainer::List::const_iterator it = containers.constBegin();
          it != containers.constEnd();
          ++it)
     {
-        delete *it;
+        (*it)->deleteLater();
     }
 
     for (PixmapList::const_iterator it = frames.constBegin();
@@ -138,32 +150,49 @@ TaskBar::~TaskBar()
     {
         delete *it;
     }
+
+    delete m_textShadowEngine;
+    m_textShadowEngine = 0;
 }
 
 QSize TaskBar::sizeHint() const
 {
-    return QSize(BUTTON_MIN_WIDTH, TaskBarSettings::minimumButtonHeight());
+    // get our minimum height based on the minimum button height or the
+    // height of the font in use, which is largest
+    QFontMetrics fm(KGlobalSettings::taskbarFont());
+    int minButtonHeight = fm.height() > TaskBarSettings::minimumButtonHeight() ?
+                          fm.height() : TaskBarSettings::minimumButtonHeight();
+
+    return QSize(BUTTON_MIN_WIDTH, minButtonHeight);
 }
 
 QSize TaskBar::sizeHint( KPanelExtension::Position p, QSize maxSize) const
 {
+    // get our minimum height based on the minimum button height or the
+    // height of the font in use, which is largest
+    QFontMetrics fm(KGlobalSettings::taskbarFont());
+    int minButtonHeight = fm.height() > TaskBarSettings::minimumButtonHeight() ?
+                          fm.height() : TaskBarSettings::minimumButtonHeight();
+
     if ( p == KPanelExtension::Left || p == KPanelExtension::Right )
     {
-        int actualMax = BUTTON_HEIGHT * containerCount();
+        int actualMax = minButtonHeight * containerCount();
+
+        if (containerCount() == 0)
+        {
+            actualMax = minButtonHeight;
+        }
 
         if (actualMax > maxSize.height())
         {
             return maxSize;
         }
-        else
-        {
-            return QSize( maxSize.width(), actualMax );
-        }
+        return QSize( maxSize.width(), actualMax );
     }
     else
     {
         int rows = KickerSettings::conserveSpace() ?
-                   contentsRect().height() / TaskBarSettings::minimumButtonHeight() :
+                   contentsRect().height() / minButtonHeight :
                    1;
         if ( rows < 1 )
         {
@@ -182,15 +211,16 @@ QSize TaskBar::sizeHint( KPanelExtension::Position p, QSize maxSize) const
         {
             actualMax += maxWidth;
         }
+        if (containerCount() == 0)
+        {
+            actualMax = maxWidth;
+        }
 
         if (actualMax > maxSize.width())
         {
            return maxSize;
         }
-        else
-        {
-           return QSize( actualMax, maxSize.height() );
-        }
+        return QSize( actualMax, maxSize.height() );
     }
 }
 
@@ -199,10 +229,12 @@ void TaskBar::configure()
     bool wasShowWindows = m_showAllWindows;
     bool wasSortByDesktop = m_sortByDesktop;
     bool wasShowIcon = m_showIcon;
+    bool wasShowOnlyIconified = m_showOnlyIconified;
 
     m_showAllWindows = TaskBarSettings::showAllWindows();
     m_sortByDesktop = m_showAllWindows && TaskBarSettings::sortByDesktop();
     m_showIcon = TaskBarSettings::showIcon();
+    m_showOnlyIconified = TaskBarSettings::showOnlyIconified();
 
     m_currentScreen = -1;    // Show all screens or re-get our screen
     m_showOnlyCurrentScreen = TaskBarSettings::showCurrentScreenOnly() &&
@@ -219,12 +251,13 @@ void TaskBar::configure()
                     this, SLOT(windowChangedGeometry(Task::Ptr)));
         connect(TaskManager::the(), SIGNAL(windowChangedGeometry(Task::Ptr)),
                  this, SLOT(windowChangedGeometry(Task::Ptr)));
-        TaskManager::the()->trackGeometry();
     }
+    TaskManager::the()->trackGeometry(m_showOnlyCurrentScreen);
 
     if (wasShowWindows != m_showAllWindows ||
         wasSortByDesktop != m_sortByDesktop ||
-        wasShowIcon != m_showIcon)
+        wasShowIcon != m_showIcon ||
+        wasShowOnlyIconified != m_showOnlyIconified)
     {
         // relevant settings changed, update our task containers
         for (TaskContainer::Iterator it = containers.begin();
@@ -339,7 +372,7 @@ void TaskBar::showTaskContainer(TaskContainer* container)
     }
 
     // try to place the container after one of the same app
-    if (TaskBarSettings::sortByApp() && !isGrouping)
+    if (TaskBarSettings::sortByApp())
     {
         TaskContainer::Iterator it = containers.begin();
         for (; it != containers.end(); ++it)
@@ -389,7 +422,8 @@ void TaskBar::remove(Task::Ptr task, TaskContainer* container)
     {
         if ((*it)->contains(task))
         {
-            delete *it;
+            (*it)->finish();
+            m_deletableContainers.append(*it);
             m_hiddenContainers.erase(it);
             break;
         }
@@ -425,7 +459,8 @@ void TaskBar::remove(Task::Ptr task, TaskContainer* container)
         }
 
         removeChild(container);
-        delete container;
+        container->finish();
+        m_deletableContainers.append(container);
 
         reLayoutEventually();
         emit containerCountChanged();
@@ -445,8 +480,15 @@ void TaskBar::remove(Startup::Ptr startup, TaskContainer* container)
     {
         if ((*it)->contains(startup))
         {
-            delete *it;
-            m_hiddenContainers.erase(it);
+            (*it)->remove(startup);
+
+            if ((*it)->isEmpty())
+            {
+                (*it)->finish();
+                m_deletableContainers.append(*it);
+                m_hiddenContainers.erase(it);
+            }
+
             break;
         }
     }
@@ -470,6 +512,12 @@ void TaskBar::remove(Startup::Ptr startup, TaskContainer* container)
         }
     }
 
+    container->remove(startup);
+    if (!container->isEmpty())
+    {
+        return;
+    }
+
     TaskContainer::List::iterator it = containers.find(container);
     if (it != containers.end())
     {
@@ -478,7 +526,8 @@ void TaskBar::remove(Startup::Ptr startup, TaskContainer* container)
 
     // startup containers only ever contain that one item. so
     // just delete the poor bastard.
-    delete container;
+    container->finish();
+    m_deletableContainers.append(container);
     reLayoutEventually();
     emit containerCountChanged();
 }
@@ -539,15 +588,12 @@ void TaskBar::windowChanged(Task::Ptr task)
 
     container->windowChanged(task);
 
-    if (!m_showAllWindows || TaskBarSettings::showOnlyIconified())
+    if (!m_showAllWindows || m_showOnlyIconified)
     {
         emit containerCountChanged();
     }
 
-    if (!blocklayout)
-    {
-        reLayoutEventually();
-    }
+    reLayoutEventually();
 }
 
 void TaskBar::windowChangedGeometry(Task::Ptr task)
@@ -597,6 +643,19 @@ void TaskBar::reLayoutEventually()
 
 void TaskBar::reLayout()
 {
+    // Because QPopupMenu::exec() creates its own event loop, deferred deletes
+    // via QObject::deleteLater() may be prematurely executed when a container's
+    // popup menu is visible.
+    //
+    // To get around this, we collect the containers and delete them manually
+    // when doing a relayout. (kling)
+    if (!m_deletableContainers.isEmpty()) {
+        TaskContainer::List::iterator it = m_deletableContainers.begin();
+        for (; it != m_deletableContainers.end(); ++it)
+            delete *it;
+        m_deletableContainers.clear();
+    }
+
     // filter task container list
     TaskContainer::List list = filteredContainers();
 
@@ -622,14 +681,23 @@ void TaskBar::reLayout()
     // up to use to initialize it. =(
     resizeContents(contentsRect().width(), contentsRect().height());
 
+    // number of rows simply depends on our height which is either the
+    // minimum button height or the height of the font in use, whichever is
+    // largest
+    QFontMetrics fm(KGlobalSettings::taskbarFont());
+    int minButtonHeight = fm.height() > TaskBarSettings::minimumButtonHeight() ?
+                          fm.height() : TaskBarSettings::minimumButtonHeight();
+
     // horizontal layout
     if (orientation() == Horizontal)
     {
         int bwidth = BUTTON_MIN_WIDTH;
+        int rows = contentsRect().height() / minButtonHeight;
 
-        // number of rows simply depends on our height
-        int rows = contentsRect().height() / TaskBarSettings::minimumButtonHeight();
-        if ( rows < 1 ) rows = 1;
+        if ( rows < 1 )
+        {
+            rows = 1;
+        }
 
         // actual button height
         int bheight = contentsRect().height() / rows;
@@ -697,9 +765,9 @@ void TaskBar::reLayout()
     else // vertical layout
     {
         // adjust content size
-        if (contentsRect().height() < (int)list.count() * BUTTON_HEIGHT)
+        if (contentsRect().height() < (int)list.count() * minButtonHeight)
         {
-            resizeContents(contentsRect().width(), list.count()*BUTTON_HEIGHT);
+            resizeContents(contentsRect().width(), list.count() * minButtonHeight);
         }
 
         // layout containers
@@ -711,10 +779,10 @@ void TaskBar::reLayout()
             TaskContainer* c = *it;
 
             c->setArrowType(arrowType);
-            c->resize(contentsRect().width(), BUTTON_HEIGHT);
+            c->resize(contentsRect().width(), minButtonHeight);
             c->show();
 
-            moveChild(c, 0, i * BUTTON_HEIGHT);
+            moveChild(c, 0, i * minButtonHeight);
             i++;
             c->update();
         }
@@ -827,8 +895,15 @@ int TaskBar::taskCount() const
 
 int TaskBar::maximumButtonsWithoutShrinking() const
 {
-    int rows = contentsRect().height() / TaskBarSettings::minimumButtonHeight();
-    if ( rows < 1 ) rows = 1;
+    QFontMetrics fm(KGlobalSettings::taskbarFont());
+    int minButtonHeight = fm.height() > TaskBarSettings::minimumButtonHeight() ?
+                          fm.height() : TaskBarSettings::minimumButtonHeight();
+    int rows = contentsRect().height() / minButtonHeight;
+
+    if (rows < 1)
+    {
+        rows = 1;
+    }
 
     if ( orientation() == Horizontal ) {
         // maxWidth of 0 means no max width, drop back to default
@@ -865,7 +940,8 @@ void TaskBar::reGroup()
          it != lastContainer;
          ++it)
     {
-        delete *it;
+        (*it)->finish();
+        m_deletableContainers.append(*it);
     }
     m_hiddenContainers.clear();
 
@@ -873,7 +949,8 @@ void TaskBar::reGroup()
          it != containers.constEnd();
          ++it)
     {
-        delete *it;
+        (*it)->finish();
+        m_deletableContainers.append(*it);
     }
     containers.clear();
 
@@ -910,7 +987,7 @@ TaskContainer::List TaskBar::filteredContainers()
     {
         TaskContainer* c = *it;
         if ((m_showAllWindows || c->onCurrentDesktop()) &&
-            (!TaskBarSettings::showOnlyIconified() || c->isIconified()) &&
+            (!m_showOnlyIconified || c->isIconified()) &&
             ((showScreen() == -1) || c->isOnScreen()))
         {
             list.append(c);
@@ -928,9 +1005,10 @@ TaskContainer::List TaskBar::filteredContainers()
 void TaskBar::activateNextTask(bool forward)
 {
     bool forcenext = false;
-    TaskContainer::List list = containers;
+    TaskContainer::List list = filteredContainers();
 
-    // TODO: figure out if/when/why this is necessary here
+    // this is necessary here, because 'containers' is unsorted and
+    // we want to iterate over the _shown_ task containers in a linear way
     if (m_sortByDesktop)
     {
         sortContainersByDesktop(list);
@@ -1036,8 +1114,7 @@ int TaskBar::showScreen() const
 }
 
 // taken from mtaskbar, by Sebastian Wolff
-void TaskBar::drawShadowText(QPainter  &p, QRect tr, int tf, const QString & str,
-                             int len, QRect * brect, QTextParag ** internal)
+void TaskBar::drawShadowText(QPainter &p, const QRect &tr, int tf, const QString &str, const QSize &size)
 {
     // get the color of the shadow: white for dark text, black for bright text
     QPen textPen = p.pen();
@@ -1056,7 +1133,7 @@ void TaskBar::drawShadowText(QPainter  &p, QRect tr, int tf, const QString & str
 
     // get a transparent pixmap
     QPainter pixPainter;
-    QPixmap textPixmap(width(), height());
+    QPixmap textPixmap(size);
 
     textPixmap.fill(QColor(0,0,0));
     textPixmap.setMask(textPixmap.createHeuristicMask(true));
@@ -1065,7 +1142,7 @@ void TaskBar::drawShadowText(QPainter  &p, QRect tr, int tf, const QString & str
     pixPainter.begin(&textPixmap);
     pixPainter.setPen(white);
     pixPainter.setFont(p.font()); // get the font from the root painter
-    pixPainter.drawText( tr, tf, str, len, brect, internal );
+    pixPainter.drawText(tr, tf, str);
     pixPainter.end();
 
     if (!m_textShadowEngine)
@@ -1083,7 +1160,40 @@ void TaskBar::drawShadowText(QPainter  &p, QRect tr, int tf, const QString & str
 
     // return
     p.drawImage(0, 0, img);
-    p.drawText(tr, tf, str, len, brect, internal);
+    p.drawText(tr, tf, str);
+}
+
+QImage* TaskBar::blendGradient(const QSize& size)
+{
+    if (m_blendGradient.isNull() || m_blendGradient.size() != size)
+    {
+        QPixmap bgpm(size);
+        QPainter bgp(&bgpm);
+        bgpm.fill(black);
+
+        if (QApplication::reverseLayout())
+        {
+            QImage gradient = KImageEffect::gradient(
+                    QSize(30, size.height()),
+                    QColor(255,255,255),
+                    QColor(0,0,0),
+                    KImageEffect::HorizontalGradient);
+            bgp.drawImage(0, 0, gradient);
+        }
+        else
+        {
+            QImage gradient = KImageEffect::gradient(
+                    QSize(30, size.height()),
+                    QColor(0,0,0),
+                    QColor(255,255,255),
+                    KImageEffect::HorizontalGradient);
+            bgp.drawImage(size.width() - 30, 0, gradient);
+        }
+
+        m_blendGradient = bgpm.convertToImage();
+    }
+
+    return &m_blendGradient;
 }
 
 void TaskBar::sortContainersByDesktop(TaskContainer::List& list)
@@ -1095,8 +1205,8 @@ void TaskBar::sortContainersByDesktop(TaskContainer::List& list)
 
     TaskContainer::List::ConstIterator lastUnsorted(list.constEnd());
     for (TaskContainer::List::ConstIterator it = list.constBegin();
-         it != lastUnsorted;
-         ++it)
+            it != lastUnsorted;
+            ++it)
     {
         sorted[i] = qMakePair((*it)->desktop(), qMakePair(i, *it));
         ++i;
