@@ -55,6 +55,7 @@ public:
 
     void add(Task::Ptr);
     void remove(Task::Ptr);
+    void remove(Startup::Ptr);
 
     bool contains(Task::Ptr);
     bool contains(Startup::Ptr);
@@ -84,6 +85,8 @@ public:
 
     void updateKickerTip(KickerTip::Data&);
 
+    void finish();
+
 signals:
     void showMe(TaskContainer*);
 
@@ -112,8 +115,9 @@ protected slots:
     void dragSwitch();
     void iconChanged();
     void setLastActivated();
-    void taskChanged();
+    void taskChanged(bool geometryChangeOnly);
     void showMe();
+    void setPaintEventCompression();
 
 private:
     void checkAttention(const Task::Ptr changed_task = NULL);
@@ -121,6 +125,7 @@ private:
     QTimer                      animationTimer;
     QTimer                      dragSwitchTimer;
     QTimer                      attentionTimer;
+    QTimer                      m_paintEventCompressionTimer;
     int                         currentFrame;
     PixmapList                  frames;
     int                         attentionState;
@@ -136,8 +141,8 @@ private:
     bool                        discardNextMouseEvent;
     bool                        aboutToActivate;
     bool                        m_mouseOver;
+    bool                        m_paintEventCompression;
     enum                        { ATTENTION_BLINK_TIMEOUT = 4 };
-    static QImage               blendGradient;
     QPoint                      m_dragStartPos;
 };
 
