@@ -14,16 +14,14 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <iostream.h>
-
 #ifdef HAVE_STRING_H
 #include <string.h>
 #else
@@ -51,7 +49,7 @@
 
 using namespace KIO;
 
-extern "C" { int kdemain(int argc, char **argv); }
+extern "C" { KDE_EXPORT int kdemain(int argc, char **argv); }
 
 int kdemain( int argc, char **argv )
 {
@@ -71,8 +69,8 @@ int kdemain( int argc, char **argv )
 
 void getDriveAndPath(const QString& path, QString& drive, QString& rest)
 {
-   drive="";
-   rest="";
+   drive=QString::null;
+   rest=QString::null;
    QStringList list=QStringList::split("/",path);
    for (QStringList::Iterator it=list.begin(); it!=list.end(); it++)
    {
@@ -80,8 +78,8 @@ void getDriveAndPath(const QString& path, QString& drive, QString& rest)
          drive=(*it)+":";
       else
          rest=rest+"/"+(*it);
-   };
-};
+   }
+}
 
 FloppyProtocol::FloppyProtocol (const QCString &pool, const QCString &app )
 :SlaveBase( "floppy", pool, app )
@@ -92,20 +90,17 @@ FloppyProtocol::FloppyProtocol (const QCString &pool, const QCString &app )
 ,m_stderrSize(0)
 {
    kdDebug(7101)<<"Floppy::Floppy: -"<<pool<<"-"<<endl;
-};
+}
 
 FloppyProtocol::~FloppyProtocol()
 {
-   if (m_stdoutBuffer!=0)
-      delete [] m_stdoutBuffer;
-   if (m_stderrBuffer!=0)
-      delete [] m_stderrBuffer;
-   if (m_mtool!=0)
-      delete m_mtool;
+   delete [] m_stdoutBuffer;
+   delete [] m_stderrBuffer;
+   delete m_mtool;
    m_mtool=0;
    m_stdoutBuffer=0;
    m_stderrBuffer=0;
-};
+}
 
 int FloppyProtocol::readStdout()
 {
@@ -122,21 +117,18 @@ int FloppyProtocol::readStdout()
    if (m_stdoutBuffer!=0)
    {
       memcpy(newBuffer, m_stdoutBuffer, m_stdoutSize);
-   };
+   }
    memcpy(newBuffer+m_stdoutSize, buffer, length);
    m_stdoutSize+=length;
    newBuffer[m_stdoutSize]='\0';
 
-   if (m_stdoutBuffer!=0)
-   {
-      delete [] m_stdoutBuffer;
-   };
+   delete [] m_stdoutBuffer;
    m_stdoutBuffer=newBuffer;
    //kdDebug(7101)<<"Floppy::readStdout(): -"<<m_stdoutBuffer<<"-"<<endl;
 
    //kdDebug(7101)<<"Floppy::readStdout ends"<<endl;
    return length;
-};
+}
 
 int FloppyProtocol::readStderr()
 {
@@ -151,7 +143,7 @@ int FloppyProtocol::readStderr()
    char buffer[16*1024];
    int length=::read(m_mtool->stderrFD(),buffer,16*1024);
    kdDebug(7101)<<"Floppy::readStderr(): read "<<length<<" bytes"<<endl;
-   if (length==0) return 0;
+   if (length<=0) return 0;
 
    //+1 gives us room for a terminating 0
    char *newBuffer=new char[length+m_stderrSize+1];
@@ -163,23 +155,20 @@ int FloppyProtocol::readStderr()
    m_stderrBuffer=newBuffer;
    kdDebug(7101)<<"Floppy::readStderr(): -"<<m_stderrBuffer<<"-"<<endl;
 
-   //kdDebug(7101)<<"Floppy::readStdout ends"<<endl;
    return length;
-};
+}
 
 void FloppyProtocol::clearBuffers()
 {
    kdDebug(7101)<<"Floppy::clearBuffers()"<<endl;
    m_stdoutSize=0;
    m_stderrSize=0;
-   if (m_stdoutBuffer!=0)
-      delete [] m_stdoutBuffer;
+   delete [] m_stdoutBuffer;
    m_stdoutBuffer=0;
-   if (m_stderrBuffer!=0)
-      delete [] m_stderrBuffer;
+   delete [] m_stderrBuffer;
    m_stderrBuffer=0;
    //kdDebug(7101)<<"Floppy::clearBuffers() ends"<<endl;
-};
+}
 
 void FloppyProtocol::terminateBuffers()
 {
@@ -188,13 +177,13 @@ void FloppyProtocol::terminateBuffers()
    if (m_stdoutBuffer!=0)
    {
       m_stdoutBuffer[m_stdoutSize]='\0';
-   };
+   }
    if (m_stderrBuffer!=0)
    {
       m_stderrBuffer[m_stderrSize]='\0';
-   };
+   }
    //kdDebug(7101)<<"Floppy::terminateBuffers() ends"<<endl;
-};
+}
 
 bool FloppyProtocol::stopAfterError(const KURL& url, const QString& drive)
 {
@@ -206,58 +195,67 @@ bool FloppyProtocol::stopAfterError(const KURL& url, const QString& drive)
    QTextIStream output(&outputString);
    QString line=output.readLine();
    kdDebug(7101)<<"line: -"<<line<<"-"<<endl;
-   if (line.contains("resource busy"))
+   if (line.find("resource busy") > -1)
    {
-      error( KIO::ERR_COULD_NOT_STAT, i18n("drive %1.\nThe drive is still busy.\nWait until it has stopped working and then try again.").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not access drive %1.\nThe drive is still busy.\nWait until it is inactive and then try again.").arg(drive));
    }
-   else if ((line.contains("Disk full")) || (line.contains("No free cluster")))
+   else if ((line.find("Disk full") > -1) || (line.find("No free cluster") > -1))
    {
-      error( KIO::ERR_COULD_NOT_WRITE, url.prettyURL()+i18n("\nThe disk in drive %1 is probably full.").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not write to file %1.\nThe disk in drive %2 is probably full.").arg(url.prettyURL(),drive));
    }
    //file not found
-   else if (line.contains("not found"))
+   else if (line.find("not found") > -1)
    {
       error( KIO::ERR_DOES_NOT_EXIST, url.prettyURL());
    }
    //no disk
-   else if (line.contains("not configured"))
+   else if (line.find("not configured") > -1)
    {
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("\nThere is probably no disk in the drive %1").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not access %1.\nThere is probably no disk in the drive %2").arg(url.prettyURL(),drive));
    }
-   else if (line.contains("not supported"))
+   else if (line.find("No such device") > -1)
    {
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("\nThe drive %1 is not supported.").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not access %1.\nThere is probably no disk in the drive %2 or you do not have enough permissions to access the drive.").arg(url.prettyURL(),drive));
+   }
+   else if (line.find("not supported") > -1)
+   {
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not access %1.\nThe drive %2 is not supported.").arg(url.prettyURL(),drive));
    }
    //not supported or no such drive
-   else if (line.contains("Permission denied"))
+   else if (line.find("Permission denied") > -1)
    {
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("\nMake sure the floppy in drive %1 is a DOS formatted floppy disk \nand that the permissions of the device file (e.g. /dev/fd0) are set correctly (e.g. rwxrwxrwx).").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not access %1.\nMake sure the floppy in drive %2 is a DOS-formatted floppy disk \nand that the permissions of the device file (e.g. /dev/fd0) are set correctly (e.g. rwxrwxrwx).").arg(url.prettyURL(),drive));
    }
-   else if (line.contains("non DOS media"))
+   else if (line.find("non DOS media") > -1)
    {
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("\nThe disk in drive %1 is probably not a DOS formatted floppy disk.").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not access %1.\nThe disk in drive %2 is probably not a DOS-formatted floppy disk.").arg(url.prettyURL(),drive));
    }
-   else if (line.contains("Read-only"))
+   else if (line.find("Read-only") > -1)
    {
-      error( KIO::ERR_WRITE_ACCESS_DENIED, url.prettyURL()+i18n("\nThe disk in drive %1 is probably write-protected.").arg(drive));
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Access denied.\nCould not write to %1.\nThe disk in drive %2 is probably write-protected.").arg(url.prettyURL(),drive));
    }
-   else if ((outputString.contains("already exists")) || (outputString.contains("Skipping ")))
+   else if ((outputString.find("already exists") > -1) || (outputString.find("Skipping ") > -1))
    {
       error( KIO::ERR_FILE_ALREADY_EXIST,url.prettyURL());
       //return false;
    }
+   else if (outputString.find("could not read boot sector") > -1)
+   {
+      error( KIO::ERR_SLAVE_DEFINED, i18n("Could not read boot sector for %1.\nThere is probably not any disk in drive %2.").arg(url.prettyURL(),drive));
+      //return false;
+   }
    else
    {
       error( KIO::ERR_UNKNOWN, outputString);
-   };
+   }
    return true;
-};
+}
 
 void FloppyProtocol::listDir( const KURL& _url)
 {
    kdDebug(7101)<<"Floppy::listDir() "<<_url.path()<<endl;
    KURL url(_url);
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
 
    if ((path.isEmpty()) || (path=="/"))
    {
@@ -265,7 +263,7 @@ void FloppyProtocol::listDir( const KURL& _url)
       redirection(url);
       finished();
       return;
-   };
+   }
    QString drive;
    QString floppyPath;
    getDriveAndPath(path,drive,floppyPath);
@@ -283,9 +281,9 @@ void FloppyProtocol::listDir( const KURL& _url)
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mdir"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mdir");
       return;
-   };
+   }
 
    int result;
    bool loopFinished(false);
@@ -307,8 +305,8 @@ void FloppyProtocol::listDir( const KURL& _url)
             {
                loopFinished=true;
                errorOccured=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    delete m_mtool;
@@ -339,7 +337,7 @@ void FloppyProtocol::listDir( const KURL& _url)
          {
             kdDebug(7101)<<"Floppy::listDir(): switching to mode 1"<<endl;
             mode=1;
-         };
+         }
       }
       else if (mode==1)
       {
@@ -348,7 +346,7 @@ void FloppyProtocol::listDir( const KURL& _url)
             kdDebug(7101)<<"Floppy::listDir(): ende"<<endl;
             totalSize(totalNumber);
             break;
-         };
+         }
          entry.clear();
          StatInfo info=createStatInfo(line);
          if (info.isValid)
@@ -357,13 +355,18 @@ void FloppyProtocol::listDir( const KURL& _url)
             //kdDebug(7101)<<"Floppy::listDir(): creating UDSEntry"<<endl;
             listEntry( entry, false);
             totalNumber++;
-         };
-      };
-   };
+         }
+      }
+   }
    listEntry( entry, true ); // ready
    finished();
    //kdDebug(7101)<<"Floppy::listDir() ends"<<endl;
-};
+}
+
+void FloppyProtocol::errorMissingMToolsProgram(const QString& name)
+{
+     error(KIO::ERR_SLAVE_DEFINED,i18n("Could not start program \"%1\".\nEnsure that the mtools package is installed correctly on your system.").arg(name));
+ }
 
 void FloppyProtocol::createUDSEntry(const StatInfo& info, UDSEntry& entry)
 {
@@ -387,7 +390,7 @@ void FloppyProtocol::createUDSEntry(const StatInfo& info, UDSEntry& entry)
    atom.m_uds = KIO::UDS_FILE_TYPE;
    atom.m_long =(info.isDir?S_IFDIR:S_IFREG);
    entry.append( atom );
-};
+}
 
 StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const QString& dirName)
 {
@@ -399,11 +402,10 @@ StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const
    QString hour, minute;
    StatInfo info;
 
-   static QDateTime beginningOfTimes(QDate(1970,1,1),QTime(1,0));
-
    if (line.length()==41)
    {
       int nameLength=line.find(' ');
+      kdDebug(7101)<<"Floppy::createStatInfo: line find: "<<nameLength <<"= -"<<line<<"-"<<endl;
       if (nameLength>0)
       {
          name=line.mid(0,nameLength);
@@ -411,14 +413,14 @@ StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const
          ext=ext.stripWhiteSpace();
          if (!ext.isEmpty())
             name+="."+ext;
-      };
-      kdDebug(7101)<<"Floppy::createUDSEntry() name 8.3= -"<<name<<"-"<<endl;
+      }
+      kdDebug(7101)<<"Floppy::createStatInfo() name 8.3= -"<<name<<"-"<<endl;
    }
    else if (line.length()>41)
    {
       name=line.mid(42);
-      kdDebug(7101)<<"Floppy::createUDSEntry() name vfat: -"<<name<<"-"<<endl;
-   };
+      kdDebug(7101)<<"Floppy::createStatInfo() name vfat: -"<<name<<"-"<<endl;
+   }
    if ((name==".") || (name==".."))
    {
       if (makeStat)
@@ -427,8 +429,8 @@ StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const
       {
          info.isValid=false;
          return info;
-      };
-   };
+      }
+   }
 
    if (line.mid(13,5)=="<DIR>")
    {
@@ -440,11 +442,21 @@ StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const
    {
       size=line.mid(13,9);
       //kdDebug(7101)<<"Floppy::createUDSEntry() size: -"<<size<<"-"<<endl;
-   };
+   }
 
-   day=line.mid(26,2);
-   month=line.mid(23,2);
-   year=line.mid(29,4);
+        //TEEKANNE JPG     70796 01-02-2003  17:47  Teekanne.jpg
+   if (line[25]=='-')
+   {
+      month=line.mid(23,2);
+      day=line.mid(26,2);
+      year=line.mid(29,4);
+   }
+   else //SETUP    PKG      1019 1997-09-25  10:31  setup.pkg
+   {
+      year=line.mid(23,4);
+      month=line.mid(28,2);
+      day=line.mid(31,2);
+   }
    hour=line.mid(35,2);
    minute=line.mid(38,2);
    //kdDebug(7101)<<"Floppy::createUDSEntry() day: -"<<day<<"-"<<month<<"-"<<year<<"- -"<<hour<<"-"<<minute<<"-"<<endl;
@@ -453,13 +465,13 @@ StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const
    {
       info.isValid=false;
       return info;
-   };
+   }
 
    info.name=name;
    info.size=size.toInt();
 
    QDateTime date(QDate(year.toInt(),month.toInt(),day.toInt()),QTime(hour.toInt(),minute.toInt()));
-   info.time=beginningOfTimes.secsTo(date);
+   info.time=date.toTime_t();
 
    if (isDir)
       info.mode = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH| S_IWOTH|S_IWGRP|S_IWUSR  ;
@@ -471,13 +483,13 @@ StatInfo FloppyProtocol::createStatInfo(const QString line, bool makeStat, const
    info.isValid=true;
    //kdDebug(7101)<<"Floppy::createUDSEntry() ends"<<endl;
    return info;
-};
+}
 
 StatInfo FloppyProtocol::_stat(const KURL& url)
 {
    StatInfo info;
 
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
    QString drive;
    QString floppyPath;
    getDriveAndPath(path,drive,floppyPath);
@@ -493,7 +505,7 @@ StatInfo FloppyProtocol::_stat(const KURL& url)
       info.isValid=true;
 
       return info;
-   };
+   }
 
    //kdDebug(7101)<<"Floppy::_stat(): delete m_mtool"<<endl;
    if (m_mtool!=0)
@@ -509,9 +521,9 @@ StatInfo FloppyProtocol::_stat(const KURL& url)
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mdir"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mdir");
       return info;
-   };
+   }
 
 
    clearBuffers();
@@ -536,8 +548,8 @@ StatInfo FloppyProtocol::_stat(const KURL& url)
             {
                loopFinished=true;
                errorOccured=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    //kdDebug(7101)<<"Floppy::_stat(): delete m_mtool"<<endl;
@@ -551,12 +563,12 @@ StatInfo FloppyProtocol::_stat(const KURL& url)
    {
       info.isValid=false;
       return info;
-   };
+   }
 
    if (m_stdoutSize==0)
    {
       info.isValid=false;
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("Don't know why, sorry."));
+      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL());
       return info;
    }
 
@@ -564,27 +576,24 @@ StatInfo FloppyProtocol::_stat(const KURL& url)
    QString outputString(m_stdoutBuffer);
    QTextIStream output(&outputString);
    QString line;
-   int lineNumber(0);
-   while (!output.atEnd())
+   for (int lineNumber=0; !output.atEnd(); lineNumber++)
    {
       line=output.readLine();
-      if (lineNumber==4)
-      {
-         StatInfo info=createStatInfo(line,true,url.fileName());
-         if (info.isValid==false)
-            error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("Don't know why, sorry."));
-         return info;
-      };
-      lineNumber++;
-   };
+      if ( (lineNumber<3) || (line.isEmpty()) )
+         continue;
+      StatInfo info=createStatInfo(line,true,url.fileName());
+      if (info.isValid==false)
+         error( KIO::ERR_COULD_NOT_STAT, url.prettyURL());
+      return info;
+   }
    if (info.isValid==false)
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("Don't know why, sorry."));
+      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL());
    return info;
-};
+}
 
 int FloppyProtocol::freeSpace(const KURL& url)
 {
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
    QString drive;
    QString floppyPath;
    getDriveAndPath(path,drive,floppyPath);
@@ -603,9 +612,9 @@ int FloppyProtocol::freeSpace(const KURL& url)
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mdir"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mdir");
       return -1;
-   };
+   }
 
 
    clearBuffers();
@@ -630,8 +639,8 @@ int FloppyProtocol::freeSpace(const KURL& url)
             {
                loopFinished=true;
                errorOccured=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    //kdDebug(7101)<<"Floppy::freeSpace(): delete m_mtool"<<endl;
@@ -644,11 +653,11 @@ int FloppyProtocol::freeSpace(const KURL& url)
    if (errorOccured)
    {
       return -1;
-   };
+   }
 
    if (m_stdoutSize==0)
    {
-      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL()+i18n("Don't know why, sorry."));
+      error( KIO::ERR_COULD_NOT_STAT, url.prettyURL());
       return -1;
    }
 
@@ -670,17 +679,17 @@ int FloppyProtocol::freeSpace(const KURL& url)
          tmp=tmp.stripWhiteSpace();
 
          return tmp.toInt();
-      };
+      }
       lineNumber++;
-   };
+   }
    return -1;
-};
+}
 
 void FloppyProtocol::stat( const KURL & _url)
 {
    kdDebug(7101)<<"Floppy::stat() "<<_url.path()<<endl;
    KURL url(_url);
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
 
    if ((path.isEmpty()) || (path=="/"))
    {
@@ -688,7 +697,7 @@ void FloppyProtocol::stat( const KURL & _url)
       redirection(url);
       finished();
       return;
-   };
+   }
    StatInfo info=this->_stat(url);
    if (info.isValid)
    {
@@ -698,14 +707,14 @@ void FloppyProtocol::stat( const KURL & _url)
       finished();
       //kdDebug(7101)<<"Floppy::stat(): ends"<<endl;
       return;
-   };
+   }
    //otherwise the error() was already reported in _stat()
 }
 
 void FloppyProtocol::mkdir( const KURL& url, int)
 {
    kdDebug(7101)<<"FloppyProtocol::mkdir()"<<endl;
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
 
    if ((path.isEmpty()) || (path=="/"))
    {
@@ -714,7 +723,7 @@ void FloppyProtocol::mkdir( const KURL& url, int)
       redirection(newUrl);
       finished();
       return;
-   };
+   }
    QString drive;
    QString floppyPath;
    getDriveAndPath(path,drive,floppyPath);
@@ -722,7 +731,7 @@ void FloppyProtocol::mkdir( const KURL& url, int)
    {
       finished();
       return;
-   };
+   }
    if (m_mtool!=0)
       delete m_mtool;
    //kdDebug(7101)<<"Floppy::stat(): create args"<<endl;
@@ -736,9 +745,9 @@ void FloppyProtocol::mkdir( const KURL& url, int)
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mmd"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mmd");
       return;
-   };
+   }
 
 
    clearBuffers();
@@ -762,8 +771,8 @@ void FloppyProtocol::mkdir( const KURL& url, int)
             {
                loopFinished=true;
                errorOccured=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    delete m_mtool;
@@ -777,7 +786,7 @@ void FloppyProtocol::mkdir( const KURL& url, int)
 void FloppyProtocol::del( const KURL& url, bool isfile)
 {
    kdDebug(7101)<<"FloppyProtocol::del()"<<endl;
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
 
    if ((path.isEmpty()) || (path=="/"))
    {
@@ -786,7 +795,7 @@ void FloppyProtocol::del( const KURL& url, bool isfile)
       redirection(newUrl);
       finished();
       return;
-   };
+   }
    QString drive;
    QString floppyPath;
    getDriveAndPath(path,drive,floppyPath);
@@ -794,28 +803,36 @@ void FloppyProtocol::del( const KURL& url, bool isfile)
    {
       finished();
       return;
-   };
+   }
 
    if (m_mtool!=0)
       delete m_mtool;
    //kdDebug(7101)<<"Floppy::stat(): create args"<<endl;
    QStringList args;
+   
+   bool usingmdel;
 
    if (isfile)
+   {
       args<<"mdel"<<(drive+floppyPath);
+      usingmdel=true;
+   }
    else
+   {
       args<<"mrd"<<(drive+floppyPath);
+      usingmdel=false;
+   }
 
-   kdDebug(7101)<<"Floppy::del(): executing: mrd -"<<(drive+floppyPath)<<"-"<<endl;
+   kdDebug(7101)<<"Floppy::del(): executing: " << (usingmdel ? QString("mdel") : QString("mrd") ) << "-"<<(drive+floppyPath)<<"-"<<endl;
 
    m_mtool=new Program(args);
    if (!m_mtool->start())
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mrd"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram(usingmdel ? QString("mdel") : QString("mrd"));
       return;
-   };
+   }
 
 
    clearBuffers();
@@ -839,8 +856,8 @@ void FloppyProtocol::del( const KURL& url, bool isfile)
             {
                loopFinished=true;
                errorOccured=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    delete m_mtool;
@@ -849,12 +866,12 @@ void FloppyProtocol::del( const KURL& url, bool isfile)
    if (errorOccured)
       return;
    finished();
-};
+}
 
 void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite )
 {
-   QString srcPath( QFile::encodeName(src.path()));
-   QString destPath( QFile::encodeName(dest.path()));
+   QString srcPath(src.path());
+   QString destPath(dest.path());
 
    kdDebug(7101)<<"Floppy::rename() -"<<srcPath<<"- to -"<<destPath<<"-"<<endl;
 
@@ -871,7 +888,7 @@ void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite
    {
       finished();
       return;
-   };
+   }
 
    QString destDrive;
    QString destFloppyPath;
@@ -880,7 +897,7 @@ void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite
    {
       finished();
       return;
-   };
+   }
 
    if (m_mtool!=0)
       delete m_mtool;
@@ -890,7 +907,7 @@ void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite
    if (_overwrite)
       args<<"mren"<<"-o"<<(srcDrive+srcFloppyPath)<<(destDrive+destFloppyPath);
    else
-      args<<"mren"<<"-s"<<(srcDrive+srcFloppyPath)<<(destDrive+destFloppyPath);
+      args<<"mren"<<"-D"<<"s"<<(srcDrive+srcFloppyPath)<<(destDrive+destFloppyPath);
 
    kdDebug(7101)<<"Floppy::move(): executing: mren -"<<(srcDrive+srcFloppyPath)<<"  "<<(destDrive+destFloppyPath)<<endl;
 
@@ -899,9 +916,9 @@ void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mren"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mren");
       return;
-   };
+   }
 
 
    clearBuffers();
@@ -925,8 +942,8 @@ void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite
             {
                loopFinished=true;
                errorOccured=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    delete m_mtool;
@@ -935,11 +952,11 @@ void FloppyProtocol::rename( const KURL &src, const KURL &dest, bool _overwrite
    if (errorOccured)
       return;
    finished();
-};
+}
 
 void FloppyProtocol::get( const KURL& url )
 {
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
    kdDebug(7101)<<"Floppy::get() -"<<path<<"-"<<endl;
 
    if ((path.isEmpty()) || (path=="/"))
@@ -949,7 +966,7 @@ void FloppyProtocol::get( const KURL& url )
       redirection(newUrl);
       finished();
       return;
-   };
+   }
    StatInfo info=this->_stat(url);
    //the error was already reported in _stat()
    if (info.isValid==false)
@@ -964,7 +981,7 @@ void FloppyProtocol::get( const KURL& url )
    {
       finished();
       return;
-   };
+   }
 
    if (m_mtool!=0)
       delete m_mtool;
@@ -979,13 +996,9 @@ void FloppyProtocol::get( const KURL& url )
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mcopy"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mcopy");
       return;
-   };
-
-
-   time_t t_start = time( 0L );
-   time_t t_last = t_start;
+   }
 
    clearBuffers();
    int result;
@@ -1000,8 +1013,7 @@ void FloppyProtocol::get( const KURL& url )
       result=m_mtool->select(1,0,stdoutEvent, stderrEvent);
       if (stdoutEvent)
       {
-         if (m_stdoutBuffer!=0)
-            delete [] m_stdoutBuffer;
+         delete [] m_stdoutBuffer;
          m_stdoutBuffer=0;
          m_stdoutSize=0;
          if (readStdout()>0)
@@ -1012,19 +1024,12 @@ void FloppyProtocol::get( const KURL& url )
             data( array );
             array.resetRawData(m_stdoutBuffer, m_stdoutSize);
 
-            time_t t = time( 0L );
-            if ( t - t_last >= 1 )
-            {
-               processedSize(bytesRead);
-               speed(bytesRead/(t-t_start));
-               t_last = t;
-            }
          }
          else
          {
             loopFinished=true;
          }
-      };
+      }
       if (stderrEvent)
       {
          if (readStderr()==0)
@@ -1034,8 +1039,8 @@ void FloppyProtocol::get( const KURL& url )
             {
                errorOccured=true;
                loopFinished=true;
-            };
-      };
+            }
+      }
    } while (!loopFinished);
 
    //kdDebug(7101)<<"Floppy::get(): deleting m_mtool"<<endl;
@@ -1047,11 +1052,11 @@ void FloppyProtocol::get( const KURL& url )
    //kdDebug(7101)<<"Floppy::get(): finishing"<<endl;
    data( QByteArray() );
    finished();
-};
+}
 
 void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
 {
-   QString path( QFile::encodeName(url.path()));
+   QString path(url.path());
    kdDebug(7101)<<"Floppy::put() -"<<path<<"-"<<endl;
 
    if ((path.isEmpty()) || (path=="/"))
@@ -1061,7 +1066,7 @@ void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
       redirection(newUrl);
       finished();
       return;
-   };
+   }
    QString drive;
    QString floppyPath;
    getDriveAndPath(path,drive,floppyPath);
@@ -1069,7 +1074,7 @@ void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
    {
       finished();
       return;
-   };
+   }
    int freeSpaceLeft=freeSpace(url);
    if (freeSpaceLeft==-1)
       return;
@@ -1090,9 +1095,9 @@ void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
    {
       delete m_mtool;
       m_mtool=0;
-      error(ERR_CANNOT_LAUNCH_PROCESS,"mcopy"+i18n("\nEnsure that the mtools package is installed correctly on your system."));
+      errorMissingMToolsProgram("mcopy");
       return;
-   };
+   }
 
 
    clearBuffers();
@@ -1112,7 +1117,7 @@ void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
       {
          if (readStdout()==0)
             result=0;
-      };
+      }
       if (stderrEvent)
       {
          if (readStderr()==0)
@@ -1136,16 +1141,16 @@ void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
             if (bytesRead>freeSpaceLeft)
             {
                result=0;
-               error( KIO::ERR_COULD_NOT_WRITE, url.prettyURL()+i18n("\nThe disk in drive %1 is probably full.").arg(drive));
+               error( KIO::ERR_SLAVE_DEFINED, i18n("Could not write to file %1.\nThe disk in drive %2 is probably full.").arg(url.prettyURL(),drive));
             }
             else
             {
                //kdDebug(7101)<<"Floppy::put(): writing..."<<endl;
                result=::write(m_mtool->stdinFD(),buffer.data(), buffer.size());
                kdDebug(7101)<<"Floppy::put(): after write(), wrote "<<result<<" bytes"<<endl;
-            };
+            }
          }
-      };
+      }
    }
    while ( result > 0 );
 
@@ -1154,11 +1159,11 @@ void FloppyProtocol::put( const KURL& url, int , bool overwrite, bool )
       perror("writing to stdin");
       error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, url.prettyURL());
       return;
-   };
+   }
 
    delete m_mtool;
    m_mtool=0;
 
    finished();
-};
+}
 
