@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "KonqMainWindowIface.h"
@@ -28,6 +28,7 @@
 #include <kdcopactionproxy.h>
 #include <kdcoppropertyproxy.h>
 #include <kdebug.h>
+#include <kstartupinfo.h>
 #include <kwin.h>
 
 KonqMainWindowIface::KonqMainWindowIface( KonqMainWindow * mainWindow )
@@ -54,6 +55,22 @@ void KonqMainWindowIface::newTab( QString url )
   m_pMainWindow->openFilteredURL( url, true );
 }
 
+void KonqMainWindowIface::openURL( QString url, bool tempFile )
+{
+  m_pMainWindow->openFilteredURL( url, false, tempFile );
+}
+
+void KonqMainWindowIface::newTab( QString url, bool tempFile )
+{
+  m_pMainWindow->openFilteredURL( url, true, tempFile );
+}
+
+void KonqMainWindowIface::newTabASN( QString url, const QCString& startup_id, bool tempFile )
+{
+  KStartupInfo::setNewStartupId( m_pMainWindow, startup_id );
+  m_pMainWindow->openFilteredURL( url, true, tempFile );
+}
+
 void KonqMainWindowIface::reload()
 {
   m_pMainWindow->slotReload();
@@ -123,9 +140,6 @@ bool KonqMainWindowIface::windowCanBeUsedForTab()
         return false; // this window shows on different desktop
     if( KonqMainWindow::isPreloaded() )
         return false; // we want a tab in an already shown window
-    if ( m_pMainWindow->isMinimized() )
-        m_pMainWindow->showNormal();
-
-    m_pMainWindow->raise();
     return true;
 }
+
