@@ -1,6 +1,6 @@
 /*
     This file is part of the KDE system
-    Copyright (C)  1999,2000 Boloni Laszlo
+    Copyright (C)  1999,2000 Boloni Laszlo <lboloni@cpe.ucf.edu>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,23 +14,21 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
-    konsole_part.cpp
  */
 
 #include "konsole_part.h"
 
-#include <sys/stat.h>
-#include <stdlib.h>
+#include <assert.h>
 
 #include <qfile.h>
 #include <qlayout.h>
 #include <qwmatrix.h>
-#include <qregexp.h>
 
 #include <kaboutdata.h>
+#include <kcharsets.h>
 #include <kdebug.h>
 #include <kfontdialog.h>
 #include <kglobalsettings.h>
@@ -50,18 +48,10 @@
 #include <kconfig.h>
 #include <kaction.h>
 
-extern "C"
-{
-  /**
-   * This function is the 'main' function of this part.  It takes
-   * the form 'void *init_lib<library name>()  It always returns a
-   * new factory object
-   */
-  void *init_libkonsolepart()
-  {
-    return new konsoleFactory;
-  }
-}
+// We can't use the ARGB32 visual when embedded in another application
+bool argb_visual = false;
+
+K_EXPORT_COMPONENT_FACTORY( libkonsolepart, konsoleFactory )
 
 /**
  * We need one static instance of the factory for our C 'main' function
@@ -89,7 +79,7 @@ KParts::Part *konsoleFactory::createPartObject(QWidget *parentWidget, const char
                                          QObject *parent, const char *name, const char *classname,
                                          const QStringList&)
 {
-  kdDebug(1211) << "konsoleFactory::createPart parentWidget=" << parentWidget << " parent=" << parent << endl;
+//  kdDebug(1211) << "konsoleFactory::createPart parentWidget=" << parentWidget << " parent=" << parent << endl;
   KParts::Part *obj = new konsolePart(parentWidget, widgetName, parent, name, classname);
   return obj;
 }
@@ -98,55 +88,12 @@ KInstance *konsoleFactory::instance()
 {
   if ( !s_instance )
     {
-      s_aboutData = new KAboutData("konsole", I18N_NOOP("Konsole"), "1.1");
+      s_aboutData = new KAboutData("konsole", I18N_NOOP("Konsole"), "1.5");
       s_instance = new KInstance( s_aboutData );
     }
   return s_instance;
 }
 
-//////////////////////////////////////////////////////////////////////
-
-class KonsoleFontSelectAction : public KSelectAction {
-public:
-    KonsoleFontSelectAction(const QString &text, int accel,
-                            const QObject* receiver, const char* slot,
-                            QObject* parent, const char* name = 0 )
-        : KSelectAction(text, accel, receiver, slot, parent, name) {}
-    KonsoleFontSelectAction( const QString &text, const QIconSet& pix,
-                             int accel, const QObject* receiver,
-                             const char* slot, QObject* parent,
-                             const char* name = 0 )
-        : KSelectAction(text, pix, accel, receiver, slot, parent, name) {}
-
-    virtual void slotActivated( int index );
-};
-
-void KonsoleFontSelectAction::slotActivated(int index) {
-    // emit even if it's already activated
-    if (currentItem() == index) {
-      KSelectAction::slotActivated();
-      return;
-    } else {
-      KSelectAction::slotActivated(index);
-    }
-}
-
-//////////////////////////////////////////////////////////////////////
-
-static const char * const fonts[] = {
- "13",
- "7",   // tiny font, never used
- "10",  // small font
- "13",  // medium
- "15",  // large
- "20", // huge
- "-misc-console-medium-r-normal--16-160-72-72-c-80-iso10646-1", // "Linux"
- "-misc-fixed-medium-r-normal--15-140-75-75-c-90-iso10646-1",    // "Unicode"
- };
-
-#define TOPFONT (sizeof(fonts)/sizeof(char*))
-#define DEFAULTFONT TOPFONT
-
 #define DEFAULT_HISTORY_SIZE 1000
 
 konsolePart::konsolePart(QWidget *_parentWidget, const char *widgetName, QObject *parent, const char *name, const char *classname)
@@ -157,8 +104,8 @@ konsolePart::konsolePart(QWidget *_parentWidget, const char *widgetName, QObject
 ,rootxpm(0)
 ,blinkingCursor(0)
 ,showFrame(0)
+,m_useKonsoleSettings(0)
 ,selectBell(0)
-,selectFont(0)
 ,selectLineSpacing(0)
 ,selectScrollbar(0)
 ,m_keytab(0)
@@ -166,6 +113,9 @@ konsolePart::konsolePart(QWidget *_parentWidget, const char *widgetName, QObject
 ,m_signals(0)
 ,m_options(0)
 ,m_popupMenu(0)
+,b_useKonsoleSettings(false)
+,b_autoDestroy(true)
+,b_autoStartShell(true)
 ,m_histSize(DEFAULT_HISTORY_SIZE)
 ,m_runningShell( false )
 {
@@ -198,6 +148,12 @@ konsolePart::konsolePart(QWidget *_parentWidget, const char *widgetName, QObject
   colors->checkSchemas();
   colors->sort();
 
+  // Check to see which config file we use: konsolepartrc or konsolerc
+  KConfig* config = new KConfig("konsolepartrc", true);
+  config->setDesktopGroup();
+  b_useKonsoleSettings = config->readBoolEntry("use_konsole_settings", false);
+  delete config;
+
   readProperties();
 
   makeGUI();
@@ -221,9 +177,24 @@ konsolePart::konsolePart(QWidget *_parentWidget, const char *widgetName, QObject
   // insert keymaps into menu
   if (m_keytab)
   {
+     m_keytab->clear();
+
+     QStringList kt_titles;
+     typedef QMap<QString,KeyTrans*> QStringKeyTransMap;
+     QStringKeyTransMap kt_map;
+
      for (int i = 0; i < KeyTrans::count(); i++)
      {
         KeyTrans* ktr = KeyTrans::find(i);
+        assert( ktr );
+        QString title=ktr->hdr().lower();
+        kt_titles << title;
+        kt_map[title] = ktr;
+     }
+     kt_titles.sort();
+     for ( QStringList::Iterator it = kt_titles.begin(); it != kt_titles.end(); ++it ) {
+        KeyTrans* ktr = kt_map[*it];
+        assert( ktr );
         QString title=ktr->hdr();
         m_keytab->insertItem(title.replace('&',"&&"),ktr->numb());
      }
@@ -231,33 +202,48 @@ konsolePart::konsolePart(QWidget *_parentWidget, const char *widgetName, QObject
 
   applySettingsToGUI();
 
-  // kDebugInfo("Loading successful");
+  QTimer::singleShot( 0, this, SLOT( autoShowShell() ) );
+}
 
-  QTimer::singleShot( 0, this, SLOT( showShell() ) );
+void konsolePart::autoShowShell()
+{
+    // possibly clear the screen?
+    if (b_autoStartShell)
+        showShell();
+}
 
+void konsolePart::setAutoDestroy( bool enabled )
+{
+    b_autoDestroy = enabled;
+}
+
+void konsolePart::setAutoStartShell( bool enabled )
+{
+    b_autoStartShell = enabled;
 }
 
 void konsolePart::doneSession(TESession*)
 {
   // see doneSession in konsole.cpp
-  if (se)
+  if (se && b_autoDestroy)
   {
-    kdDebug(1211) << "doneSession - disconnecting done" << endl;
+//    kdDebug(1211) << "doneSession - disconnecting done" << endl;
     disconnect( se,SIGNAL(done(TESession*)),
                 this,SLOT(doneSession(TESession*)) );
     se->setConnect(false);
     //QTimer::singleShot(100,se,SLOT(terminate()));
-    kdDebug(1211) << "se->terminate()" << endl;
+//    kdDebug(1211) << "se->terminate()" << endl;
     se->terminate();
   }
 }
 
 void konsolePart::sessionDestroyed()
 {
-  kdDebug(1211) << "sessionDestroyed()" << endl;
+//  kdDebug(1211) << "sessionDestroyed()" << endl;
   disconnect( se, SIGNAL( destroyed() ), this, SLOT( sessionDestroyed() ) );
   se = 0;
-  delete this;
+  if (b_autoDestroy)
+      delete this;
 }
 
 void konsolePart::configureRequest(TEWidget*_te,int,int x,int y)
@@ -268,10 +254,10 @@ void konsolePart::configureRequest(TEWidget*_te,int,int x,int y)
 
 konsolePart::~konsolePart()
 {
-  kdDebug(1211) << "konsolePart::~konsolePart() this=" << this << endl;
+//  kdDebug(1211) << "konsolePart::~konsolePart() this=" << this << endl;
   if ( se ) {
     disconnect( se, SIGNAL( destroyed() ), this, SLOT( sessionDestroyed() ) );
-    kdDebug(1211) << "Deleting se session" << endl;
+//    kdDebug(1211) << "Deleting se session" << endl;
     delete se;
     se=0;
   }
@@ -293,7 +279,7 @@ bool konsolePart::openURL( const KURL & url )
 
   m_url = url;
   emit setWindowCaption( url.prettyURL() );
-  kdDebug(1211) << "Set Window Caption to " << url.prettyURL() << "\n";
+//  kdDebug(1211) << "Set Window Caption to " << url.prettyURL() << "\n";
   emit started( 0 );
 
   if ( url.isLocalFile() ) {
@@ -323,7 +309,8 @@ void konsolePart::makeGUI()
   if (!kapp->authorizeKAction("konsole_rmb"))
      return;
 
-  KActionCollection* actions = new KActionCollection(this);
+  actions = new KActionCollection( (KMainWindow*)parentWidget );
+  settingsActions = new KActionCollection( (KMainWindow*)parentWidget );
 
   // Send Signal Menu -------------------------------------------------------------
   if (kapp->authorizeKAction("send_signal"))
@@ -347,7 +334,7 @@ void konsolePart::makeGUI()
 
      // Scrollbar
      selectScrollbar = new KSelectAction(i18n("Sc&rollbar"), 0, this,
-                                      SLOT(slotSelectScrollbar()), actions);
+                                      SLOT(slotSelectScrollbar()), settingsActions);
 
      QStringList scrollitems;
      scrollitems << i18n("&Hide") << i18n("&Left") << i18n("&Right");
@@ -357,32 +344,29 @@ void konsolePart::makeGUI()
      // Select Bell
      m_options->insertSeparator();
      selectBell = new KSelectAction(i18n("&Bell"), SmallIconSet( "bell"), 0 , this,
-                                 SLOT(slotSelectBell()), actions, "bell");
+                                 SLOT(slotSelectBell()), settingsActions, "bell");
 
      QStringList bellitems;
-     bellitems << i18n("&None")
-            << i18n("&System Notification")
-            << i18n("&Visible Bell");
+     bellitems << i18n("System &Bell")
+            << i18n("System &Notification")
+            << i18n("&Visible Bell")
+            << i18n("N&one");
      selectBell->setItems(bellitems);
      selectBell->plug(m_options);
 
-     // Select font
-     selectFont = new KonsoleFontSelectAction( i18n( "&Font" ), SmallIconSet( "text" ), 0,
-                                            this, SLOT(slotSelectFont()), actions, "font");
-     QStringList it;
-     it << i18n("&Normal")
-     << i18n("&Tiny")
-     << i18n("&Small")
-     << i18n("&Medium")
-     << i18n("&Large")
-     << i18n("&Huge")
-     << ""
-     << i18n("&Linux")
-     << i18n("&Unicode")
-     << ""
-     << i18n("&Custom...");
-     selectFont->setItems(it);
-     selectFont->plug(m_options);
+      m_fontsizes = new KActionMenu( i18n( "Font" ), SmallIconSet( "text" ), settingsActions, 0L );
+      m_fontsizes->insert( new KAction( i18n( "&Enlarge Font" ), SmallIconSet( "viewmag+" ), 0, this, SLOT( biggerFont() ), settingsActions, "enlarge_font" ) );
+      m_fontsizes->insert( new KAction( i18n( "&Shrink Font" ), SmallIconSet( "viewmag-" ), 0, this, SLOT( smallerFont() ), settingsActions, "shrink_font" ) );
+      m_fontsizes->insert( new KAction( i18n( "Se&lect..." ), SmallIconSet( "font" ), 0, this, SLOT( slotSelectFont() ), settingsActions, "select_font" ) );
+      m_fontsizes->plug(m_options);
+
+      // encoding menu, start with default checked !
+      selectSetEncoding = new KSelectAction( i18n( "&Encoding" ), SmallIconSet("charset" ), 0, this, SLOT(slotSetEncoding()), settingsActions, "set_encoding" );
+      QStringList list = KGlobal::charsets()->descriptiveEncodingNames();
+      list.prepend( i18n( "Default" ) );
+      selectSetEncoding->setItems(list);
+      selectSetEncoding->setCurrentItem (0);
+      selectSetEncoding->plug(m_options);
 
      // Keyboard Options Menu ---------------------------------------------------
      if (kapp->authorizeKAction("keyboard"))
@@ -405,24 +389,16 @@ void konsolePart::makeGUI()
 
 
      KAction *historyType = new KAction(i18n("&History..."), "history", 0, this,
-                                        SLOT(slotHistoryType()), actions, "history");
+                                        SLOT(slotHistoryType()), settingsActions, "history");
      historyType->plug(m_options);
      m_options->insertSeparator();
 
      // Select line spacing
-     selectLineSpacing =
-       new KSelectAction
-       (
-        i18n("Li&ne Spacing"),
-        SmallIconSet("leftjust"), // Non-code hack.
-        0,
-        this,
-        SLOT(slotSelectLineSpacing()),
-        this
-       );
+     selectLineSpacing = new KSelectAction(i18n("Li&ne Spacing"),
+        SmallIconSet("leftjust"), 0, this,
+        SLOT(slotSelectLineSpacing()), settingsActions );
 
      QStringList lineSpacingList;
-
      lineSpacingList
        << i18n("&0")
        << i18n("&1")
@@ -433,28 +409,35 @@ void konsolePart::makeGUI()
        << i18n("&6")
        << i18n("&7")
        << i18n("&8");
-
      selectLineSpacing->setItems(lineSpacingList);
      selectLineSpacing->plug(m_options);
 
      // Blinking Cursor
      blinkingCursor = new KToggleAction (i18n("Blinking &Cursor"),
-                                      0, this,SLOT(slotBlinkingCursor()), actions);
+                                      0, this,SLOT(slotBlinkingCursor()), settingsActions);
      blinkingCursor->plug(m_options);
 
      // Frame on/off
      showFrame = new KToggleAction(i18n("Show Fr&ame"), 0,
-                                this, SLOT(slotToggleFrame()), actions);
+                                this, SLOT(slotToggleFrame()), settingsActions);
+     showFrame->setCheckedState(i18n("Hide Fr&ame"));
      showFrame->plug(m_options);
 
      // Word Connectors
      KAction *WordSeps = new KAction(i18n("Wor&d Connectors..."), 0, this,
-                                  SLOT(slotWordSeps()), actions);
+                                  SLOT(slotWordSeps()), settingsActions);
      WordSeps->plug(m_options);
 
+     // Use Konsole's Settings
+     m_options->insertSeparator();
+     m_useKonsoleSettings = new KToggleAction( i18n("&Use Konsole's Settings"),
+                          0, this, SLOT(slotUseKonsoleSettings()), 0, "use_konsole_settings" );
+     m_useKonsoleSettings->plug(m_options);
+
      // Save Settings
      m_options->insertSeparator();
-     KAction *saveSettings = KStdAction::saveOptions(this, SLOT(saveProperties()), actions);
+     KAction *saveSettings = new KAction(i18n("&Save as Default"), "filesave", 0, this, 
+                    SLOT(saveProperties()), actions, "save_default");
      saveSettings->plug(m_options);
      if (KGlobalSettings::insertTearOffHandle())
         m_options->insertTearOffHandle();
@@ -495,12 +478,18 @@ void konsolePart::makeGUI()
 
 void konsolePart::applySettingsToGUI()
 {
+  m_useKonsoleSettings->setChecked( b_useKonsoleSettings );
+  setSettingsMenuEnabled( !b_useKonsoleSettings );
+
+  applyProperties();
+
+  if ( b_useKonsoleSettings )
+    return; // Don't change Settings menu items
+
   if (showFrame)
      showFrame->setChecked( b_framevis );
   if (selectScrollbar)
      selectScrollbar->setCurrentItem(n_scroll);
-  if (selectFont)
-     selectFont->setCurrentItem(n_font);
   updateKeytabMenu();
   if (selectBell)
      selectBell->setCurrentItem(n_bell);
@@ -510,27 +499,70 @@ void konsolePart::applySettingsToGUI()
      blinkingCursor->setChecked(te->blinkingCursor());
   if (m_schema)
      m_schema->setItemChecked(curr_schema,true);
+  if (selectSetEncoding)
+     selectSetEncoding->setCurrentItem(n_encoding);
+}
+
+void konsolePart::applyProperties()
+{
+   if ( !se ) return;
+
+   if ( b_histEnabled && m_histSize )
+      se->setHistory( HistoryTypeBuffer(m_histSize ) );
+   else if ( b_histEnabled && !m_histSize )
+      se->setHistory(HistoryTypeFile() );
+   else
+     se->setHistory( HistoryTypeNone() );
+   se->setKeymapNo( n_keytab );
+
+   // FIXME:  Move this somewhere else...
+   KConfig* config = new KConfig("konsolerc",true);
+   config->setGroup("UTMP");
+   se->setAddToUtmp( config->readBoolEntry("AddToUtmp",true));
+   delete config;
+
+   se->widget()->setVTFont( defaultFont );
+   se->setSchemaNo( curr_schema );
+   slotSetEncoding();
+}
+
+void konsolePart::setSettingsMenuEnabled( bool enable )
+{
+   uint count = settingsActions->count();
+   for ( uint i = 0; i < count; i++ )
+   {
+      settingsActions->action( i )->setEnabled( enable );
+   }
+
+   // FIXME: These are not in settingsActions.
+   //  When disabled, the icons are not 'grey-ed' out.
+   m_keytab->setEnabled( enable );
+   m_schema->setEnabled( enable );
 }
 
 void konsolePart::readProperties()
 {
-  KConfig* config = new KConfig("konsolepartrc",true);
+  KConfig* config;
+
+  if ( b_useKonsoleSettings )
+    config = new KConfig( "konsolerc", true );
+  else
+    config = new KConfig( "konsolepartrc", true );
+
   config->setDesktopGroup();
 
   b_framevis = config->readBoolEntry("has frame",false);
   b_histEnabled = config->readBoolEntry("historyenabled",true);
-  n_bell = QMIN(config->readUnsignedNumEntry("bellmode",TEWidget::BELLSYSTEM),2);
-  n_font = QMIN(config->readUnsignedNumEntry("font",3),TOPFONT);
+  n_bell = QMIN(config->readUnsignedNumEntry("bellmode",TEWidget::BELLSYSTEM),3);
   n_keytab=config->readNumEntry("keytab",0); // act. the keytab for this session
   n_scroll = QMIN(config->readUnsignedNumEntry("scrollbar",TEWidget::SCRRIGHT),2);
   m_histSize = config->readNumEntry("history",DEFAULT_HISTORY_SIZE);
   s_word_seps= config->readEntry("wordseps",":@-./_~");
 
-  QFont tmpFont("fixed");
-  tmpFont.setFixedPitch(true);
-  tmpFont.setStyleHint(QFont::TypeWriter);
+  n_encoding = config->readNumEntry("encoding",0);
+
+  QFont tmpFont = KGlobalSettings::fixedFont();
   defaultFont = config->readFontEntry("defaultfont", &tmpFont);
-  setFont(QMIN(config->readUnsignedNumEntry("font",3),TOPFONT));
 
   QString schema = config->readEntry("Schema");
 
@@ -546,7 +578,6 @@ void konsolePart::readProperties()
   te->setColorTable(sch->table()); //FIXME: set twice here to work around a bug
 
   if (sch->useTransparency()) {
-    //KONSOLEDEBUG << "Setting up transparency" << endl;
     if (!rootxpm)
       rootxpm = new KRootPixmap(te);
     rootxpm->setFadeEffect(sch->tr_x(), QColor(sch->tr_r(), sch->tr_g(), sch->tr_b()));
@@ -582,18 +613,23 @@ void konsolePart::saveProperties()
   KConfig* config = new KConfig("konsolepartrc");
   config->setDesktopGroup();
 
-  config->writeEntry("bellmode",n_bell);
-  config->writeEntry("BlinkingCursor", te->blinkingCursor());
-  config->writeEntry("defaultfont", defaultFont);
-  config->writeEntry("font",n_font);
-  config->writeEntry("history", se->history().getSize());
-  config->writeEntry("historyenabled", b_histEnabled);
-  config->writeEntry("keytab",n_keytab);
-  config->writeEntry("has frame",b_framevis);
-  config->writeEntry("LineSpacing", te->lineSpacing());
-  config->writeEntry("schema",s_kconfigSchema);
-  config->writeEntry("scrollbar",n_scroll);
-  config->writeEntry("wordseps",s_word_seps);
+  if ( b_useKonsoleSettings ) { // Don't save Settings if using konsolerc
+    config->writeEntry("use_konsole_settings", m_useKonsoleSettings->isChecked());
+  } else {
+    config->writeEntry("bellmode",n_bell);
+    config->writeEntry("BlinkingCursor", te->blinkingCursor());
+    config->writeEntry("defaultfont", (se->widget())->getVTFont());
+    config->writeEntry("history", se->history().getSize());
+    config->writeEntry("historyenabled", b_histEnabled);
+    config->writeEntry("keytab",n_keytab);
+    config->writeEntry("has frame",b_framevis);
+    config->writeEntry("LineSpacing", te->lineSpacing());
+    config->writeEntry("schema",s_kconfigSchema);
+    config->writeEntry("scrollbar",n_scroll);
+    config->writeEntry("wordseps",s_word_seps);
+    config->writeEntry("encoding",n_encoding);
+    config->writeEntry("use_konsole_settings",m_useKonsoleSettings->isChecked());
+  }
 
   config->sync();
   delete config;
@@ -623,67 +659,41 @@ void konsolePart::slotSelectScrollbar()
 }
 
 void konsolePart::slotSelectFont() {
-  int item = selectFont->currentItem();
-  if( item > 9 ) // compensate for the two separators
-      --item;
-  if( item > 6 )
-      --item;
-  // KONSOLEDEBUG << "slotSelectFont " << item << endl;
-  if (item == DEFAULTFONT) {
-    if ( KFontDialog::getFont(defaultFont, true) == QDialog::Rejected ) {
-      selectFont->setCurrentItem(n_font);
+   if ( !se ) return;
+
+   QFont font = se->widget()->getVTFont();
+   if ( KFontDialog::getFont( font, true ) != QDialog::Accepted )
       return;
-    }
-  }
-  setFont(item);
+
+   se->widget()->setVTFont( font );
 }
 
-void konsolePart::setFont(int fontno)
-{
-  if ( ! se ) return;
-  QFont f;
-  if (fontno == DEFAULTFONT)
-    f = defaultFont;
-  else
-    if (fonts[fontno][0] == '-')
-    {
-      f.setRawName( fonts[fontno] );
-      f.setFixedPitch(true);
-      f.setStyleHint( QFont::TypeWriter );
-      if ( !f.exactMatch() && fontno != DEFAULTFONT)
-      {
-        // Ugly hack to prevent bug #20487
-        fontNotFound_par=fonts[fontno];
-        QTimer::singleShot(1,this,SLOT(fontNotFound()));
-        return;
-      }
-    }
-    else
-    {
-      f.setFamily("fixed");
-      f.setFixedPitch(true);
-      f.setStyleHint(QFont::TypeWriter);
-      f.setPixelSize(QString(fonts[fontno]).toInt());
-    }
+void konsolePart::biggerFont(void) {
+    if ( !se ) return;
 
-
-  se->setFontNo(fontno);
-  te->setVTFont(f);
-  n_font = fontno;
+    QFont f = te->getVTFont();
+    f.setPointSize( f.pointSize() + 1 );
+    te->setVTFont( f );
 }
 
-void konsolePart::fontNotFound()
-{
-  QString msg = i18n("Font `%1' not found.\nCheck README.linux.console for help.").arg(fontNotFound_par);
-  KMessageBox::information(parentWidget,  msg, i18n("Font Not Found"), QString("font_not_found_%1").arg(fontNotFound_par));
+void konsolePart::smallerFont(void) {
+    if ( !se ) return;
+
+    QFont f = te->getVTFont();
+    if ( f.pointSize() < 6 ) return;      // A minimum size
+    f.setPointSize( f.pointSize() - 1 );
+    te->setVTFont( f );
 }
 
 void konsolePart::updateKeytabMenu()
 {
-  if ( ! se || ! m_keytab ) return;
-  m_keytab->setItemChecked(n_keytab,false);
-  m_keytab->setItemChecked(se->keymapNo(),true);
-  n_keytab = se->keymapNo();
+  if ( se && m_keytab ) {
+    m_keytab->setItemChecked(n_keytab,false);
+    m_keytab->setItemChecked(se->keymapNo(),true);
+    n_keytab = se->keymapNo();
+  } else if ( m_keytab ) {    // no se yet, happens at startup
+    m_keytab->setItemChecked(n_keytab, true);
+  }
 }
 
 void konsolePart::keytab_menu_activated(int item)
@@ -775,7 +785,7 @@ void konsolePart::setSchema(ColorSchema* s)
   se->setSchemaNo(s->numb());
 }
 
-void konsolePart::notifySize(int /* lines */, int /* columns */)
+void konsolePart::notifySize(int /* columns */, int /* lines */)
 {
   ColorSchema *sch=colors->find(s_schema);
 
@@ -856,6 +866,24 @@ void konsolePart::slotSelectBell() {
   te->setBellMode(n_bell);
 }
 
+void konsolePart::slotSetEncoding()
+{
+  if (!se) return;
+
+  bool found;
+  QString enc = KGlobal::charsets()->encodingForName(selectSetEncoding->currentText());
+  QTextCodec * qtc = KGlobal::charsets()->codecForName(enc, found);
+  if(!found)
+  {
+    kdDebug() << "Codec " << selectSetEncoding->currentText() << " not found!" << endl;
+    qtc = QTextCodec::codecForLocale();
+  }
+
+  n_encoding = selectSetEncoding->currentItem();
+  se->setEncodingNo(selectSetEncoding->currentItem());
+  se->getEmulation()->setCodec(qtc);
+}
+
 void konsolePart::slotSelectLineSpacing()
 {
   te->setLineSpacing( selectLineSpacing->currentItem() );
@@ -866,6 +894,17 @@ void konsolePart::slotBlinkingCursor()
   te->setBlinkingCursor(blinkingCursor->isChecked());
 }
 
+void konsolePart::slotUseKonsoleSettings()
+{
+   b_useKonsoleSettings = m_useKonsoleSettings->isChecked();
+
+   setSettingsMenuEnabled( !b_useKonsoleSettings );
+
+   readProperties();
+
+   applySettingsToGUI();
+}
+
 void konsolePart::slotWordSeps() {
   bool ok;
 
@@ -883,7 +922,7 @@ void konsolePart::enableMasterModeConnections()
   if ( se ) se->setListenToKeyPress(true);
 }
 
-void konsolePart::updateTitle()
+void konsolePart::updateTitle(TESession *)
 {
   if ( se ) emit setWindowCaption( se->fullTitle() );
 }
@@ -1009,21 +1048,32 @@ const char* sensibleShell()
 
 void konsolePart::startProgram( const QString& program,
                                 const QStrList& args )
+{
+//    kdDebug(1211) << "konsolePart::startProgram for " << program << endl;
+    if ( !se )
+        newSession();
+    se->setProgram( program, args );
+    se->run();
+}
+
+void konsolePart::newSession()
 {
   if ( se ) delete se;
-  se = new TESession(te, program, args, "xterm", parentWidget->winId());
+  se = new TESession(te, "xterm", parentWidget->winId());
   connect( se,SIGNAL(done(TESession*)),
            this,SLOT(doneSession(TESession*)) );
   connect( se,SIGNAL(openURLRequest(const QString &)),
            this,SLOT(emitOpenURLRequest(const QString &)) );
-  connect( se, SIGNAL( updateTitle() ),
-           this, SLOT( updateTitle() ) );
+  connect( se, SIGNAL( updateTitle(TESession*) ),
+           this, SLOT( updateTitle(TESession*) ) );
   connect( se, SIGNAL(enableMasterModeConnections()),
            this, SLOT(enableMasterModeConnections()) );
-  connect( se, SIGNAL( processExited() ),
-           this, SLOT( slotProcessExited() ) );
+  connect( se, SIGNAL( processExited(KProcess *) ),
+           this, SIGNAL( processExited(KProcess *) ) );
   connect( se, SIGNAL( receivedData( const QString& ) ),
-           this, SLOT( slotReceivedData( const QString& ) ) );
+           this, SIGNAL( receivedData( const QString& ) ) );
+  connect( se, SIGNAL( forkedChild() ),
+           this, SIGNAL( forkedChild() ));
 
   // We ignore the following signals
   //connect( se, SIGNAL(renameSession(TESession*,const QString&)),
@@ -1033,25 +1083,13 @@ void konsolePart::startProgram( const QString& program,
   //connect( se, SIGNAL(disableMasterModeConnections()),
   //        this, SLOT(disableMasterModeConnections()) );
 
-  if (b_histEnabled && m_histSize)
-    se->setHistory(HistoryTypeBuffer(m_histSize));
-  else if (b_histEnabled && !m_histSize)
-    se->setHistory(HistoryTypeFile());
-  else
-    se->setHistory(HistoryTypeNone());
-  se->setKeymapNo(n_keytab);
-  KConfig* config = new KConfig("konsolerc",true);
-  config->setGroup("UTMP");
-  se->setAddToUtmp( config->readBoolEntry("AddToUtmp",true));
-  delete config;
+  applyProperties();
 
   se->setConnect(true);
-  se->setSchemaNo(curr_schema);
-  se->run();
+  // se->run();
   connect( se, SIGNAL( destroyed() ), this, SLOT( sessionDestroyed() ) );
-  setFont( n_font ); // we do this here, to make TEWidget recalculate
+//  setFont( n_font ); // we do this here, to make TEWidget recalculate
                      // its geometry..
-  te->emitText( QString::fromLatin1( "\014" ) );
 }
 
 void konsolePart::showShellInDir( const QString& dir )
@@ -1084,13 +1122,4 @@ void konsolePart::sendInput( const QString& text )
     te->emitText( text );
 }
 
-void konsolePart::slotProcessExited()
-{
-    emit processExited();
-}
-void konsolePart::slotReceivedData( const QString& s )
-{
-    emit receivedData( s );
-}
-
 #include "konsole_part.moc"
