@@ -87,9 +87,9 @@ void Workspace::updateClientArea( bool force )
         }
     for ( ClientList::ConstIterator it = clients.begin(); it != clients.end(); ++it)
         {
-            QRect r = (*it)->adjustedClientArea( desktopArea, desktopArea );
-            if( r == desktopArea ) // should be sufficient
+            if( !(*it)->hasStrut())
                 continue;
+            QRect r = (*it)->adjustedClientArea( desktopArea, desktopArea );
             if( (*it)->isOnAllDesktops())
                 for( int i = 1;
                         i <= numberOfDesktops();
@@ -248,11 +248,10 @@ QRect Workspace::clientArea( clientAreaOption opt, const QPoint& p, int desktop
                 return desktopwidget->geometry();
         case WorkArea:
             return warea;
-        case FullXineramaArea:
         case FullArea:
             return desktopwidget->geometry();
         case ScreenArea:
-            return sarea;
+            return desktopwidget->screenGeometry( screen );
         }
     assert( false );
     return QRect();
@@ -759,6 +758,30 @@ QRect Client::adjustedClientArea( const QRect &desktopArea, const QRect& area )
 //            kdDebug () << "Trimming right-strut to geometry(): " << stareaR << endl;
         }
     }
+
+    QRect screenarea = workspace()->clientArea( ScreenArea, this );
+    // HACK: workarea handling is not xinerama aware, so if this strut
+    // reserves place at a xinerama edge that's inside the virtual screen,
+    // ignore the strut for workspace setting.
+    if( area == kapp->desktop()->geometry())
+        {
+        if( stareaL.left() < screenarea.left())
+            stareaL = QRect();
+        if( stareaR.right() > screenarea.right())
+            stareaR = QRect();
+        if( stareaT.top() < screenarea.top())
+            stareaT = QRect();
+        if( stareaB.bottom() < screenarea.bottom())
+            stareaB = QRect();
+        }
+    // Handle struts at xinerama edges that are inside the virtual screen.
+    // They're given in virtual screen coordinates, make them affect only
+    // their xinerama screen.
+    stareaL.setLeft( KMAX( stareaL.left(), screenarea.left()));
+    stareaR.setRight( KMIN( stareaR.right(), screenarea.right()));
+    stareaT.setTop( KMAX( stareaT.top(), screenarea.top()));
+    stareaB.setBottom( KMIN( stareaB.bottom(), screenarea.bottom()));
+
     if (stareaL . intersects (area)) {
 //        kdDebug () << "Moving left of: " << r << " to " << stareaL.right() + 1 << endl;
         r . setLeft( stareaL . right() + 1 );
@@ -813,13 +836,12 @@ NETExtendedStrut Client::strut() const
         }
     return ext;
     }
+
 bool Client::hasStrut() const
     {
     NETExtendedStrut ext = strut();
     if( ext.left_width == 0 && ext.right_width == 0 && ext.top_width == 0 && ext.bottom_width == 0 )
-        {
-            return false;
-        }
+        return false;
     return true;
     }
 
@@ -1004,6 +1026,8 @@ QSize Client::adjustedSize( const QSize& frame, Sizemode mode ) const
 
     QSize wsize( frame.width() - ( border_left + border_right ),
              frame.height() - ( border_top + border_bottom ));
+    if( wsize.isEmpty())
+        wsize = QSize( 1, 1 );
 
     return sizeForClientSize( wsize, mode, false );
     }
@@ -1209,6 +1233,8 @@ void Client::getWmNormalHints()
         xSizeHint.flags = 0;
     // set defined values for the fields, even if they're not in flags
 
+    if( ! ( xSizeHint.flags & PMinSize ))
+        xSizeHint.min_width = xSizeHint.min_height = 0;
     if( xSizeHint.flags & PBaseSize )
         {
         // PBaseSize is a fallback for PMinSize according to ICCCM 4.1.2.3
@@ -1222,8 +1248,6 @@ void Client::getWmNormalHints()
         }
     else
         xSizeHint.base_width = xSizeHint.base_height = 0;
-    if( ! ( xSizeHint.flags & PMinSize ))
-        xSizeHint.min_width = xSizeHint.min_height = 0;
     if( ! ( xSizeHint.flags & PMaxSize ))
         xSizeHint.max_width = xSizeHint.max_height = INT_MAX;
     else
@@ -1259,7 +1283,21 @@ void Client::getWmNormalHints()
         { // update to match restrictions
         QSize new_size = adjustedSize();
         if( new_size != size() && !isFullScreen())
+            {
+            QRect orig_geometry = geometry();
             resizeWithChecks( new_size );
+            if( ( !isSpecialWindow() || isToolbar()) && !isFullScreen())
+                {
+                // try to keep the window in its xinerama screen if possible,
+                // if that fails at least keep it visible somewhere
+                QRect area = workspace()->clientArea( MovementArea, this );
+                if( area.contains( orig_geometry ))
+                    keepInArea( area );
+                area = workspace()->clientArea( WorkArea, this );
+                if( area.contains( orig_geometry ))
+                    keepInArea( area );
+                }
+            }
         }
     updateAllowedActions(); // affects isResizeable()
     }
@@ -1379,8 +1417,8 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
         // the location to the current location but miscalculate the
         // frame size due to kwin being a double-reparenting window
         // manager
-        if ( new_pos.x() == x() + clientPos().x() &&
-             new_pos.y() == y() + clientPos().y() && gravity == NorthWestGravity )
+        if ( new_pos.x() == x() + clientPos().x() && new_pos.y() == y() + clientPos().y()
+            && gravity == NorthWestGravity && !from_tool )
             {
             new_pos.setX( x());
             new_pos.setY( y());
@@ -1438,10 +1476,17 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             resizeWithChecks( ns );
             xSizeHint.win_gravity = save_gravity;
             updateFullScreenHack( QRect( calculateGravitation( true, xSizeHint.win_gravity ), QSize( nw, nh )));
-            QRect area = workspace()->clientArea( WorkArea, this );
-            if( !from_tool && ( !isSpecialWindow() || isToolbar()) && !isFullScreen()
-                && area.contains( orig_geometry ))
-                keepInArea( area );
+            if( !from_tool && ( !isSpecialWindow() || isToolbar()) && !isFullScreen())
+                {
+                // try to keep the window in its xinerama screen if possible,
+                // if that fails at least keep it visible somewhere
+                QRect area = workspace()->clientArea( MovementArea, this );
+                if( area.contains( orig_geometry ))
+                    keepInArea( area );
+                area = workspace()->clientArea( WorkArea, this );
+                if( area.contains( orig_geometry ))
+                    keepInArea( area );
+                }
             }
         }
     // No need to send synthetic configure notify event here, either it's sent together
@@ -1664,8 +1709,7 @@ void Client::setGeometry( int x, int y, int w, int h, ForceGeometry_t force )
             cs.width(), cs.height());
         XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
         }
-    if( shape())
-        updateShape();
+    updateShape();
     // SELI TODO won't this be too expensive?
     updateWorkareaDiffs();
     sendSyntheticConfigureNotify();
@@ -1719,8 +1763,7 @@ void Client::plainResize( int w, int h, ForceGeometry_t force )
             cs.width(), cs.height());
         XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
         }
-    if( shape())
-        updateShape();
+    updateShape();
     updateWorkareaDiffs();
     sendSyntheticConfigureNotify();
     updateWindowRules();
@@ -1811,7 +1854,7 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
 
     // maximing one way and unmaximizing the other way shouldn't happen
     Q_ASSERT( !( vertical && horizontal )
-        || (( max_mode & MaximizeVertical != 0 ) == ( max_mode & MaximizeHorizontal != 0 )));
+        || ((( max_mode & MaximizeVertical ) != 0 ) == (( max_mode & MaximizeHorizontal ) != 0 )));
 
     QRect clientArea = workspace()->clientArea( MaximizeArea, this );
 
@@ -2063,18 +2106,11 @@ void Client::setFullScreen( bool set, bool user )
     workspace()->updateClientLayer( this ); // active fullscreens get different layer
     info->setState( isFullScreen() ? NET::FullScreen : 0, NET::FullScreen );
     updateDecoration( false, false );
-    // XINERAMA
-    if( isFullScreen() && xSizeHint.width > workspace()->clientArea(FullScreenArea, this).width() )
-        setGeometry( workspace()->clientArea( FullXineramaArea, this ));
-    else if( isFullScreen() && xSizeHint.height > workspace()->clientArea(FullScreenArea, this).height() )
-        setGeometry( workspace()->clientArea( FullXineramaArea, this ));
-    else if( isFullScreen())
+    if( isFullScreen())
         setGeometry( workspace()->clientArea( FullScreenArea, this ));
     else
         {
-        if( maximizeMode() != MaximizeRestore )
-            changeMaximize( false, false, true ); // adjust size
-        else if( !geom_fs_restore.isNull())
+        if( !geom_fs_restore.isNull())
             setGeometry( QRect( geom_fs_restore.topLeft(), adjustedSize( geom_fs_restore.size())));
         // TODO isShaded() ?
         else
@@ -2085,24 +2121,38 @@ void Client::setFullScreen( bool set, bool user )
     updateWindowRules();
     }
 
-bool Client::checkFullScreenHack( const QRect& geom ) const
+int Client::checkFullScreenHack( const QRect& geom ) const
     {
     // if it's noborder window, and has size of one screen or the whole desktop geometry, it's fullscreen hack
-    return (( geom.size() == workspace()->clientArea( FullArea, geom.center(), desktop()).size()
-            || geom.size() == workspace()->clientArea( ScreenArea, geom.center(), desktop()).size())
-        && noBorder() && !isUserNoBorder() && isFullScreenable( true ));
+    if( noBorder() && !isUserNoBorder() && isFullScreenable( true ))
+        {
+        if( geom.size() == workspace()->clientArea( FullArea, geom.center(), desktop()).size())
+            return 2; // full area fullscreen hack
+        if( geom.size() == workspace()->clientArea( ScreenArea, geom.center(), desktop()).size())
+            return 1; // xinerama-aware fullscreen hack
+        }
+    return 0;
     }
 
 void Client::updateFullScreenHack( const QRect& geom )
     {
-    bool is_hack = checkFullScreenHack( geom );
-    if( fullscreen_mode == FullScreenNone && is_hack )
+    int type = checkFullScreenHack( geom );
+    if( fullscreen_mode == FullScreenNone && type != 0 )
         {
         fullscreen_mode = FullScreenHack;
         updateDecoration( false, false );
-        setGeometry( workspace()->clientArea( FullScreenArea, this ));
+        QRect geom;
+        if( rules()->checkStrictGeometry( false ))
+            {
+            geom = type == 2 // 1 - it's xinerama-aware fullscreen hack, 2 - it's full area
+                ? workspace()->clientArea( FullArea, geom.center(), desktop())
+                : workspace()->clientArea( ScreenArea, geom.center(), desktop());
+            }
+        else
+            geom = workspace()->clientArea( FullScreenArea, geom.center(), desktop());
+        setGeometry( geom );
         }
-    else if( fullscreen_mode == FullScreenHack && !is_hack )
+    else if( fullscreen_mode == FullScreenHack && type == 0 )
         {
         fullscreen_mode = FullScreenNone;
         updateDecoration( false, false );
