@@ -1,5 +1,5 @@
 /*
- * $Id: kompmgr.c 431747 2005-07-05 06:02:02Z adridg $
+ * $Id: kompmgr.c 576937 2006-08-25 08:50:31Z mueller $
  *
  * Copyright © 2003 Keith Packard
  *
@@ -201,7 +201,7 @@ conv            *gaussianMap;
 typedef enum _compMode {
     CompSimple,		/* looks like a regular X server */
     CompServerShadows,	/* use window alpha for shadow; sharp, but precise */
-    CompClientShadows,	/* use window extents for shadow, blurred */
+    CompClientShadows	/* use window extents for shadow, blurred */
 } CompMode;
 
 static void
@@ -316,8 +316,8 @@ set_fade (Display *dpy, win *w, double start, double finish, double step,
 	f = find_fade (w);
 	if (!f)
 	{
-   	if (start == finish)
-      	return;
+		if (start == finish)
+			return;
 		f = malloc (sizeof (fade));
 		f->next = 0;
 		f->w = w;
@@ -326,15 +326,12 @@ set_fade (Display *dpy, win *w, double start, double finish, double step,
 		enqueue_fade (dpy, f);
 	}
 	else if(!override)
-         return;
-   else if (start == finish)
-   {
-      dequeue_fade(dpy,f);
-      return;
-   }
-	else if (exec_callback && f->callback)
-      (*f->callback)(dpy, f->w, f->gone);
-
+		return;
+	else
+	{
+		if (exec_callback && f->callback)
+			(*f->callback)(dpy, f->w, f->gone);
+	}
 	if (finish < 0)
 		finish = 0;
 	if (finish > 1)
@@ -344,7 +341,7 @@ set_fade (Display *dpy, win *w, double start, double finish, double step,
 		f->step = step;
 	else if (f->cur > finish)
 		f->step = -step;
-   f->gone = gone && (exec_callback || f->callback != unmap_callback);
+	f->gone = gone && (exec_callback || f->callback != unmap_callback);
 	f->callback = callback;
 	w->opacity = f->cur * OPAQUE;
 	if (wholeWin)
@@ -395,7 +392,7 @@ run_fades (Display *dpy)
 		win *w = f->w;
 		next = f->next;
 		f->cur += f->step * steps;
-		if (f->cur > 1)
+		if (f->cur >= 1)
 			f->cur = 1;
 		else if (f->cur < 0)
 			f->cur = 0;
@@ -1200,6 +1197,9 @@ paint_all (Display *dpy, XserverRegion region)
 				case CompSimple:
 					break;
 				case CompServerShadows:
+					/* dont' bother drawing shadows on desktop windows */
+					if (w->windowType == winDesktopAtom)
+					        break;
 					XFixesSetPictureClipRegion (dpy, rootBuffer, 0, 0, w->borderClip);
 					set_ignore (dpy, NextRequest (dpy));
 					if (w->opacity != OPAQUE && !w->shadowPict)
@@ -2201,7 +2201,7 @@ expose_root (Display *dpy, Window root, XRectangle *rects, int nrects)
 	static int
 ev_serial (XEvent *ev)
 {
-	if (ev->type & 0x7f != KeymapNotify)
+	if ((ev->type & 0x7f) != KeymapNotify)
 		return ev->xany.serial;
 	return NextRequest (ev->xany.display);
 }
@@ -2413,10 +2413,10 @@ loadConfig(char *filename){
 	if( filename == NULL ){
 		const char *home = getenv("HOME");
 		const char *configfile = "/.xcompmgrrc"; 
-        wasNull = True;
-      int n = strlen(home)+strlen(configfile)+1;
+		int n = strlen(home)+strlen(configfile)+1;
 		filename = (char*)malloc(n*sizeof(char));
-      memset(filename,0,n);
+		memset(filename,0,n);
+		wasNull = True;
 
 		strcat(filename, home);
 		strcat(filename, configfile);
@@ -2484,6 +2484,17 @@ usage (char *program)
 	exit (1);
 }
 
+	static void
+give_me_a_name(void)
+{
+  Window w;
+
+  w = XCreateSimpleWindow (dpy, RootWindow(dpy, 0), 0, 0, 1, 1, 0, None,
+		  None);
+  Xutf8SetWMProperties(dpy, w, "kcompmgr", "kcompmgr", NULL, 0, NULL, NULL,
+		  NULL);
+}	
+
 	int
 main (int argc, char **argv)
 {
@@ -2628,6 +2639,9 @@ main (int argc, char **argv)
 	}
 
 	fprintf(stderr, "Started\n");
+
+	give_me_a_name();
+
 	/* get atoms */
 	shadowAtom = XInternAtom (dpy, SHADOW_PROP, False);
 	opacityAtom = XInternAtom (dpy, OPACITY_PROP, False);
@@ -2710,7 +2724,7 @@ main (int argc, char **argv)
 			}
 
 			XNextEvent (dpy, &ev);
-			if (ev.type & 0x7f != KeymapNotify)
+			if ((ev.type & 0x7f) != KeymapNotify)
 				discard_ignore (dpy, ev.xany.serial);
 #if DEBUG_EVENTS
 			printf ("event %10.10s serial 0x%08x window 0x%08x\n",
@@ -2836,13 +2850,13 @@ main (int argc, char **argv)
                                                 if (w)
                                                 {
                                                     unsigned int tmp = get_dim_prop(dpy, w);
-                                                    if (tmp < OPAQUE)
-                                                        w->dimPicture = solid_picture (dpy, True, (double)tmp/OPAQUE, 0.1, 0.1, 0.1);
-                                                    else if (w->dimPicture)
+                                                    if (w->dimPicture)
                                                     {
                                                         XRenderFreePicture (dpy, w->dimPicture);
                                                         w->dimPicture = None;
                                                     }
+                                                    if (tmp < OPAQUE)
+                                                        w->dimPicture = solid_picture (dpy, True, (double)tmp/OPAQUE, 0.1, 0.1, 0.1);
                                                 }
                                                 else
                                                 printf("arrrg, window not found\n");
@@ -2864,7 +2878,16 @@ main (int argc, char **argv)
 									break;*/ /*skip if opacity does not change*/
 								if (fadeTrans)
 								{
-								    set_fade (dpy, w, w->opacity*1.0/OPAQUE, (tmp*1.0)/OPAQUE, fade_out_step, 0, False, True, True, False);
+									static double start, finish, step;
+									start = w->opacity*1.0/OPAQUE;
+									finish = (tmp*1.0)/OPAQUE;
+									
+									if ( start > finish )
+										step = fade_out_step;
+									else
+										step = fade_in_step;
+									
+								    set_fade (dpy, w, start, finish, step, 0, False, True, True, False);
                                     break;
                                     }
                                 else
