@@ -29,7 +29,7 @@ License. See the file "COPYING" for the exact licensing terms.
  the clients were created.
 
  Every window has one layer assigned in which it is. There are 6 layers,
- from bottom : DesktopLayer, BelowLayer, NormalLayer, AboveLayer, DockLayer
+ from bottom : DesktopLayer, BelowLayer, NormalLayer, DockLayer, AboveLayer
  and ActiveLayer (see also NETWM sect.7.10.). The layer a window is in depends
  on the window type, and on other things like whether the window is active.
 
@@ -70,7 +70,6 @@ License. See the file "COPYING" for the exact licensing terms.
 #include "client.h"
 #include "workspace.h"
 #include "tabbox.h"
-#include "popupinfo.h"
 #include "group.h"
 #include "rules.h"
 
@@ -101,7 +100,7 @@ void Workspace::updateStackingOrder( bool propagate_new_clients )
     {
     if( block_stacking_updates > 0 )
         {
-        blocked_propagating_new_clients |= propagate_new_clients;
+        blocked_propagating_new_clients = blocked_propagating_new_clients || propagate_new_clients;
         return;
         }
     ClientList new_stacking_order = constrainedStackingOrder();
@@ -114,7 +113,7 @@ void Workspace::updateStackingOrder( bool propagate_new_clients )
         for( ClientList::ConstIterator it = stacking_order.begin();
              it != stacking_order.end();
              ++it )
-            kdDebug() << (void*)(*it) << *it << endl;
+            kdDebug() << (void*)(*it) << *it << ":" << (*it)->layer() << endl;
         }
 #endif
     if( changed || propagate_new_clients )
@@ -182,16 +181,6 @@ void Workspace::propagateClients( bool propagate_new_clients )
         cl[pos++] =  (*it)->window();
     rootInfo->setClientListStacking( cl, pos );
     delete [] cl;
-
-#if 0 // not necessary anymore?
-    if ( tab_box->isVisible() )
-        tab_box->raise();
-
-    if ( popupinfo->isVisible() )
-        popupinfo->raise();
-
-    raiseElectricBorders();
-#endif
     }
 
 
@@ -432,21 +421,41 @@ void Workspace::restackClientUnderActive( Client* c )
             }
         }
     assert( unconstrained_stacking_order.contains( c ));
-    if( c->wantsTabFocus() && focus_chain.contains( active_client ))
+    for( int desktop = 1;
+         desktop <= numberOfDesktops();
+         ++desktop )
+        { // do for every virtual desktop to handle the case of onalldesktop windows
+        if( c->wantsTabFocus() && c->isOnDesktop( desktop ) && focus_chain[ desktop ].contains( active_client ))
+            {
+            // also put in focus_chain[currentDesktop()] after all windows belonging to the active applicationa
+            focus_chain[ desktop ].remove( c );
+            for( ClientList::Iterator it = focus_chain[ desktop ].fromLast();
+                 it != focus_chain[ desktop ].end();
+                 --it )
+      	        {
+                if( Client::belongToSameApplication( active_client, *it ))
+                    {
+                    focus_chain[ desktop ].insert( it, c );
+                    break;
+                    }
+                }
+            }
+      	}
+    // the same for global_focus_chain
+    if( c->wantsTabFocus() && global_focus_chain.contains( active_client ))
         {
-        // also put in focus_chain after all windows belonging to the active application
-        focus_chain.remove( c );
-        for( ClientList::Iterator it = focus_chain.fromLast();
-             it != focus_chain.end();
+        global_focus_chain.remove( c );
+        for( ClientList::Iterator it = global_focus_chain.fromLast();
+             it != global_focus_chain.end();
              --it )
             {
             if( Client::belongToSameApplication( active_client, *it ))
                 {
-                focus_chain.insert( it, c );
+                global_focus_chain.insert( it, c );
                 break;
                 }
             }
-        }
+      	}
     updateStackingOrder();
     }
 
@@ -504,7 +513,7 @@ ClientList Workspace::constrainedStackingOrder()
     for( ClientList::ConstIterator it = stacking.begin();
          it != stacking.end();
          ++it )
-        kdDebug() << (void*)(*it) << *it << endl;
+        kdDebug() << (void*)(*it) << *it << ":" << (*it)->layer() << endl;
 #endif
     // now keep transients above their mainwindows
     // TODO this could(?) use some optimization
@@ -571,7 +580,7 @@ ClientList Workspace::constrainedStackingOrder()
     for( ClientList::ConstIterator it = stacking.begin();
          it != stacking.end();
          ++it )
-        kdDebug() << (void*)(*it) << *it << endl;
+        kdDebug() << (void*)(*it) << *it << ":" << (*it)->layer() << endl;
     kdDebug() << "\n\n" << endl;
 #endif
     return stacking;
@@ -723,8 +732,8 @@ Layer Client::belongsToLayer() const
     const Client* ac = workspace()->mostRecentlyActivatedClient(); // instead of activeClient() - avoids flicker
     const Client* top = workspace()->topClientOnDesktop( desktop(), true );
     if( isFullScreen() && ac != NULL && top != NULL
-          && ( ac == this || this->group() == ac->group())
-          && ( top == this || this->group() == top->group()))
+        && ( ac == this || this->group() == ac->group())
+        && ( top == this || this->group() == top->group()))
         return ActiveLayer;
     if( keepAbove())
         return AboveLayer;
