@@ -13,9 +13,11 @@ License. See the file "COPYING" for the exact licensing terms.
 #define KWIN_WORKSPACE_H
 
 #include <qtimer.h>
+#include <qvaluevector.h>
 #include <kshortcut.h>
 #include <qcursor.h>
 #include <netwm.h>
+#include <kxmessages.h>
 
 #include "KWinInterface.h"
 #include "utils.h"
@@ -27,8 +29,13 @@ License. See the file "COPYING" for the exact licensing terms.
 class QPopupMenu;
 class KConfig;
 class KGlobalAccel;
+class KShortcutDialog;
 class KStartupInfo;
+class KStartupInfoId;
 class KStartupInfoData;
+class QSlider;
+class QPushButton;
+class KProcess;
 
 namespace KWinInternal
 {
@@ -39,6 +46,8 @@ class PopupInfo;
 class RootInfo;
 class PluginMgr;
 class Placement;
+class Rules;
+class WindowRules;
 
 class SystemTrayWindow
     {
@@ -80,11 +89,6 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         template< typename T1, typename T2 > void forEachClient( T1 procedure, T2 predicate );
         template< typename T > void forEachClient( T procedure );
 
-        Group* findGroup( Window leader ) const;
-        void addGroup( Group* group, allowed_t );
-        void removeGroup( Group* group, allowed_t );
-        Group* findClientLeaderGroup( const Client* c ) const;
-
         QRect clientArea( clientAreaOption, const QPoint& p, int desktop ) const;
         QRect clientArea( clientAreaOption, const Client* c ) const;
 
@@ -109,16 +113,17 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         // stealing prevention code.
         Client* mostRecentlyActivatedClient() const;
 
-        void setActiveClient( Client*, allowed_t );
         void activateClient( Client*, bool force = FALSE  );
         void requestFocus( Client* c, bool force = FALSE );
-        bool allowClientActivation( const Client* c, Time time = -1U, bool focus_in = false,
-            bool session_active = false );
+        void takeActivity( Client* c, int flags, bool handled ); // flags are ActivityFlags
+        void handleTakeActivity( Client* c, Time timestamp, int flags ); // flags are ActivityFlags
+        bool allowClientActivation( const Client* c, Time time = -1U, bool focus_in = false );
         void restoreFocus();
         void gotFocusIn( const Client* );
+        void setShouldGetFocus( Client* );
         bool fakeRequestedActivity( Client* c );
         void unfakeActivity( Client* c );
-        void activateNextClient( Client* c );
+        bool activateNextClient( Client* c );
         bool focusChangeEnabled() { return block_focus == 0; }
 
         void updateColormap();
@@ -132,10 +137,11 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void placeSmart( Client* c, const QRect& area );
 
         QPoint adjustClientPosition( Client* c, QPoint pos );
+        QRect adjustClientSize( Client* c, QRect moveResizeGeom, int mode );
         void raiseClient( Client* c );
         void lowerClient( Client* c );
-        void raiseClientRequest( Client* c );
-        void lowerClientRequest( Client* c );
+        void raiseClientRequest( Client* c, NET::RequestSource src, Time timestamp );
+        void lowerClientRequest( Client* c, NET::RequestSource src, Time timestamp );
         void restackClientUnderActive( Client* );
         void updateClientLayer( Client* c );
         void raiseOrLowerClient( Client * );
@@ -165,6 +171,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         Client* previousStaticClient(Client*) const;
         int nextDesktopFocusChain( int iDesktop ) const;
         int previousDesktopFocusChain( int iDesktop ) const;
+        void closeTabBox();
 
          /**
          * Returns the list of clients sorted in stacking order, with topmost client
@@ -177,14 +184,20 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         Client* topClientOnDesktop( int desktop, bool unconstrained = false ) const;
         Client* findDesktop( bool topmost, int desktop ) const;
         void sendClientToDesktop( Client* c, int desktop, bool dont_activate );
+        void windowToPreviousDesktop( Client* c );
+        void windowToNextDesktop( Client* c );
 
     // KDE4 remove me - and it's also in the DCOP interface :(
         void showWindowMenuAt( unsigned long id, int x, int y );
 
         /**
-         * Shows the menu operations menu for the client
-         * and makes it active if it's not already.
+	 * Shows the menu operations menu for the client and makes it active if
+	 * it's not already.
          */
+        void showWindowMenu( const QRect &pos, Client* cl );
+	/**
+	 * Backwards compatibility.
+	 */
         void showWindowMenu( int x, int y, Client* cl );
         void showWindowMenu( QPoint pos, Client* cl );
 
@@ -197,7 +210,9 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void storeSession( KConfig* config, SMSavePhase phase );
 
         SessionInfo* takeSessionInfo( Client* );
-
+        WindowRules findWindowRules( const Client*, bool );
+        void rulesUpdated();
+        void discardUsedWindowRules( Client* c, bool withdraw );
 
     // dcop interface
         void cascadeDesktop();
@@ -210,25 +225,51 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         QString desktopName( int desk ) const;
         void setDesktopLayout(int o, int x, int y);
+        void setShowingDesktop( bool showing );
+        void resetShowingDesktop( bool keep_hidden );
+        bool showingDesktop() const;
 
         bool isNotManaged( const QString& title );  // ### setter or getter ?
 
         void sendPingToWindow( Window w, Time timestamp ); // called from Client::pingWindow()
+        void sendTakeActivity( Client* c, Time timestamp, long flags ); // called from Client::takeActivity()
+        
+        void stopKompmgr();
+        bool kompmgrIsRunning();
+        void setOpacity(unsigned long winId, unsigned int opacityPercent);
+        void setShadowSize(unsigned long winId, unsigned int shadowSizePercent);
+        void setUnshadowed(unsigned long winId); // redundant, equals setShadowSize(inId, 0)
 
     // only called from Client::destroyClient() or Client::releaseWindow()
         void removeClient( Client*, allowed_t );
+        void setActiveClient( Client*, allowed_t );
+        Group* findGroup( Window leader ) const;
+        void addGroup( Group* group, allowed_t );
+        void removeGroup( Group* group, allowed_t );
+        Group* findClientLeaderGroup( const Client* c ) const;
 
-        bool checkStartupNotification( Window w, KStartupInfoData& data );
+        bool checkStartupNotification( Window w, KStartupInfoId& id, KStartupInfoData& data );
 
         void focusToNull(); // SELI public?
+        enum FocusChainChange { FocusChainMakeFirst, FocusChainMakeLast, FocusChainUpdate };
+        void updateFocusChains( Client* c, FocusChainChange change );
+        
+        bool forcedGlobalMouseGrab() const;
+        void clientShortcutUpdated( Client* c );
+        bool shortcutAvailable( const KShortcut& cut, Client* ignore = NULL ) const;
+        bool globalShortcutsDisabled() const;
+        void disableGlobalShortcuts( bool disable );
+        void disableGlobalShortcutsForClient( bool disable );
 
         void sessionSaveStarted();
         void sessionSaveDone();
         void setWasUserInteraction();
+        bool wasUserInteraction() const;
         bool sessionSaving() const;
 
         bool managingTopMenus() const;
         int topMenuHeight() const;
+        void updateCurrentTopMenu();
 
         int packPositionLeft( const Client* cl, int oldx, bool left_edge ) const;
         int packPositionRight( const Client* cl, int oldx, bool right_edge ) const;
@@ -237,6 +278,11 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         static QStringList configModules(bool controlCenter);
 
+        void cancelDelayFocus();
+        void requestDelayFocus( Client* );
+        
+        void toggleTopDockShadows(bool on);
+
     public slots:
         void refresh();
     // keybindings
@@ -289,8 +335,13 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         void slotWindowToNextDesktop();
         void slotWindowToPreviousDesktop();
+        void slotWindowToDesktopRight();
+        void slotWindowToDesktopLeft();
+        void slotWindowToDesktopUp();
+        void slotWindowToDesktopDown();
 
         void slotMouseEmulation();
+        void slotDisableGlobalShortcuts();
 
         void slotSettingsChanged( int category );
 
@@ -301,18 +352,37 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void slotGrabWindow();
         void slotGrabDesktop();
 
+        void slotSetupWindowShortcut();
+        void setupWindowShortcutDone( bool );
+
         void updateClientArea();
+        
+        // kompmgr, also dcop
+        void startKompmgr();
 
     private slots:
         void desktopPopupAboutToShow();
         void clientPopupAboutToShow();
-        void sendToDesktop( int );
+        void slotSendToDesktop( int );
         void clientPopupActivated( int );
         void configureWM();
         void desktopResized();
         void slotUpdateToolWindows();
         void lostTopMenuSelection();
         void lostTopMenuOwner();
+        void delayFocus();
+        void gotTemporaryRulesMessage( const QString& );
+        void cleanupTemporaryRules();
+        void writeWindowRules();
+        void kipcMessage( int id, int data );
+        // kompmgr
+        void setPopupClientOpacity(int v);
+        void resetClientOpacity();
+        void setTransButtonText(int value);
+        void unblockKompmgrRestart();
+        void restartKompmgr();
+        void handleKompmgrOutput( KProcess *proc, char *buffer, int buflen);
+        // end 
 
     protected:
         bool keyPressMouseEmulation( XKeyEvent& ev );
@@ -323,6 +393,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void initShortcuts();
         void readShortcuts();
         void initDesktopPopup();
+        void setupWindowShortcut( Client* c );
 
         bool startKDEWalkThroughWindows();
         bool startWalkThroughDesktops( int mode ); // TabBox::Mode::DesktopMode | DesktopListMode
@@ -335,16 +406,21 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void oneStepThroughDesktops( bool forward, int mode ); // TabBox::Mode::DesktopMode | DesktopListMode
         void oneStepThroughDesktops( bool forward );
         void oneStepThroughDesktopList( bool forward );
+        bool establishTabBoxGrab();
+        void removeTabBoxGrab();
+        int desktopToRight( int desktop ) const;
+        int desktopToLeft( int desktop ) const;
+        int desktopUp( int desktop ) const;
+        int desktopDown( int desktop ) const;
 
         void updateStackingOrder( bool propagate_new_clients = false );
         void propagateClients( bool propagate_new_clients ); // called only from updateStackingOrder
         ClientList constrainedStackingOrder();
         void raiseClientWithinApplication( Client* c );
         void lowerClientWithinApplication( Client* c );
-        bool allowFullClientRaising( const Client* c );
+        bool allowFullClientRaising( const Client* c, Time timestamp );
         bool keepTransientAbove( const Client* mainwindow, const Client* transient );
         void blockStackingUpdates( bool block );
-        void updateCurrentTopMenu();
         void addTopMenu( Client* c );
         void removeTopMenu( Client* c );
         void setupTopMenuHandling();
@@ -379,18 +455,20 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void tabBoxKeyRelease( const XKeyEvent& ev );
 
     // electric borders
+        void checkElectricBorders( bool force = false );
         void createBorderWindows();
         void destroyBorderWindows();
-        void electricBorder(XEvent * e);
+        bool electricBorder(XEvent * e);
         void raiseElectricBorders();
 
     // ------------------
     
         void helperDialog( const QString& message, const Client* c );
 
-        void calcDesktopLayout(int &x, int &y);
+        void calcDesktopLayout(int &x, int &y) const;
 
         QPopupMenu* clientPopup();
+        void closeActivePopup();
 
         void updateClientArea( bool force );
 
@@ -400,17 +478,19 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         int number_of_desktops;
         QMemArray<int> desktop_focus_chain;
 
-        Client* popup_client;
-
-        void loadSessionInfo();
+        QWidget* active_popup;
+        Client* active_popup_client;
 
         QWidget* desktop_widget;
 
+        void loadSessionInfo();
+        void loadWindowRules();
+        void editWindowRules( Client* c, bool whole_app );
+
         QPtrList<SessionInfo> session;
-        QPtrList<SessionInfo> fakeSession;
-        void loadFakeSessionInfo();
-        void storeFakeSessionInfo( Client* c );
-        void writeFakeSessionInfo();
+        QValueList<Rules*> rules;
+        KXMessages temporaryRulesMessages;
+        QTimer rulesUpdatedTimer;
         static const char* windowTypeToTxt( NET::WindowType type );
         static NET::WindowType txtToWindowType( const char* txt );
         static bool sessionInfoWindowTypeMatch( Client* c, SessionInfo* info );
@@ -419,15 +499,25 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         Client* last_active_client;
         Client* most_recently_raised; // used _only_ by raiseOrLowerClient()
         Client* movingClient;
+        Client* pending_take_activity;
+
+    // delay(ed) window focus timer and client
+        QTimer* delayFocusTimer;
+        Client* delayfocus_client;
 
         ClientList clients;
         ClientList desktops;
 
         ClientList unconstrained_stacking_order;
         ClientList stacking_order;
-        ClientList focus_chain;
+        QValueVector< ClientList > focus_chain;
+        ClientList global_focus_chain; // this one is only for things like tabbox's MRU
         ClientList should_get_focus; // last is most recent
         ClientList attention_chain;
+        
+        bool showing_desktop;
+        ClientList showing_desktop_clients;
+        int block_showing_desktop;
 
         GroupList groups;
 
@@ -458,6 +548,13 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         int desk_popup_index;
 
         KGlobalAccel *keys;
+        KGlobalAccel *client_keys;
+        ShortcutDialog* client_keys_dialog;
+        Client* client_keys_client;
+        KGlobalAccel *disable_shortcuts_keys;
+        bool global_shortcuts_disabled;
+        bool global_shortcuts_disabled_for_client;
+
         WId root;
 
         PluginMgr *mgr;
@@ -504,6 +601,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         Placement *initPositioning;
 
         QRect* workarea; //  array of workareas for virtual desktops
+        QRect** screenarea; // array of workareas per xinerama screen for all virtual desktops
 
         bool managing_topmenus;
         KSelectionOwner* topmenu_selection;
@@ -515,7 +613,22 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         int set_active_client_recursion;
         int block_stacking_updates; // when >0, stacking updates are temporarily disabled
         bool blocked_propagating_new_clients; // propagate also new clients after enabling stacking updates?
+        Window null_focus_window;
+        bool forced_global_mouse_grab;
         friend class StackingUpdatesBlocker;
+        
+        //kompmgr
+        QSlider *transSlider;
+        QPushButton *transButton;
+        // not used yet
+        /*Client* topDock;
+        int maximizedWindowCounter;
+        int topDockShadowSize;*/
+        //end
+        
+     signals:
+        void kompmgrStarted();
+        void kompmgrStopped();
     };
 
 // helper for Workspace::blockStackingUpdates() being called in pairs (true/false)
@@ -531,7 +644,7 @@ class StackingUpdatesBlocker
     };
 
 // NET WM Protocol handler class
-class RootInfo : public NETRootInfo2
+class RootInfo : public NETRootInfo4
     {
     private:
         typedef KWinInternal::Client Client;  // because of NET::Client
@@ -546,7 +659,9 @@ class RootInfo : public NETRootInfo2
         virtual void moveResize(Window w, int x_root, int y_root, unsigned long direction);
         virtual void moveResizeWindow(Window w, int flags, int x, int y, int width, int height );
         virtual void gotPing(Window w, Time timestamp);
-        virtual void restackWindow(Window w, Window above, int detail);
+        virtual void restackWindow(Window w, RequestSource source, Window above, int detail, Time timestamp);
+        virtual void gotTakeActivity(Window w, Time timestamp, long flags );
+        virtual void changeShowingDesktop( bool showing );
     private:
         Workspace* workspace;
     };
@@ -600,7 +715,12 @@ inline const ClientList& Workspace::stackingOrder() const
 
 inline void Workspace::showWindowMenu(QPoint pos, Client* cl)
     {
-    showWindowMenu(pos.x(), pos.y(), cl);
+    showWindowMenu(QRect(pos, pos), cl);
+    }
+
+inline void Workspace::showWindowMenu(int x, int y, Client* cl)
+    {
+    showWindowMenu(QRect(QPoint(x, y), QPoint(x, y)), cl);
     }
 
 inline
@@ -609,6 +729,12 @@ void Workspace::setWasUserInteraction()
     was_user_interaction = true;
     }
 
+inline
+bool Workspace::wasUserInteraction() const
+    {
+    return was_user_interaction;
+    }
+
 inline
 bool Workspace::managingTopMenus() const
     {
@@ -630,6 +756,21 @@ inline bool Workspace::sessionSaving() const
     return session_saving;
     }
 
+inline bool Workspace::forcedGlobalMouseGrab() const
+    {
+    return forced_global_mouse_grab;
+    }
+
+inline bool Workspace::showingDesktop() const
+    {
+    return showing_desktop;
+    }
+
+inline bool Workspace::globalShortcutsDisabled() const
+    {
+    return global_shortcuts_disabled || global_shortcuts_disabled_for_client;
+    }
+
 template< typename T >
 inline Client* Workspace::findClient( T predicate )
     {
