@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #ifndef __konq_operations_h__
@@ -49,14 +49,25 @@ public:
 
     enum { TRASH, DEL, SHRED, COPY, MOVE, LINK, EMPTYTRASH, STAT, MKDIR, RESTORE, UNKNOWN };
     /**
+     * Delete the @p selectedURLs if possible.
+     *
+     * @param parent parent widget (for error dialog box if any)
      * @param method should be TRASH, DEL or SHRED
+     * @param selectedURLs the URLs to be deleted
      */
     static void del( QWidget * parent, int method, const KURL::List & selectedURLs );
 
     /**
+     * Copy the @p selectedURLs to the destination @p destURL.
+     *
+     * @param parent parent widget (for error dialog box if any)
      * @param method should be COPY, MOVE or LINK
+     * @param selectedURLs the URLs to copy
+     * @param destURL destination of the copy
+     *
+     * @todo document restrictions on the kind of destination
      */
-    static void copy( QWidget * parent, int method, const KURL::List & selectedURLs, const KURL& destUrl );
+    static void copy( QWidget * parent, int method, const KURL::List & selectedURLs, const KURL& destURL );
     /**
      * Drop
      * @param destItem destination KFileItem for the drop (background or item)
@@ -143,12 +154,12 @@ protected:
     // internal, for doDrop
     void setDropInfo( DropInfo * info ) { m_info = info; }
 
-    struct KIOPasteInfo
+    struct KIOPasteInfo // KDE4: remove and use DropInfo instead or a QPoint member
     {
-        QByteArray data;
-        KURL destURL;
+        QByteArray data;  // unused
+        KURL destURL;     // unused
         QPoint mousePos;
-        QString dialogText;
+        QString dialogText; // unused
     };
     void setPasteInfo( KIOPasteInfo * info ) { m_pasteInfo = info; }
 
@@ -158,7 +169,6 @@ protected slots:
     void slotResult( KIO::Job * job );
     void slotStatResult( KIO::Job * job );
     void asyncDrop( const KFileItem * item );
-    void slotKIOPaste();
     void doFileCopy();
 
 private:
