@@ -588,7 +588,6 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
    : DCOPObject(), QObject( parent, name ) 
 {
     Q_UNUSED(embed);
-   _firstResize = true;
    _visible = false;
    _npp = privateData;
    _npp->ndata = this;
@@ -895,7 +894,11 @@ void NSPluginInstance::streamFinished( NSPluginStreamBase* strm )
 {
    kdDebug(1431) << "-> NSPluginInstance::streamFinished" << endl;
    emitStatus( QString::null );
-   _streams.remove(strm); // deletes automatically
+   _streams.setAutoDelete(false); // Don't delete it yet!!  we get called from
+                                  // its slot!
+   _streams.remove(strm);
+   _streams.setAutoDelete(true);
+   strm->deleteLater();
    _timer->start( 100, true );
 }
 
@@ -984,10 +987,7 @@ void NSPluginInstance::resizePlugin(int w, int h)
 
    resizeWidgets(XtWindow(_form), _width, _height);
 
-   if (_firstResize) {
-      _firstResize = false;
-      setWindow();
-   }
+   setWindow();
 
    kdDebug(1431) << "<- NSPluginInstance::resizePlugin" << endl;
 }
@@ -1598,6 +1598,12 @@ bool NSPluginStreamBase::create( const QString& url, const QString& mimeType, vo
     return true;
 }
 
+void NSPluginStreamBase::updateURL( const KURL& newURL )
+{
+    _url = newURL;
+    free(const_cast<char*>(_stream->url));
+    _stream->url = strdup(_url.url().ascii());
+}
 
 int NSPluginStreamBase::process( const QByteArray &data, int start )
 {
@@ -1820,6 +1826,8 @@ bool NSPluginStream::get( const QString& url, const QString& mimeType,
                 SLOT(totalSize(KIO::Job *, KIO::filesize_t)));
         connect(_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
                 SLOT(mimetype(KIO::Job *, const QString &)));
+        connect(_job, SIGNAL(redirection(KIO::Job *, const KURL&)),
+                SLOT(redirection(KIO::Job *, const KURL&)));
     }
 
     return false;
@@ -1843,6 +1851,8 @@ bool NSPluginStream::post( const QString& url, const QByteArray& data,
                 SLOT(totalSize(KIO::Job *, KIO::filesize_t)));
         connect(_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
                 SLOT(mimetype(KIO::Job *, const QString &)));
+        connect(_job, SIGNAL(redirection(KIO::Job *, const KURL&)),
+                SLOT(redirection(KIO::Job *, const KURL&)));
     }
 
     return false;
@@ -1859,6 +1869,11 @@ void NSPluginStream::data(KIO::Job * job, const QByteArray &data)
     }
 }
 
+void NSPluginStream::redirection(KIO::Job * /*job*/, const KURL& url)
+{
+    updateURL( url );
+}
+
 void NSPluginStream::totalSize(KIO::Job * job, KIO::filesize_t size)
 {
     kdDebug(1431) << "NSPluginStream::totalSize - job=" << (void*)job << " size=" << KIO::number(size) << endl;
